/*	File: Interval.hpp

	A C++ wrapper for mpfi library.

	Uses: mpfi library, available from 
		https://gforge.inria.fr/projects/mpfi/

	Author: Jordi-Lluis Figueras, Jordi Villadelprat, Warwick Tucker
	Latest edit: Thu June 21 10:25:00 CET 2012
*/

#include<mpfi.h>
#include<mpfi_io.h>

int MY_PREC = 256;

class INTERVAL
{
public:
	mpfi_t x;
	INTERVAL();
	INTERVAL(double a);
	INTERVAL(double a, double b);
	~INTERVAL();

	void setfull(double a, double b);
	void setfull(mpfr_t a, mpfr_t b);
	int position(double a);
	int position(mpfr_t a);
	int is_inside(double a);

	double left();
	double right();

	INTERVAL& operator = (const INTERVAL& y);
	INTERVAL& operator = (const double& y);
	INTERVAL& operator = (const mpfr_t& y);
};

inline INTERVAL operator+ (const INTERVAL&, const INTERVAL&);
inline INTERVAL operator+ (const double&, const INTERVAL&);
inline INTERVAL operator+ (const INTERVAL&, const double&);
inline INTERVAL operator+ (const mpfr_t&, const INTERVAL&);
inline INTERVAL operator+ (const INTERVAL&, const mpfr_t&);

inline INTERVAL operator- (const INTERVAL&, const INTERVAL&);
inline INTERVAL operator- (const double&, const INTERVAL&);
inline INTERVAL operator- (const INTERVAL&, const double&);
inline INTERVAL operator- (const mpfr_t&, const INTERVAL&);
inline INTERVAL operator- (const INTERVAL&, const mpfr_t&);

inline INTERVAL operator* (const INTERVAL&, const INTERVAL&);
inline INTERVAL operator* (const double&, const INTERVAL&);
inline INTERVAL operator* (const INTERVAL&, const double&);
inline INTERVAL operator* (const int&, const INTERVAL&);
inline INTERVAL operator* (const INTERVAL&, const int&);

inline INTERVAL operator/ (const INTERVAL&, const INTERVAL&);
inline INTERVAL operator/ (const double&, const INTERVAL&);
inline INTERVAL operator/ (const INTERVAL&, const double&);

inline INTERVAL sin(const INTERVAL& P);
inline INTERVAL cos(const INTERVAL& P);
inline INTERVAL exp(const INTERVAL& P);
inline INTERVAL log(const INTERVAL& P);
inline INTERVAL sqrt(const INTERVAL& P);
inline INTERVAL pow(const INTERVAL& P, int N);

INTERVAL str_to_INTERVAL(char *s);

bool check_invertibility(INTERVAL **A, int dim);

/*-------------------------------------------------------------------------------------------------------------------------------------------*/

INTERVAL::INTERVAL () 
{
	mpfi_init_set_d(x, 0.);
}

INTERVAL::INTERVAL (double a) 
{
	mpfi_init_set_d(x, a);
}

INTERVAL::INTERVAL (double a, double b) 
{
	mpfi_init(x);
	mpfi_interv_d(x, a, b);
}


INTERVAL::~INTERVAL()
{
	mpfi_clear(x);
}

void INTERVAL::setfull(double a, double b)
{
	mpfi_interv_d(x, a, b);

	return ;
}

void INTERVAL::setfull(mpfr_t a, mpfr_t b)
{
	mpfi_interv_fr(x, a, b);

	return ;
}

int INTERVAL::position(double a)
{
	mpfr_t left, right;
	int flag = 0, retl, retr;

	mpfr_init(left);
	mpfr_init(right);

	mpfi_get_left(left, x);
	mpfi_get_right(right, x);

	retl = mpfr_cmp_d(left, a);
	retr = mpfr_cmp_d(right, a);
	
	if(retl > 0)
	{
		flag = -1;
	}
	if(retr < 0)
	{
		flag = 1;
	}

	mpfr_clear(left);
	mpfr_clear(right);

	return flag;
}

int INTERVAL::position(mpfr_t a)
{
	mpfr_t left, right;
	int flag = 0, retl, retr;

	mpfr_init(left);
	mpfr_init(right);

	mpfi_get_left(left, x);
	mpfi_get_right(right, x);

	retl = mpfr_cmp(left, a);
	retr = mpfr_cmp(right, a);
	
	if(retl > 0)
	{
		flag = -1;
	}
	if(retr < 0)
	{
		flag = 1;
	}

	mpfr_clear(left);
	mpfr_clear(right);

	return flag;
}

//Returns 0 if a is not inside x, 1 otherwise
int INTERVAL::is_inside(double a)
{
	int flag;

	flag = mpfi_is_inside_d(a, x);

	return flag;
}

double INTERVAL::left(void )
{	
	mpfr_t left;
	double res;

	mpfr_init(left);

	mpfi_get_left(left, x);
	res = mpfr_get_d(left, GMP_RNDN);

	mpfr_clear(left);

	return res;
}

double INTERVAL::right(void )
{	
	mpfr_t right;
	double res;

	mpfr_init(right);

	mpfi_get_right(right, x);
	res = mpfr_get_d(right, GMP_RNDN);

	mpfr_clear(right);

	return res;
}

INTERVAL& INTERVAL::operator = (const INTERVAL& P)
{
	if(this != &P)
	{
		mpfi_set(x, P.x);
	}
	return *this;
}

INTERVAL& INTERVAL::operator = (const double& P)
{
	mpfi_set_d(x, P);
	return *this;
}

INTERVAL& INTERVAL::operator = (const mpfr_t& P)
{
	mpfi_set_fr(x, P);
	return *this;
}

inline INTERVAL operator+ (const INTERVAL& P, const INTERVAL& Q)
{
	INTERVAL res;

	mpfi_add(res.x, P.x, Q.x);

	return res;
}

inline INTERVAL operator+ (const double& P, const INTERVAL& Q)
{
	INTERVAL res;

	mpfi_add_d(res.x, Q.x, P);

	return res;
}

inline INTERVAL operator+ (const INTERVAL& Q, const double& P)
{
	INTERVAL res;

	mpfi_add_d(res.x, Q.x, P);

	return res;
}

inline INTERVAL operator+ (const mpfr_t& P, const INTERVAL& Q)
{
	INTERVAL res;

	mpfi_add_fr(res.x, Q.x, P);

	return res;
}

inline INTERVAL operator+ (const INTERVAL& Q, const mpfr_t& P)
{
	INTERVAL res;

	mpfi_add_fr(res.x, Q.x, P);

	return res;
}

inline INTERVAL operator- (const INTERVAL& P, const INTERVAL& Q)
{
	INTERVAL res;

	mpfi_sub(res.x, P.x, Q.x);

	return res;
}

inline INTERVAL operator- (const INTERVAL& P, const double& Q)
{
	INTERVAL res;

	mpfi_sub_d(res.x, P.x, Q);

	return res;
}

inline INTERVAL operator- (const double& P, const INTERVAL& Q)
{
	INTERVAL res;

	mpfi_d_sub(res.x, P, Q.x);

	return res;
}

inline INTERVAL operator- (const INTERVAL& P, const mpfr_t& Q)
{
	INTERVAL res;

	mpfi_sub_fr(res.x, P.x, Q);

	return res;
}

inline INTERVAL operator- (const mpfr_t& P, const INTERVAL& Q)
{
	INTERVAL res;

	mpfi_fr_sub(res.x, P, Q.x);

	return res;
}

inline INTERVAL operator* (const INTERVAL& P, const INTERVAL& Q)
{
	INTERVAL res;

	mpfi_mul(res.x, P.x, Q.x);

	return res;
}

inline INTERVAL operator* (const INTERVAL& P, const double& Q)
{
	INTERVAL res;

	mpfi_mul_d(res.x, P.x, Q);

	return res;
}
inline INTERVAL operator* (const double& P, const INTERVAL& Q)
{
	INTERVAL res;

	mpfi_mul_d(res.x, Q.x, P);

	return res;
}

inline INTERVAL operator* (const INTERVAL& P, const int& Q)
{
	INTERVAL res;

	mpfi_mul_si(res.x, P.x, Q);

	return res;
}
inline INTERVAL operator* (const int& P, const INTERVAL& Q)
{
	INTERVAL res;

	mpfi_mul_si(res.x, Q.x, P);

	return res;
}

inline INTERVAL operator/ (const INTERVAL& P, const INTERVAL& Q)
{
	INTERVAL res;

	mpfi_div(res.x, P.x, Q.x);

	return res;
}

inline INTERVAL operator/ (const INTERVAL& P, const double& Q)
{
	INTERVAL res;

	mpfi_div_d(res.x, P.x, Q);

	return res;
}
inline INTERVAL operator/ (const double& P, const INTERVAL& Q)
{
	INTERVAL res;

	mpfi_div_d(res.x, Q.x, P);

	return res;
}

inline INTERVAL cos(const INTERVAL& P)
{
	INTERVAL res;

	mpfi_cos(res.x, P.x);

	return res;
}

inline INTERVAL sin(const INTERVAL& P)
{
	INTERVAL res;

	mpfi_sin(res.x, P.x);

	return res;
}

inline INTERVAL exp(const INTERVAL& P)
{
	INTERVAL res;

	mpfi_exp(res.x, P.x);

	return res;
}

inline INTERVAL log(const INTERVAL& P)
{
	INTERVAL res;

	mpfi_log(res.x, P.x);

	return res;
}

inline INTERVAL sqrt(const INTERVAL& P)
{
	INTERVAL res;

	mpfi_sqrt(res.x, P.x);

	return res;
}
inline INTERVAL pow(const INTERVAL& P, int N)
{
	INTERVAL res;
	int i;

	res = P;
	for(i = 1; i < N; i++)
	{
		res = res*P;
	}

	return res;
}

INTERVAL str_to_INTERVAL(char *s)
{
	INTERVAL res;

	mpfi_set_str(res.x, s, 10);

	return res;
}

bool check_invertibility(INTERVAL **A, int dim)
{
	bool res = true;
	int i, j, k, flag = 0, ipivotpos = 0, jpivotpos = 0;
	INTERVAL pivot, pivotline, pivotabs, *aux, aux0, aux1;
	mpfr_t left0, left1;

	mpfr_init(left0);
	mpfr_init(left1);

	aux = new INTERVAL[dim];

	for(i = 0; i < dim; i++)
	{
	/*	for(j = 0; j < dim; j++)
		{
			for(k = 0; k < dim; k++)
			{
				mpfi_out_str(stdout, 10, 8, A[j][k].x);
			}
			printf("\n");
		}
		printf("\n");*/
		flag = 0;
		for(j = i; j < dim; j++)
		{
			for(k = j; k < dim; k++)
			{
				mpfi_abs(aux0.x, A[j][k].x);
				if(mpfi_has_zero(aux0.x) > 0)
				{
					continue;
				}else
				{
					if(flag == 0)
					{
						ipivotpos = j;
						jpivotpos = k;
						pivotabs = aux0;
						flag = 1;
					}else
					{
						mpfi_mig(left0, aux0.x);
						mpfi_mig(left1, pivotabs.x);
						if(mpfr_cmp(left0, left1) > 0)
						{
							pivotabs = aux0;
							ipivotpos = j;
							jpivotpos = k;
						}
					}
				}
			}
		}
		if(flag == 0)
		{
			res = false;
			break;
		}
		if(ipivotpos != i)
		{
			for(j = 0; j < dim; j++)
			{
				aux[j] = A[ipivotpos][j];
				A[ipivotpos][j] = A[i][j];
				A[i][j] = aux[j];
			}
		}
		if(jpivotpos != i)
		{
			for(j = 0; j < dim; j++)
			{
				aux[j] = A[j][jpivotpos];
				A[j][jpivotpos] = A[j][i];
				A[j][i] = aux[j];
			}
		}
		pivot = A[i][i];
		for(j = i+1; j < dim; j++)
		{
			pivotline = A[j][i]/pivot;
			for(k = i; k < dim; k++)
			{
				A[j][k] = A[j][k]-A[i][k]*pivotline;
			}
		}
	}

	delete []aux;

	return res;
}
