/*	File: involution.cc

	Routines used for rigorous enclosures of the involution. More concretely, 
	given an interval $X=[a, b]\subset (-\frac12, 0)$, 
	\verb$int obtain_involution(const INTERVAL& X, INTERVAL& Z)$
	computes a rigorous enclosure of the involution that satisfies 
	$A(Z)=A(X)$.

	Uses: Interval.hpp

	Author: Jordi-Lluis Figueras, Jordi Villadelprat, Warwick Tucker
	Latest edit: Thu June 21 10:25:00 CET 2012
*/

double f_involution(double z, double x);
INTERVAL f_involution_interval(const INTERVAL& z, const INTERVAL& x);
int obtain_involution(const INTERVAL& X, INTERVAL& Z);

double f_involution(double z, double x)
{
	return (2.*z+log(2.*x+1.)-2.*x-log(2.*z+1.));
}

INTERVAL f_involution_interval(const INTERVAL& z, const INTERVAL& x)
{
	INTERVAL res;

	res = (2*z+log(2*x+1)-2*x-log(2*z+1));

	return res;
}

int obtain_involution(const INTERVAL& X0, INTERVAL& res)
{
	int flag = 1, flag2;
	mpfr_t mid, aux, aux2;
	INTERVAL X, Xmin, Xmax;
	INTERVAL Zmin, Zmax, Zinit, Z, Zlow, Zmid, Zhigh, FZlow, FZmid, FZhigh;
	INTERVAL Xcmin, Xcmax, Zcmin, Zcmax, FZcmin, FZcmax;

	mpfr_init(mid);
	mpfr_init(aux);
	mpfr_init(aux2);

	X = X0;
	mpfi_get_left(aux, X.x);
	mpfi_set_fr(Xmin.x, aux); 
	mpfi_get_right(aux, X.x);
	mpfi_set_fr(Xmax.x, aux); 

	Zmax = 2.*log(1.+sqrt((9.*exp(2.*Xmin))/(6.*Xmin+3.)-3.)/3.);
	Zmin = Xmax/2.-log(2.*Xmax+1.)/4.+sqrt((2.*Xmax-log(2.*Xmax+1.))*(2.*Xmax-log(2.*Xmax+1.))+16.*Xmax-8.*log(2.*Xmax+1.))/4.;

	Zinit = Zmax;
	mpfi_put(Zinit.x, Zmin.x);
	Z = Zinit;

	mpfi_mid(mid, Z.x);
	mpfi_diam_abs(aux2, Z.x);

	mpfi_get_left(aux, Z.x);
	mpfi_set_fr(Zlow.x, aux); 
	mpfi_get_right(aux, Z.x);
	mpfi_set_fr(Zhigh.x, aux); 
	mpfi_set_fr(Zmid.x, mid);

	FZlow = f_involution_interval(Zlow, Xmin);
	FZhigh = f_involution_interval(Zhigh, Xmin);
	if(mpfi_is_strictly_pos(FZhigh.x) <= 0 || mpfi_is_strictly_neg(FZlow.x) <= 0)
	{
		flag = -1;
	}
	while(1)
	{
		FZmid = f_involution_interval(Zmid, Xmin);
		flag2 = 0;
		if(mpfi_is_strictly_pos(FZmid.x) > 0)
		{
			Zhigh = Zmid;
			flag2 = 1;
		}else
		{
			if(mpfi_is_strictly_neg(FZmid.x) > 0)
			{
				Zlow = Zmid;
				flag2 = 1;
			}
		}
		if(flag2 == 0)
		{
			Zmax = Zlow;
			mpfi_put(Zmax.x, Zhigh.x);
			break;
		}
		Z = Zlow;
		mpfi_put(Z.x, Zhigh.x);
		mpfi_diam_abs(aux, Z.x);
		if(mpfr_cmp(aux, aux2) == 0)
		{
			Zmax = Zlow;
			mpfi_put(Zmax.x, Zhigh.x);
			break;
		}else
		{
			mpfr_set(aux2, aux, GMP_RNDN);	
		}

		mpfi_mid(mid, Z.x);
		mpfi_set_fr(Zmid.x, mid);
		mpfi_get_left(aux, Z.x);
		mpfi_set_fr(Zlow.x, aux); 
		mpfi_get_right(aux, Z.x);
		mpfi_set_fr(Zhigh.x, aux); 
	}
	
	Zinit = Zmax;
	mpfi_put(Zinit.x, Zmin.x);
	Z = Zinit;
	mpfi_diam_abs(aux2, Z.x);

	mpfi_mid(mid, Z.x);

	mpfi_get_left(aux, Z.x);
	mpfi_set_fr(Zlow.x, aux); 
	mpfi_get_right(aux, Z.x);
	mpfi_set_fr(Zhigh.x, aux); 
	mpfi_set_fr(Zmid.x, mid);

	FZlow = f_involution_interval(Zlow, Xmax);
	FZhigh = f_involution_interval(Zhigh, Xmax);
	if(mpfi_is_strictly_pos(FZhigh.x) <= 0 || mpfi_is_strictly_neg(FZlow.x) <= 0)
	{
		flag = -1;
	}
	while(1)
	{
		FZmid = f_involution_interval(Zmid, Xmax);
		flag2 = 0;
		if(mpfi_is_strictly_pos(FZmid.x) > 0)
		{
			Zhigh = Zmid;
			flag2 = 1;
		}else
		{
			if(mpfi_is_strictly_neg(FZmid.x) > 0)
			{
				Zlow = Zmid;
				flag2 = 1;
			}
		}
		if(flag2 == 0)
		{
			Zmin = Zlow;
			mpfi_put(Zmin.x, Zhigh.x);
			break;
		}
		Z = Zlow;
		mpfi_put(Z.x, Zhigh.x);
		mpfi_diam_abs(aux, Z.x);
		if(mpfr_cmp(aux, aux2) == 0)
		{
			Zmin = Zlow;
			mpfi_put(Zmin.x, Zhigh.x);
			break;
		}else
		{
			mpfr_set(aux2, aux, GMP_RNDN);	
		}

		mpfi_mid(mid, Z.x);
		mpfi_get_left(aux, Z.x);
		mpfi_set_fr(Zlow.x, aux); 
		mpfi_get_right(aux, Z.x);
		mpfi_set_fr(Zhigh.x, aux); 
		mpfi_set_fr(Zmid.x, mid);
	}

	res = Zmax;
	mpfi_put(res.x, Zmax.x);
	mpfi_put(res.x, Zmin.x);

	//Checking it works

	mpfi_get_left(aux, X.x);
	mpfi_set_fr(Xcmin.x, aux);
	mpfi_get_right(aux, X.x);
	mpfi_set_fr(Xcmax.x, aux);
	mpfi_get_left(aux, res.x);
	mpfi_set_fr(Zcmin.x, aux);
	mpfi_get_right(aux, res.x);
	mpfi_set_fr(Zcmax.x, aux);
	FZcmin = f_involution_interval(Zcmax, Xcmin);
	FZcmax = f_involution_interval(Zcmin, Xcmax);

	if(mpfi_is_strictly_pos(FZcmin.x) >= 0 && mpfi_is_strictly_neg(FZcmax.x) >= 0)
	{
		flag = 1;
	}else
	{
		flag = 0;
	}

	mpfr_clear(mid);
	mpfr_clear(aux);
	mpfr_clear(aux2);

	return flag;
}

