/*	File: fujiwara.cc

	This code checks the Fujiwara's bounds in Lemma 4.12.

	Uses: Interval.hpp, involution.cc

	Author: Jordi-Lluis Figueras, Jordi Villadelprat, Warwick Tucker
	Latest edit: Thu June 21 10:25:00 CET 2012
*/

#include<stdio.h>
#include<time.h>
#include<math.h>
#include<stdlib.h>
#include "Interval.hpp"
#include "involution.cc"

#define DIGITSPRINT 40

int fujiwara(const INTERVAL& x);
void polynomial_determinant(INTERVAL *coef, const INTERVAL& x);
void polynomial_determinant_tail(INTERVAL *coef, const INTERVAL& x);

int main(int argc, char* argv[])
{
	if(argc < 4)
	{
		printf("Usage:\n\t[1]precision of the rigorous computations\n\t[2]Lower bound x (-0.5+$2)\n\t[3]Upper bound x (-0.5+$3)\n\n");
		printf("Execution:\n./fujiwara 512 0. 1.e-20\n\n");
		exit(1);
	}
	
	int prec = atoi(argv[1]);
	mpfr_t xmin, xmax, x, deltax, tolstep, aux;

	mpfr_set_default_prec(prec);

	INTERVAL X, Xaux, Xaux2, GoodXleft, GoodXright, z;

	mpfr_init(xmin);
	mpfr_init(xmax);
	mpfr_init(x);
	mpfr_init(deltax);
	mpfr_init(tolstep);
	mpfr_init(aux);

	mpfr_set_str(tolstep, "1.e-75", 10, GMP_RNDN);
	mpfr_set_str(xmin, argv[2], 10, GMP_RNDN);
	mpfr_add_d(xmin, xmin, -0.5, GMP_RNDN);
	mpfr_set_str(xmax, argv[3], 10, GMP_RNDN);
	mpfr_add_d(xmax, xmax, -0.5, GMP_RNDN);

	mpfr_sub(deltax, xmin, xmax, GMP_RNDN);
	mpfr_set_str(aux, "1.e4", 10, GMP_RNDN);
	mpfr_div(deltax, deltax, aux, GMP_RNDN);

	mpfr_sub(x, xmin, deltax, GMP_RNDN);
	X.setfull(x, xmin);

	GoodXright = X;

	printf("x = ");
	mpfi_out_str(stdout, 10, DIGITSPRINT, X.x);
	printf("\n");
	
	fujiwara(X);

	return 0;
}

int fujiwara(const INTERVAL& x)
{
	mpfr_t rightextrem;
	INTERVAL xextrem, z, z1, aux, supz, upperbound, aux2;
	int flag = 0;
	int i;

	upperbound = 0.;
	INTERVAL *coef;
	
	coef = new INTERVAL[22];

	mpfr_init(rightextrem);

	polynomial_determinant(coef, x);
	polynomial_determinant_tail(coef, x);

	mpfi_get_right(rightextrem, x.x);

	mpfi_set_fr(xextrem.x, rightextrem);
	
	flag = obtain_involution(xextrem, z1);
	z = sqrt(2.*z1+1.);

	supz = sqrt(sqrt(2.*exp(2.*x)-(4.*x+2.))+0.5*sqrt(2.*x+1.));

	coef[18] = coef[18]+coef[19]*sqrt(sqrt(2.*x+1.))*supz;
	
	coef[18] = coef[18]+coef[20]*supz*supz;

	coef[18] = coef[18]+coef[21]/str_to_INTERVAL((char*)"8.");
	
	if(mpfi_has_zero(coef[18].x) <= 0)
	{
		for(i = 17; i > -1; i--)
		{
			coef[i] = coef[i]/coef[18];
			mpfi_abs(aux.x, coef[i].x);
			if(i != 0)
			{
				aux = exp(log(aux)/(18-i));
			}else
			{
				aux = exp(log(0.5*aux)/(18-i));
			}
			if(mpfi_cmp(upperbound.x, aux.x) < 0)
			{
				upperbound = aux;
			}
		}
		upperbound = 2*upperbound;
		printf("upperbound = ");
		mpfi_out_str(stdout, 10, DIGITSPRINT, upperbound.x);
		printf("\n");
		printf("z = ");
		mpfi_out_str(stdout, 10, DIGITSPRINT, z.x);
		printf("\n");
		if(mpfi_cmp(z.x , upperbound.x) > 0)
		{
			printf("Proved!\n");
			flag = 1;
		}else
		{
			printf("Not proved!\n");
			flag = -1;
		}
	}else
	{
		flag = -1;
	}


	mpfr_clear(rightextrem);

	delete[] coef;

	return flag;
}

void polynomial_determinant(INTERVAL *coef, const INTERVAL& x)
{
	INTERVAL sqrt2xplus1;
	INTERVAL y;
	
	sqrt2xplus1 = sqrt(2*x+1);
	y= exp(x);
coef[0] =   pow( (2 * x + 1),  6) * (-6 + (24 + (-36 + (24 - 6 * y * y) * y * y) * y * y) * y * y + (-16 + (16 + (48 + (-80 + 32 * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-158") + (str_to_INTERVAL((char*)"360") + (str_to_INTERVAL((char*)"-276") + (str_to_INTERVAL((char*)"104") - 30 * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"225") + (str_to_INTERVAL((char*)"-840") + (str_to_INTERVAL((char*)"822") + (str_to_INTERVAL((char*)"-216") + 9 * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"600") + (str_to_INTERVAL((char*)"440") + (str_to_INTERVAL((char*)"-1224") + 88 * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"1328") + (str_to_INTERVAL((char*)"-192") + str_to_INTERVAL((char*)"1056") * y * y) * y * y + (str_to_INTERVAL((char*)"1152") * y * y + str_to_INTERVAL((char*)"768") + str_to_INTERVAL((char*)"1024") * x) * x) * x) * x) * x) * x) * x);
coef[1] = 60 * sqrt2xplus1* pow( x,  3) *  pow( (2 * x + 1),  5) * (-15 + (30 - 15 * y * y) * y * y + (-25 + (-10 + 35 * y * y) * y * y + (-32 - 28 *  pow( y,  4) + (-7 + (-14 - 7 * y * y) * y * y + (2 + (-4 + 30 * y * y) * y * y + (48 * y * y + 8 + 64 * x) * x) * x) * x) * x) * x);
coef[2] = 0.-10 *   pow( (2 * x + 1),  6) * (-6 + (24 + (-36 + (24 - 6 * y * y) * y * y) * y * y) * y * y + (-16 + (16 + (48 + (-80 + 32 * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-158") + (str_to_INTERVAL((char*)"360") + (str_to_INTERVAL((char*)"-276") + (str_to_INTERVAL((char*)"104") - 30 * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"225") + (str_to_INTERVAL((char*)"-840") + (str_to_INTERVAL((char*)"822") + (str_to_INTERVAL((char*)"-216") + 9 * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"600") + (str_to_INTERVAL((char*)"440") + (str_to_INTERVAL((char*)"-1224") + 88 * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"1328") + (str_to_INTERVAL((char*)"-192") + str_to_INTERVAL((char*)"1056") * y * y) * y * y + (str_to_INTERVAL((char*)"1152") * y * y + str_to_INTERVAL((char*)"768") + str_to_INTERVAL((char*)"1024") * x) * x) * x) * x) * x) * x) * x);
coef[3] = 0.-40 * sqrt2xplus1* pow( x,  3) *  pow( (2 * x + 1),  4) * (str_to_INTERVAL((char*)"-210") + (str_to_INTERVAL((char*)"405") + (str_to_INTERVAL((char*)"-180") - 15 * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-770") + (str_to_INTERVAL((char*)"675") + (60 + 35 * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-1190") + (str_to_INTERVAL((char*)"-216") + (str_to_INTERVAL((char*)"522") - 16 * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-1092") + (str_to_INTERVAL((char*)"-171") + (str_to_INTERVAL((char*)"-786") - 51 * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-322") + (str_to_INTERVAL((char*)"-486") + (str_to_INTERVAL((char*)"234") + 90 * y * y) * y * y) * y * y + (56 + (str_to_INTERVAL((char*)"576") + str_to_INTERVAL((char*)"648") * y * y) * y * y + (str_to_INTERVAL((char*)"1344") + str_to_INTERVAL((char*)"864") * y * y) * x) * x) * x) * x) * x) * x);
coef[4] = 0.-15 *   pow( (2 * x + 1),  4) * (18 + (-72 + (str_to_INTERVAL((char*)"108") + (-72 + 18 * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"120") + (str_to_INTERVAL((char*)"-336") + (str_to_INTERVAL((char*)"288") + (-48 - 24 * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"738") + (str_to_INTERVAL((char*)"-1560") + (str_to_INTERVAL((char*)"684") + (str_to_INTERVAL((char*)"360") - str_to_INTERVAL((char*)"222") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"1413") + (str_to_INTERVAL((char*)"-1992") + (str_to_INTERVAL((char*)"270") + (str_to_INTERVAL((char*)"360") - 51 * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-2604") + (str_to_INTERVAL((char*)"4440") + (str_to_INTERVAL((char*)"-2880") + (str_to_INTERVAL((char*)"1080") + str_to_INTERVAL((char*)"252") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-13884") + (str_to_INTERVAL((char*)"5376") + (str_to_INTERVAL((char*)"1656") + (str_to_INTERVAL((char*)"1536") - str_to_INTERVAL((char*)"108") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-25440") + (str_to_INTERVAL((char*)"-6432") + (str_to_INTERVAL((char*)"2016") - str_to_INTERVAL((char*)"1056") * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-28224") + (str_to_INTERVAL((char*)"-11520") - str_to_INTERVAL((char*)"12672") * y * y) * y * y + (str_to_INTERVAL((char*)"-13824") * y * y - str_to_INTERVAL((char*)"21504") + (str_to_INTERVAL((char*)"-12288") + str_to_INTERVAL((char*)"4480") * x) * x) * x) * x) * x) * x) * x) * x) * x) * x);
coef[5] = 0.-20 * sqrt2xplus1* pow( x,  3) *  pow( (2 * x + 1),  3) * (str_to_INTERVAL((char*)"1770") + (str_to_INTERVAL((char*)"-3300") + (str_to_INTERVAL((char*)"1305") + (str_to_INTERVAL((char*)"210") + 15 * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"10030") + (str_to_INTERVAL((char*)"-12100") + (str_to_INTERVAL((char*)"2175") + (-70 - 35 * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"23244") + (str_to_INTERVAL((char*)"-9328") + (str_to_INTERVAL((char*)"-5172") + (str_to_INTERVAL((char*)"-648") + 4 * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"30278") + (str_to_INTERVAL((char*)"4708") + (str_to_INTERVAL((char*)"-2043") + (str_to_INTERVAL((char*)"982") + 95 * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"23324") + (str_to_INTERVAL((char*)"6832") + (str_to_INTERVAL((char*)"10890") + (-84 - str_to_INTERVAL((char*)"150") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"8052") + (str_to_INTERVAL((char*)"3400") + (str_to_INTERVAL((char*)"-9108") - str_to_INTERVAL((char*)"1440") * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-8744") + (str_to_INTERVAL((char*)"-17968") - str_to_INTERVAL((char*)"7608") * y * y) * y * y + (str_to_INTERVAL((char*)"-3264") * y * y - str_to_INTERVAL((char*)"17184") + str_to_INTERVAL((char*)"3840") * x) * x) * x) * x) * x) * x) * x) * x);
coef[6] = str_to_INTERVAL((char*)"120") *   pow( (2 * x + 1),  3) * (6 + (-24 + (36 + (-24 + 6 * y * y) * y * y) * y * y) * y * y + (52 + (str_to_INTERVAL((char*)"-160") + (str_to_INTERVAL((char*)"168") + (-64 + 4 * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"326") + (str_to_INTERVAL((char*)"-744") + (str_to_INTERVAL((char*)"420") + (88 - 90 * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"963") + (str_to_INTERVAL((char*)"-1704") + (str_to_INTERVAL((char*)"546") + (str_to_INTERVAL((char*)"360") - str_to_INTERVAL((char*)"165") * y * y) * y * y) * y * y) * y * y + (74 + (str_to_INTERVAL((char*)"152") + (str_to_INTERVAL((char*)"-780") + (str_to_INTERVAL((char*)"600") + 50 * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-6364") + (str_to_INTERVAL((char*)"4752") + (str_to_INTERVAL((char*)"-1368") + (str_to_INTERVAL((char*)"1232") + str_to_INTERVAL((char*)"132") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-17736") + (str_to_INTERVAL((char*)"1440") + (str_to_INTERVAL((char*)"1776") + (str_to_INTERVAL((char*)"672") - 72 * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-26368") + (str_to_INTERVAL((char*)"-8128") + (str_to_INTERVAL((char*)"-2880") - str_to_INTERVAL((char*)"704") * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-25984") + (str_to_INTERVAL((char*)"-12288") - str_to_INTERVAL((char*)"8448") * y * y) * y * y + (str_to_INTERVAL((char*)"-9216") * y * y - str_to_INTERVAL((char*)"18432") + (str_to_INTERVAL((char*)"-6512") + str_to_INTERVAL((char*)"2240") * y * y + str_to_INTERVAL((char*)"3360") * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x);
coef[7] = 8 * sqrt2xplus1* pow( x,  3) *  pow( (2 * x + 1),  2) * (str_to_INTERVAL((char*)"11085") + (str_to_INTERVAL((char*)"-20040") + (str_to_INTERVAL((char*)"7110") + (str_to_INTERVAL((char*)"1560") + str_to_INTERVAL((char*)"285") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"84985") + (str_to_INTERVAL((char*)"-113560") + (str_to_INTERVAL((char*)"26070") + (str_to_INTERVAL((char*)"2600") - 95 * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"272180") + (str_to_INTERVAL((char*)"-202860") + (str_to_INTERVAL((char*)"-8580") + (str_to_INTERVAL((char*)"-5780") - str_to_INTERVAL((char*)"960") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"486967") + (str_to_INTERVAL((char*)"-81468") + (str_to_INTERVAL((char*)"-77058") + (str_to_INTERVAL((char*)"-2308") + str_to_INTERVAL((char*)"1467") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"543414") + (str_to_INTERVAL((char*)"105960") + (str_to_INTERVAL((char*)"41076") + (str_to_INTERVAL((char*)"13352") - str_to_INTERVAL((char*)"122") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"375920") + (str_to_INTERVAL((char*)"114300") + (str_to_INTERVAL((char*)"77148") + (str_to_INTERVAL((char*)"-11356") - str_to_INTERVAL((char*)"2172") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"94600") + (str_to_INTERVAL((char*)"-62040") + (str_to_INTERVAL((char*)"-152856") - str_to_INTERVAL((char*)"10904") * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-156880") + (str_to_INTERVAL((char*)"-204320") - str_to_INTERVAL((char*)"50448") * y * y) * y * y + (str_to_INTERVAL((char*)"29184") * y * y - str_to_INTERVAL((char*)"160576") + str_to_INTERVAL((char*)"30208") * x) * x) * x) * x) * x) * x) * x) * x) * x);
coef[8] = str_to_INTERVAL((char*)"-210") *   pow( (2 * x + 1),  2) * (6 + (-24 + (36 + (-24 + 6 * y * y) * y * y) * y * y) * y * y + (64 + (str_to_INTERVAL((char*)"-208") + (str_to_INTERVAL((char*)"240") + (str_to_INTERVAL((char*)"-112") + 16 * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"430") + (str_to_INTERVAL((char*)"-1064") + (str_to_INTERVAL((char*)"756") + (-40 - 82 * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"1615") + (str_to_INTERVAL((char*)"-3192") + (str_to_INTERVAL((char*)"1386") + (str_to_INTERVAL((char*)"536") - str_to_INTERVAL((char*)"345") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"2000") + (str_to_INTERVAL((char*)"-3256") + (str_to_INTERVAL((char*)"312") + (str_to_INTERVAL((char*)"1320") - str_to_INTERVAL((char*)"280") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-6216") + (str_to_INTERVAL((char*)"5056") + (str_to_INTERVAL((char*)"-2928") + (str_to_INTERVAL((char*)"2432") + str_to_INTERVAL((char*)"232") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-30464") + (str_to_INTERVAL((char*)"10944") + (str_to_INTERVAL((char*)"-960") + (str_to_INTERVAL((char*)"3136") + str_to_INTERVAL((char*)"192") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-61840") + (str_to_INTERVAL((char*)"-5248") + (str_to_INTERVAL((char*)"672") + (str_to_INTERVAL((char*)"640") - str_to_INTERVAL((char*)"144") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-78720") + (str_to_INTERVAL((char*)"-28544") + (str_to_INTERVAL((char*)"-14208") - str_to_INTERVAL((char*)"1408") * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-70400") + (str_to_INTERVAL((char*)"-33792") - str_to_INTERVAL((char*)"16896") * y * y) * y * y + (str_to_INTERVAL((char*)"-43584") + (str_to_INTERVAL((char*)"-14848") + str_to_INTERVAL((char*)"1664") * y * y) * y * y + (str_to_INTERVAL((char*)"-10496") + str_to_INTERVAL((char*)"7168") * y * y + str_to_INTERVAL((char*)"5888") * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x);
coef[9] = 0.-24 * sqrt2xplus1* pow( x,  3) *  pow( (2 * x + 1),  2) * (str_to_INTERVAL((char*)"6090") + (str_to_INTERVAL((char*)"-10710") + (str_to_INTERVAL((char*)"3465") + (str_to_INTERVAL((char*)"840") + str_to_INTERVAL((char*)"315") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"46690") + (str_to_INTERVAL((char*)"-60690") + (str_to_INTERVAL((char*)"12705") + (str_to_INTERVAL((char*)"1400") - str_to_INTERVAL((char*)"105") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"149918") + (str_to_INTERVAL((char*)"-107712") + (str_to_INTERVAL((char*)"-6222") + (str_to_INTERVAL((char*)"-3692") - str_to_INTERVAL((char*)"792") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"269972") + (str_to_INTERVAL((char*)"-40494") + (str_to_INTERVAL((char*)"-42315") + (str_to_INTERVAL((char*)"-1436") + str_to_INTERVAL((char*)"1173") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"305640") + (str_to_INTERVAL((char*)"63288") + (str_to_INTERVAL((char*)"21588") + (str_to_INTERVAL((char*)"8240") - str_to_INTERVAL((char*)"456") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"219550") + (str_to_INTERVAL((char*)"70524") + (str_to_INTERVAL((char*)"41208") + (str_to_INTERVAL((char*)"-6972") - str_to_INTERVAL((char*)"630") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"70720") + (str_to_INTERVAL((char*)"-21408") + (str_to_INTERVAL((char*)"-82224") - str_to_INTERVAL((char*)"2160") * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-62824") + (str_to_INTERVAL((char*)"-94128") - str_to_INTERVAL((char*)"6120") * y * y) * y * y + (str_to_INTERVAL((char*)"20160") * y * y - str_to_INTERVAL((char*)"74976") + str_to_INTERVAL((char*)"11520") * x) * x) * x) * x) * x) * x) * x) * x) * x);
coef[10] = 4 *  (2 * x + 1) * (str_to_INTERVAL((char*)"378") + (str_to_INTERVAL((char*)"-1512") + (str_to_INTERVAL((char*)"2268") + (str_to_INTERVAL((char*)"-1512") + str_to_INTERVAL((char*)"378") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"4788") + (str_to_INTERVAL((char*)"-16128") + (str_to_INTERVAL((char*)"19656") + (str_to_INTERVAL((char*)"-10080") + str_to_INTERVAL((char*)"1764") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"35154") + (str_to_INTERVAL((char*)"-93240") + (str_to_INTERVAL((char*)"77868") + (str_to_INTERVAL((char*)"-16632") - str_to_INTERVAL((char*)"3150") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"155925") + (str_to_INTERVAL((char*)"-335160") + (str_to_INTERVAL((char*)"182574") + (str_to_INTERVAL((char*)"28728") - str_to_INTERVAL((char*)"32067") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"329490") + (str_to_INTERVAL((char*)"-607320") + (str_to_INTERVAL((char*)"194292") + (str_to_INTERVAL((char*)"150696") - str_to_INTERVAL((char*)"61110") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-139608") + (str_to_INTERVAL((char*)"-91728") + (str_to_INTERVAL((char*)"-145152") + (str_to_INTERVAL((char*)"319536") - str_to_INTERVAL((char*)"20664") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-2702448") + (str_to_INTERVAL((char*)"1326528") + (str_to_INTERVAL((char*)"-429408") + (str_to_INTERVAL((char*)"504000") + str_to_INTERVAL((char*)"41328") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-7734384") + (str_to_INTERVAL((char*)"1048320") + (str_to_INTERVAL((char*)"-78624") + (str_to_INTERVAL((char*)"435456") + str_to_INTERVAL((char*)"15120") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-12751200") + (str_to_INTERVAL((char*)"-2459520") + (str_to_INTERVAL((char*)"-810432") + (str_to_INTERVAL((char*)"-8064") - str_to_INTERVAL((char*)"18144") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-14353920") + (str_to_INTERVAL((char*)"-5725440") + (str_to_INTERVAL((char*)"-2854656") - str_to_INTERVAL((char*)"177408") * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-11623648") + (str_to_INTERVAL((char*)"-5188608") + (str_to_INTERVAL((char*)"-1887936") + str_to_INTERVAL((char*)"31360") * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-6197568") + (str_to_INTERVAL((char*)"-1400832") + str_to_INTERVAL((char*)"481920") * y * y) * y * y + (str_to_INTERVAL((char*)"-1041024") + str_to_INTERVAL((char*)"921600") * y * y + str_to_INTERVAL((char*)"682240") * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x);
coef[11] = 8 *  pow( x,  3) * sqrt2xplus1* pow( (2 * x + 1),  2) * (str_to_INTERVAL((char*)"20685") + (str_to_INTERVAL((char*)"-35490") + (str_to_INTERVAL((char*)"10710") + (str_to_INTERVAL((char*)"2310") + str_to_INTERVAL((char*)"1785") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"158585") + (str_to_INTERVAL((char*)"-201110") + (str_to_INTERVAL((char*)"39270") + (str_to_INTERVAL((char*)"3850") - str_to_INTERVAL((char*)"595") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"510256") + (str_to_INTERVAL((char*)"-354444") + (str_to_INTERVAL((char*)"-23784") + (str_to_INTERVAL((char*)"-15364") - str_to_INTERVAL((char*)"3264") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"923635") + (str_to_INTERVAL((char*)"-123422") + (str_to_INTERVAL((char*)"-141414") + (str_to_INTERVAL((char*)"-5686") + str_to_INTERVAL((char*)"4607") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"1057430") + (str_to_INTERVAL((char*)"233404") + (str_to_INTERVAL((char*)"73992") + (str_to_INTERVAL((char*)"29564") - str_to_INTERVAL((char*)"2510") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"780900") + (str_to_INTERVAL((char*)"267492") + (str_to_INTERVAL((char*)"139188") + (str_to_INTERVAL((char*)"-24340") - str_to_INTERVAL((char*)"120") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"289800") + (str_to_INTERVAL((char*)"-23704") + (str_to_INTERVAL((char*)"-264600") + str_to_INTERVAL((char*)"1160") * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-151168") + (str_to_INTERVAL((char*)"-267200") + str_to_INTERVAL((char*)"25920") * y * y) * y * y + (str_to_INTERVAL((char*)"44160") * y * y - str_to_INTERVAL((char*)"211840") + str_to_INTERVAL((char*)"35840") * x) * x) * x) * x) * x) * x) * x) * x) * x);
coef[12] = 0.-10 *  (str_to_INTERVAL((char*)"126") + (str_to_INTERVAL((char*)"-504") + (str_to_INTERVAL((char*)"756") + (str_to_INTERVAL((char*)"-504") + str_to_INTERVAL((char*)"126") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"1848") + (str_to_INTERVAL((char*)"-6384") + (str_to_INTERVAL((char*)"8064") + (str_to_INTERVAL((char*)"-4368") + str_to_INTERVAL((char*)"840") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"14910") + (str_to_INTERVAL((char*)"-41832") + (str_to_INTERVAL((char*)"39060") + (str_to_INTERVAL((char*)"-12264") + str_to_INTERVAL((char*)"126") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"75411") + (str_to_INTERVAL((char*)"-173880") + (str_to_INTERVAL((char*)"112770") + (str_to_INTERVAL((char*)"-1512") - str_to_INTERVAL((char*)"12789") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"213780") + (str_to_INTERVAL((char*)"-425880") + (str_to_INTERVAL((char*)"186480") + (str_to_INTERVAL((char*)"69384") - str_to_INTERVAL((char*)"41748") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"173124") + (str_to_INTERVAL((char*)"-435456") + (str_to_INTERVAL((char*)"81144") + (str_to_INTERVAL((char*)"206976") - str_to_INTERVAL((char*)"47628") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-993888") + (str_to_INTERVAL((char*)"381024") + (str_to_INTERVAL((char*)"-239904") + str_to_INTERVAL((char*)"381024") * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-4379760") + (str_to_INTERVAL((char*)"1233792") + (str_to_INTERVAL((char*)"-312480") + (str_to_INTERVAL((char*)"481152") + str_to_INTERVAL((char*)"32592") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-9406656") + (str_to_INTERVAL((char*)"-120960") + (str_to_INTERVAL((char*)"-322560") + (str_to_INTERVAL((char*)"287616") + str_to_INTERVAL((char*)"4032") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-13285440") + (str_to_INTERVAL((char*)"-3548160") + (str_to_INTERVAL((char*)"-1491840") + (str_to_INTERVAL((char*)"-64512") - str_to_INTERVAL((char*)"12096") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-13444608") + (str_to_INTERVAL((char*)"-5558272") + (str_to_INTERVAL((char*)"-2493312") + (str_to_INTERVAL((char*)"-98304") + str_to_INTERVAL((char*)"3136") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-9821184") + (str_to_INTERVAL((char*)"-3996672") + (str_to_INTERVAL((char*)"-941568") + str_to_INTERVAL((char*)"39936") * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-4497408") + (str_to_INTERVAL((char*)"-768000") + str_to_INTERVAL((char*)"477696") * y * y) * y * y + (str_to_INTERVAL((char*)"520192") * y * y - str_to_INTERVAL((char*)"491520") + str_to_INTERVAL((char*)"442368") * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x);
coef[13] = 0.-40 * sqrt2xplus1* pow( x,  3) *  pow( (2 * x + 1),  2) * (str_to_INTERVAL((char*)"3255") + (str_to_INTERVAL((char*)"-5460") + (str_to_INTERVAL((char*)"1575") + (str_to_INTERVAL((char*)"210") + str_to_INTERVAL((char*)"420") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"24955") + (str_to_INTERVAL((char*)"-30940") + (str_to_INTERVAL((char*)"5775") + (str_to_INTERVAL((char*)"350") - str_to_INTERVAL((char*)"140") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"80444") + (str_to_INTERVAL((char*)"-54180") + (str_to_INTERVAL((char*)"-3738") + (str_to_INTERVAL((char*)"-3080") - str_to_INTERVAL((char*)"546") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"146293") + (str_to_INTERVAL((char*)"-17472") + (str_to_INTERVAL((char*)"-21357") + (str_to_INTERVAL((char*)"-1078") + str_to_INTERVAL((char*)"714") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"169156") + (str_to_INTERVAL((char*)"39224") + (str_to_INTERVAL((char*)"12732") + (str_to_INTERVAL((char*)"4504") - str_to_INTERVAL((char*)"412") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"127930") + (str_to_INTERVAL((char*)"45852") + (str_to_INTERVAL((char*)"22560") + (str_to_INTERVAL((char*)"-3468") + str_to_INTERVAL((char*)"102") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"52800") + (str_to_INTERVAL((char*)"2736") + (str_to_INTERVAL((char*)"-36528") + str_to_INTERVAL((char*)"704") * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-14792") + (str_to_INTERVAL((char*)"-34672") + str_to_INTERVAL((char*)"6456") * y * y) * y * y + (str_to_INTERVAL((char*)"4800") * y * y - str_to_INTERVAL((char*)"27744") + str_to_INTERVAL((char*)"4352") * x) * x) * x) * x) * x) * x) * x) * x) * x);
coef[14] = 40 *  (18 + (-72 + (str_to_INTERVAL((char*)"108") + (-72 + 18 * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"264") + (str_to_INTERVAL((char*)"-912") + (str_to_INTERVAL((char*)"1152") + (str_to_INTERVAL((char*)"-624") + str_to_INTERVAL((char*)"120") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"2130") + (str_to_INTERVAL((char*)"-5976") + (str_to_INTERVAL((char*)"5580") + (str_to_INTERVAL((char*)"-1752") + 18 * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"10773") + (str_to_INTERVAL((char*)"-24840") + (str_to_INTERVAL((char*)"16110") + (str_to_INTERVAL((char*)"-216") - str_to_INTERVAL((char*)"1827") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"30540") + (str_to_INTERVAL((char*)"-60840") + (str_to_INTERVAL((char*)"26640") + (str_to_INTERVAL((char*)"9912") - str_to_INTERVAL((char*)"5964") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"24732") + (str_to_INTERVAL((char*)"-62208") + (str_to_INTERVAL((char*)"11592") + (str_to_INTERVAL((char*)"29568") - str_to_INTERVAL((char*)"6804") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-141984") + (str_to_INTERVAL((char*)"54432") + (str_to_INTERVAL((char*)"-34272") + str_to_INTERVAL((char*)"54432") * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-625680") + (str_to_INTERVAL((char*)"176256") + (str_to_INTERVAL((char*)"-44640") + (str_to_INTERVAL((char*)"68736") + str_to_INTERVAL((char*)"4656") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-1343808") + (str_to_INTERVAL((char*)"-17280") + (str_to_INTERVAL((char*)"-46080") + (str_to_INTERVAL((char*)"41088") + str_to_INTERVAL((char*)"576") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-1897920") + (str_to_INTERVAL((char*)"-506880") + (str_to_INTERVAL((char*)"-213120") + (str_to_INTERVAL((char*)"-9216") - str_to_INTERVAL((char*)"1728") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-1921088") + (str_to_INTERVAL((char*)"-794944") + (str_to_INTERVAL((char*)"-354336") + (str_to_INTERVAL((char*)"-13312") + str_to_INTERVAL((char*)"880") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-1406464") + (str_to_INTERVAL((char*)"-576384") + (str_to_INTERVAL((char*)"-127104") + str_to_INTERVAL((char*)"7168") * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-652800") + (str_to_INTERVAL((char*)"-120576") + str_to_INTERVAL((char*)"75648") * y * y) * y * y + (str_to_INTERVAL((char*)"67072") * y * y - str_to_INTERVAL((char*)"83968") + str_to_INTERVAL((char*)"56320") * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x);
coef[15] = str_to_INTERVAL((char*)"120") *  pow( x,  3) * sqrt2xplus1* pow( (2 * x + 1),  2) * (str_to_INTERVAL((char*)"585") + (str_to_INTERVAL((char*)"-960") + (str_to_INTERVAL((char*)"270") + str_to_INTERVAL((char*)"105") *  pow( y,  4)) * y * y) * y * y + (str_to_INTERVAL((char*)"4485") + (str_to_INTERVAL((char*)"-5440") + (str_to_INTERVAL((char*)"990") - 35 *  pow( y,  4)) * y * y) * y * y + (str_to_INTERVAL((char*)"14484") + (str_to_INTERVAL((char*)"-9468") + (str_to_INTERVAL((char*)"-612") + (str_to_INTERVAL((char*)"-708") - 96 * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"26459") + (str_to_INTERVAL((char*)"-2820") + (str_to_INTERVAL((char*)"-3594") + (str_to_INTERVAL((char*)"-236") + str_to_INTERVAL((char*)"111") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"30886") + (str_to_INTERVAL((char*)"7448") + (str_to_INTERVAL((char*)"2628") + (str_to_INTERVAL((char*)"728") - 58 * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"23880") + (str_to_INTERVAL((char*)"8844") + (str_to_INTERVAL((char*)"4284") + (str_to_INTERVAL((char*)"-492") + 12 * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"10760") + (str_to_INTERVAL((char*)"1512") + (str_to_INTERVAL((char*)"-5208") + str_to_INTERVAL((char*)"104") * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-1072") + (str_to_INTERVAL((char*)"-4960") + str_to_INTERVAL((char*)"912") * y * y) * y * y + (str_to_INTERVAL((char*)"768") * y * y - str_to_INTERVAL((char*)"4032") + str_to_INTERVAL((char*)"512") * x) * x) * x) * x) * x) * x) * x) * x) * x);
coef[16] = 0.- (str_to_INTERVAL((char*)"270") + (str_to_INTERVAL((char*)"-1080") + (str_to_INTERVAL((char*)"1620") + (str_to_INTERVAL((char*)"-1080") + str_to_INTERVAL((char*)"270") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"3960") + (str_to_INTERVAL((char*)"-13680") + (str_to_INTERVAL((char*)"17280") + (str_to_INTERVAL((char*)"-9360") + str_to_INTERVAL((char*)"1800") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"31950") + (str_to_INTERVAL((char*)"-89640") + (str_to_INTERVAL((char*)"83700") + (str_to_INTERVAL((char*)"-26280") + str_to_INTERVAL((char*)"270") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"161595") + (str_to_INTERVAL((char*)"-372600") + (str_to_INTERVAL((char*)"241650") + (str_to_INTERVAL((char*)"-3240") - str_to_INTERVAL((char*)"27405") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"458100") + (str_to_INTERVAL((char*)"-912600") + (str_to_INTERVAL((char*)"399600") + (str_to_INTERVAL((char*)"148680") - str_to_INTERVAL((char*)"89460") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"370980") + (str_to_INTERVAL((char*)"-933120") + (str_to_INTERVAL((char*)"173880") + (str_to_INTERVAL((char*)"443520") - str_to_INTERVAL((char*)"102060") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-2129760") + (str_to_INTERVAL((char*)"816480") + (str_to_INTERVAL((char*)"-514080") + str_to_INTERVAL((char*)"816480") * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-9385200") + (str_to_INTERVAL((char*)"2643840") + (str_to_INTERVAL((char*)"-669600") + (str_to_INTERVAL((char*)"1031040") + str_to_INTERVAL((char*)"69840") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-20157120") + (str_to_INTERVAL((char*)"-259200") + (str_to_INTERVAL((char*)"-691200") + (str_to_INTERVAL((char*)"616320") + str_to_INTERVAL((char*)"8640") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-28468800") + (str_to_INTERVAL((char*)"-7603200") + (str_to_INTERVAL((char*)"-3196800") + (str_to_INTERVAL((char*)"-138240") - str_to_INTERVAL((char*)"25920") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-28824064") + (str_to_INTERVAL((char*)"-11933184") + (str_to_INTERVAL((char*)"-5351424") + (str_to_INTERVAL((char*)"-203264") + str_to_INTERVAL((char*)"11136") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-21158912") + (str_to_INTERVAL((char*)"-8699904") + (str_to_INTERVAL((char*)"-2052096") + str_to_INTERVAL((char*)"100352") * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-9977856") + (str_to_INTERVAL((char*)"-1916928") + str_to_INTERVAL((char*)"989184") * y * y) * y * y + (str_to_INTERVAL((char*)"933888") * y * y - str_to_INTERVAL((char*)"1507328") + str_to_INTERVAL((char*)"720896") * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x);
coef[17] = 0.-4 *  pow( x,  3) * sqrt2xplus1* pow( (2 * x + 1),  2) * (str_to_INTERVAL((char*)"6195") + (str_to_INTERVAL((char*)"-9930") + (str_to_INTERVAL((char*)"2745") + (str_to_INTERVAL((char*)"-480") + str_to_INTERVAL((char*)"1470") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"47495") + (str_to_INTERVAL((char*)"-56270") + (str_to_INTERVAL((char*)"10065") + (str_to_INTERVAL((char*)"-800") - str_to_INTERVAL((char*)"490") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"153670") + (str_to_INTERVAL((char*)"-97260") + (str_to_INTERVAL((char*)"-5550") + (str_to_INTERVAL((char*)"-9400") - str_to_INTERVAL((char*)"960") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"282019") + (str_to_INTERVAL((char*)"-26246") + (str_to_INTERVAL((char*)"-34971") + (str_to_INTERVAL((char*)"-3016") + str_to_INTERVAL((char*)"914") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"332368") + (str_to_INTERVAL((char*)"83480") + (str_to_INTERVAL((char*)"32172") + (str_to_INTERVAL((char*)"6624") - str_to_INTERVAL((char*)"384") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"262540") + (str_to_INTERVAL((char*)"100680") + (str_to_INTERVAL((char*)"48336") + (str_to_INTERVAL((char*)"-3512") + 36 * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"127600") + (str_to_INTERVAL((char*)"28160") + (str_to_INTERVAL((char*)"-38352") + str_to_INTERVAL((char*)"352") * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"5760") + (str_to_INTERVAL((char*)"-38400") + str_to_INTERVAL((char*)"4224") * y * y) * y * y + (str_to_INTERVAL((char*)"4608") * y * y - str_to_INTERVAL((char*)"31232") + str_to_INTERVAL((char*)"4096") * x) * x) * x) * x) * x) * x) * x) * x) * x);
coef[18] = 2 *  (30 + (str_to_INTERVAL((char*)"-120") + (str_to_INTERVAL((char*)"180") + (str_to_INTERVAL((char*)"-120") + 30 * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"440") + (str_to_INTERVAL((char*)"-1520") + (str_to_INTERVAL((char*)"1920") + (str_to_INTERVAL((char*)"-1040") + str_to_INTERVAL((char*)"200") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"3550") + (str_to_INTERVAL((char*)"-9960") + (str_to_INTERVAL((char*)"9300") + (str_to_INTERVAL((char*)"-2920") + 30 * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"17955") + (str_to_INTERVAL((char*)"-41400") + (str_to_INTERVAL((char*)"26850") + (str_to_INTERVAL((char*)"-360") - str_to_INTERVAL((char*)"3045") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"50900") + (str_to_INTERVAL((char*)"-101400") + (str_to_INTERVAL((char*)"44400") + (str_to_INTERVAL((char*)"16520") - str_to_INTERVAL((char*)"9940") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"41220") + (str_to_INTERVAL((char*)"-103680") + (str_to_INTERVAL((char*)"19320") + (str_to_INTERVAL((char*)"49280") - str_to_INTERVAL((char*)"11340") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-236640") + (str_to_INTERVAL((char*)"90720") + (str_to_INTERVAL((char*)"-57120") + str_to_INTERVAL((char*)"90720") * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-1042800") + (str_to_INTERVAL((char*)"293760") + (str_to_INTERVAL((char*)"-74400") + (str_to_INTERVAL((char*)"114560") + str_to_INTERVAL((char*)"7760") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-2239680") + (str_to_INTERVAL((char*)"-28800") + (str_to_INTERVAL((char*)"-76800") + (str_to_INTERVAL((char*)"68480") + str_to_INTERVAL((char*)"960") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-3163200") + (str_to_INTERVAL((char*)"-844800") + (str_to_INTERVAL((char*)"-355200") + (str_to_INTERVAL((char*)"-15360") - str_to_INTERVAL((char*)"2880") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-3203584") + (str_to_INTERVAL((char*)"-1329664") + (str_to_INTERVAL((char*)"-605184") + (str_to_INTERVAL((char*)"-25344") + str_to_INTERVAL((char*)"576") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-2358272") + (str_to_INTERVAL((char*)"-989184") + (str_to_INTERVAL((char*)"-270336") + str_to_INTERVAL((char*)"5632") * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-1130496") + (str_to_INTERVAL((char*)"-258048") + str_to_INTERVAL((char*)"67584") * y * y) * y * y + (str_to_INTERVAL((char*)"73728") * y * y - str_to_INTERVAL((char*)"196608") + str_to_INTERVAL((char*)"65536") * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x);

	return ;
}

void polynomial_determinant_tail(INTERVAL *coef, const INTERVAL& x)
{
	INTERVAL sqrt2xplus1;
	INTERVAL y;
	
	sqrt2xplus1 = sqrt(2*x+1);
	y= exp(x);

coef[19] = 0.-8 *  pow( x,  3) * pow( (2 * x + 1),  2) * (str_to_INTERVAL((char*)"-645") + (str_to_INTERVAL((char*)"1005") + (str_to_INTERVAL((char*)"-270") + (str_to_INTERVAL((char*)"105") - str_to_INTERVAL((char*)"195") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-4945") + (str_to_INTERVAL((char*)"5695") + (str_to_INTERVAL((char*)"-990") + (str_to_INTERVAL((char*)"175") + 65 * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-16034") + (str_to_INTERVAL((char*)"9756") + (str_to_INTERVAL((char*)"486") + (str_to_INTERVAL((char*)"1196") + 96 * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-29581") + (str_to_INTERVAL((char*)"2277") + (str_to_INTERVAL((char*)"3300") + (str_to_INTERVAL((char*)"373") - 69 * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-35240") + (str_to_INTERVAL((char*)"-9274") + (str_to_INTERVAL((char*)"-3834") + (str_to_INTERVAL((char*)"-570") + 18 * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-28500") + (str_to_INTERVAL((char*)"-11352") + (str_to_INTERVAL((char*)"-5364") + str_to_INTERVAL((char*)"176") * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-14960") + (str_to_INTERVAL((char*)"-4416") + str_to_INTERVAL((char*)"2112") * y * y) * y * y + (str_to_INTERVAL((char*)"2304") * y * y - str_to_INTERVAL((char*)"2688") + str_to_INTERVAL((char*)"2048") * x) * x) * x) * x) * x) * x) * x) * x);
coef[20] =  sqrt2xplus1*  pow( (2 * x + 1),  5) * (-6 + (24 + (-36 + (24 - 6 * y * y) * y * y) * y * y) * y * y + (-16 + (16 + (48 + (-80 + 32 * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"-158") + (str_to_INTERVAL((char*)"360") + (str_to_INTERVAL((char*)"-276") + (str_to_INTERVAL((char*)"104") - 30 * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"225") + (str_to_INTERVAL((char*)"-840") + (str_to_INTERVAL((char*)"822") + (str_to_INTERVAL((char*)"-216") + 9 * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"600") + (str_to_INTERVAL((char*)"440") + (str_to_INTERVAL((char*)"-1224") + 88 * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"1328") + (str_to_INTERVAL((char*)"-192") + str_to_INTERVAL((char*)"1056") * y * y) * y * y + (str_to_INTERVAL((char*)"1152") * y * y + str_to_INTERVAL((char*)"768") + str_to_INTERVAL((char*)"1024") * x) * x) * x) * x) * x) * x) * x);
coef[21] = 0.-4 *  pow( x,  3) * sqrt2xplus1* pow( (2 * x + 1),  2) * (str_to_INTERVAL((char*)"120") + (str_to_INTERVAL((char*)"-180") + (45 + (-30 + 45 * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"920") + (str_to_INTERVAL((char*)"-1020") + (str_to_INTERVAL((char*)"165") + (-50 - 15 * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"2992") + (str_to_INTERVAL((char*)"-1728") + (-78 + (str_to_INTERVAL((char*)"-268") - 18 * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"5560") + (str_to_INTERVAL((char*)"-324") + (str_to_INTERVAL((char*)"-543") + (-82 + 9 * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"6720") + (str_to_INTERVAL((char*)"1848") + (str_to_INTERVAL((char*)"804") + 88 * y * y) * y * y) * y * y + (str_to_INTERVAL((char*)"5600") + (str_to_INTERVAL((char*)"2304") + str_to_INTERVAL((char*)"1056") * y * y) * y * y + (str_to_INTERVAL((char*)"1152") * y * y + str_to_INTERVAL((char*)"3200") + str_to_INTERVAL((char*)"1024") * x) * x) * x) * x) * x) * x) * x);
	return ;
}

