/*	File: det_with_coef_mpfi.cc

	This code checks that the Wronskian W[l0, l1] does not vanish in the interval I_2, see Lemma
	4.13. 

	Uses: Interval.hpp, involution.cc

	Author: Jordi-Lluis Figueras, Jordi Villadelprat, Warwick Tucker
	Latest edit: Thu June 21 10:25:00 CET 2012
*/



#include<stdio.h>
#include<time.h>
#include<math.h>
#include<stdlib.h>
#include "Interval.hpp"
#include "involution.cc"

#define DIGITSPRINT 40

int P(const INTERVAL& x, INTERVAL& res);
void coefficients_determinant(INTERVAL *coef0, INTERVAL *coef1, const INTERVAL& x, const INTERVAL& z);

int main(int argc, char* argv[])
{
	if(argc < 4)
	{
		printf("Usage:\n\t[1]precision of the rigorous computations\n\t[2]Lower bound x\n\t[3]Upper bound x\n\n");
		printf("Execution 1:\n./det_with_coef_mpfi.x 64 1.e-3 4.e-1\n");
		printf("Execution 2:\n./det_with_coef_mpfi.x 64 1.e-4 1.e-3\n");
		printf("Execution 3:\n./det_with_coef_mpfi.x 64 1.e-5 1.e-4\n");
		printf("Execution 4:\n./det_with_coef_mpfi.x 64 1.e-6 1.e-5\n");
		printf("Execution 5:\n./det_with_coef_mpfi.x 64 1.e-7 1.e-6\n");
		printf("Execution 6:\n./det_with_coef_mpfi.x 64 1.e-8 1.e-7\n");
		printf("Execution 7:\n./det_with_coef_mpfi.x 64 1.e-9 1.e-8\n");
		printf("Execution 8:\n./det_with_coef_mpfi.x 64 1.e-10 1.e-9\n");
		printf("Execution 9:\n./det_with_coef_mpfi.x 64 1.e-11 1.e-10\n");
		printf("Execution 10:\n./det_with_coef_mpfi.x 64 1.e-12 1.e-11\n");
		printf("Execution 11:\n./det_with_coef_mpfi.x 64 1.e-13 1.e-12\n");
		printf("Execution 12:\n./det_with_coef_mpfi.x 64 1.e-14 1.e-13\n");
		printf("Execution 13:\n./det_with_coef_mpfi.x 64 1.e-15 1.e-14\n");
		printf("Execution 14:\n./det_with_coef_mpfi.x 64 1.e-16 1.e-15\n");
		printf("Execution 15:\n./det_with_coef_mpfi.x 64 1.e-17 1.e-16\n");
		printf("Execution 16:\n./det_with_coef_mpfi.x 64 1.e-18 1.e-17\n");
		printf("Execution 17:\n./det_with_coef_mpfi.x 64 1.e-19 1.e-18\n");
		printf("Execution 18:\n./det_with_coef_mpfi.x 64 1.e-20 1.e-19\n");
		printf("Execution 19:\n./det_with_coef_mpfi.x 64 1.e-21 1.e-20\n");
		exit(1);
	}
	
	int flagevaluation;
	int flagoutsidedefinition = 1;
	int prec = atoi(argv[1]), i;
	mpfr_t xmin, xmax, x, deltax, tolstep, aux;

	mpfr_set_default_prec(prec);

	INTERVAL X, Xaux, Xaux2, PX, GoodXleft, GoodXright, z;

	mpfr_init(xmin);
	mpfr_init(xmax);
	mpfr_init(x);
	mpfr_init(deltax);
	mpfr_init(tolstep);
	mpfr_init(aux);

	mpfr_set_str(tolstep, "1.e-75", 10, GMP_RNDN);
	mpfr_set_str(xmin, argv[2], 10, GMP_RNDN);
	mpfr_add_d(xmin, xmin, -0.5, GMP_RNDN);
	mpfr_set_str(xmax, argv[3], 10, GMP_RNDN);
	mpfr_add_d(xmax, xmax, -0.5, GMP_RNDN);

	mpfr_sub(deltax, xmin, xmax, GMP_RNDN);
	mpfr_set_str(aux, "1.e5", 10, GMP_RNDN);
	mpfr_div(deltax, deltax, aux, GMP_RNDN);

	mpfr_sub(x, xmin, deltax, GMP_RNDN);
	X.setfull(x, xmin);

	GoodXright = X;

	i = 0;
	do
	{	
		flagevaluation = P(X, PX);
		if(flagevaluation > 0 && mpfi_has_zero(PX.x) > 0)
		{
			flagevaluation = -1;
		}
		if(flagevaluation <= 0)
		{
			mpfi_bisect(Xaux.x, Xaux2.x, X.x); 
			X = Xaux;
		}else
		{
			flagoutsidedefinition = mpfi_cmp_fr(X.x, xmax);
			mpfi_diam_abs(deltax, X.x);
			if(i%100 == 0)
			{
				printf("x = ");
				mpfi_out_str(stdout, 10, DIGITSPRINT, X.x);
				printf("\n");
				printf("P(x) = ");
				mpfi_out_str(stdout, 10, DIGITSPRINT, PX.x);
				printf("\n");
				printf("delta = ");
				mpfr_out_str(stdout, 10, DIGITSPRINT, deltax, GMP_RNDN);
				printf("\n");
				printf("\n");
				i = 0;
			}
			i++;

			GoodXleft = X;
			mpfi_get_right(aux, X.x);
			mpfi_set_fr(X.x, aux);
			mpfr_mul_d(deltax, deltax, 1.01, GMP_RNDN);
			mpfr_add(aux, aux, deltax, GMP_RNDN);
			mpfi_put_fr(X.x, aux);
		}
	}while(mpfr_cmpabs(deltax, tolstep) > 0 && flagoutsidedefinition < 0);

	printf("Left interval validated:\n");
	mpfi_out_str(stdout, 10, DIGITSPRINT, GoodXright.x);
	printf("\n");
	printf("Right interval validated:\n");
	mpfi_out_str(stdout, 10, DIGITSPRINT, GoodXleft.x);
	printf("\n");
	
	return 0;
}

int P(const INTERVAL& x, INTERVAL& res)
{
	INTERVAL z, *coef0, *coef1;
	int flag = 0;

	flag = obtain_involution(x, z);

	if(flag > 0)
	{
		coef0 = new INTERVAL[2];
		coef1 = new INTERVAL[2];
		coefficients_determinant(coef0, coef1, x, z);

		res = coef0[0]*coef1[1]-coef1[0]*coef0[1];
		delete[] coef0;
		delete[] coef1;
	}

	return flag;
}

void coefficients_determinant(INTERVAL *coef0, INTERVAL *coef1, const INTERVAL& x, const INTERVAL& z)
{
	INTERVAL y = exp(x);
	INTERVAL w = exp(x)*sqrt((2.*z+1.)/(2.*x+1.));

	coef0[0]=(y*(-6.*y*y*x*x+3.*y*y*y*y+3.*y*y*y*y*x*x+8.*y*y*x*x*x+3.-2.*y*y*x+15.*x*x+7.*x+32.*x*x*x*x-5.*y*y*y*y*x+12.*x*x*x-6.*y*y)/(x*x*x*x*x)-w*(-6.*w*w*z*z+3.*w*w*w*w+3.*w*w*w*w*z*z+8.*w*w*z*z*z+3.-2.*w*w*z+15.*z*z+7.*z+32.*z*z*z*z-5.*w*w*w*w*z+12.*z*z*z-6.*w*w)/(z*z*z*z*z));

	coef0[1]=(y*(-2.*y*y*x+8.*y*y*x*x-y*y*y*y+2.*y*y+3.*y*y*y*y*x-12.*x*x-1.+32.*x*x*x-x)/(x*x*x)-w*(-2.*w*w*z+8.*w*w*z*z-w*w*w*w+2.*w*w+3.*w*w*w*w*z-12.*z*z-1.+32.*z*z*z-z)/(z*z*z));

	coef1[0]=(y*y*y*(-10.*y*y*x+12.*y*y*x*x-34.*y*y*x*x*x+30.*y*y+24.*y*y*x*x*x*x-15.*y*y*y*y-15.+35.*y*y*y*y*x-34.*y*y*y*y*x*x-20.*x*x*x*x-38.*x*x-25.*x-9.*x*x*x+15.*y*y*y*y*x*x*x+32.*x*x*x*x*x)/(x*x*x*x*x*x*x)-w*w*w*(-10.*w*w*z+12.*w*w*z*z-34.*w*w*z*z*z+30.*w*w+24.*w*w*z*z*z*z-15.*w*w*w*w-15.+35.*w*w*w*w*z-34.*w*w*w*w*z*z-20.*z*z*z*z-38.*z*z-25.*z-9.*z*z*z+15.*w*w*w*w*z*z*z+32.*z*z*z*z*z)/(z*z*z*z*z*z*z));

	coef1[1] = y*y*y*(10.*y*y*x-6.*y*y-14*y*y*x*x+24*y*y*x*x*x+3*y*y*y*y+3-11*y*y*y*y*x+15*y*y*y*y*x*x+32*x*x*x*x+11*x*x+x-12*x*x*x)/(x*x*x*x*x)-w*w*w*(10.*w*w*z-6.*w*w-14*w*w*z*z+24*w*w*z*z*z+3*w*w*w*w+3-11*w*w*w*w*z+15*w*w*w*w*z*z+32*z*z*z*z+11*z*z+z-12*z*z*z)/(z*z*z*z*z);

	return ;
}
