/*	File: det_mpfi.cc

	This code checks that the Wronskian W[l0, l1, l2] does not vanish in the interval I_21, see Lemma
	4.10. 

	Uses: Interval.hpp, involution.cc

	Author: Jordi-Lluis Figueras, Jordi Villadelprat, Warwick Tucker
	Latest edit: Thu June 21 10:25:00 CET 2012
*/

#include<stdio.h>
#include<time.h>
#include<math.h>
#include<stdlib.h>
#include "Interval.hpp"
#include "involution.cc"

#define DIGITSPRINT 40

int P(const INTERVAL& x, INTERVAL& res);
void polynomial_determinant(INTERVAL *coef, const INTERVAL& x);

int main(int argc, char* argv[])
{
	if(argc < 4)
	{
		printf("Usage:\n\t[1]precision of the rigorous computations\n\t[2]Lower bound x\n\t[3]Upper bound x\n\n");
		printf("Execution 1:\n./det_mpfi.x 128 1.e-32 1.e-14\n");
		printf("Execution 2:\n./det_mpfi.x 64 1.e-14 1.e-8 \n");
		printf("Execution 3:\n./det_mpfi.x 64 1.e-8 1.e-7\n");
		exit(1);
	}
	
	int flagevaluation;
	int flagoutsidedefinition = 1;
	int prec = atoi(argv[1]), i;
	mpfr_t xmin, xmax, x, deltax, tolstep, aux;

	mpfr_set_default_prec(prec);

	INTERVAL X, Xaux, Xaux2, PX, GoodXleft, GoodXright, z;

	mpfr_init(xmin);
	mpfr_init(xmax);
	mpfr_init(x);
	mpfr_init(deltax);
	mpfr_init(tolstep);
	mpfr_init(aux);

	mpfr_set_str(tolstep, "1.e-75", 10, GMP_RNDN);
	mpfr_set_str(xmin, argv[2], 10, GMP_RNDN);
	mpfr_add_d(xmin, xmin, -0.5, GMP_RNDN);
	mpfr_set_str(xmax, argv[3], 10, GMP_RNDN);
	mpfr_add_d(xmax, xmax, -0.5, GMP_RNDN);

	mpfr_sub(deltax, xmin, xmax, GMP_RNDN);
	mpfr_set_str(aux, "1.e6", 10, GMP_RNDN);
	mpfr_div(deltax, deltax, aux, GMP_RNDN);

	mpfr_sub(x, xmin, deltax, GMP_RNDN);
	X.setfull(x, xmin);

	GoodXright = X;

	i = 0;
	do
	{	
		flagevaluation = P(X, PX);

		if(flagevaluation > 0 && mpfi_has_zero(PX.x) > 0)
		{
			flagevaluation = -1;
		}
		if(flagevaluation <= 0)
		{
			mpfi_bisect(Xaux.x, Xaux2.x, X.x); 
			X = Xaux;
		}else
		{
			flagoutsidedefinition = mpfi_cmp_fr(X.x, xmax);
			mpfi_diam_abs(deltax, X.x);
			if(i%100 == 0)
			{
				printf("x = ");
				mpfi_out_str(stdout, 10, DIGITSPRINT, X.x);
				printf("\n");
				printf("P(x) = ");
				mpfi_out_str(stdout, 10, DIGITSPRINT, PX.x);
				printf("\n");
				printf("delta = ");
				mpfr_out_str(stdout, 10, DIGITSPRINT, deltax, GMP_RNDN);
				printf("\n");
				printf("\n");
				i = 0;
			}
			i++;

			GoodXleft = X;
			mpfi_get_right(aux, X.x);
			mpfi_set_fr(X.x, aux);
			mpfr_mul_d(deltax, deltax, 1.01, GMP_RNDN);
			mpfr_add(aux, aux, deltax, GMP_RNDN);
			mpfi_put_fr(X.x, aux);
		}
	}while(mpfr_cmpabs(deltax, tolstep) > 0 && flagoutsidedefinition < 0);

	printf("%d\n", flagoutsidedefinition);

	printf("Left interval validated:\n");
	mpfi_out_str(stdout, 10, DIGITSPRINT, GoodXright.x);
	printf("\n");
	printf("Right interval validated:\n");
	mpfi_out_str(stdout, 10, DIGITSPRINT, GoodXleft.x);
	printf("\n");
	
	return 0;
}

int P(const INTERVAL& x, INTERVAL& res)
{
	INTERVAL z, z1, aux, upperbound;
	int flag = 0;
	int i;

	INTERVAL *coef;
	
	coef = new INTERVAL[34];

	polynomial_determinant(coef, x);

	flag = obtain_involution(x, z1);
	z = sqrt(2.*z1+1.);

	if(flag > 0) 
	{
		res = z*coef[33];
		for(i = 32; i > 0; i--)
		{
			aux = res;
			res = z*(aux+coef[i]);
		}
		aux = res;
		res = aux+coef[0];
	}
	
	delete[] coef;
	
	return flag;
}

void polynomial_determinant(INTERVAL *coef, const INTERVAL& x)
{
	INTERVAL sqrt2xplus1;
	INTERVAL y;
	
	sqrt2xplus1 = sqrt(2*x+1);
	y= exp(x);

	coef[0] = (-1)*sqrt2xplus1 *  pow( (2 * x + 1),  10) * (-72 + (432 + (-1080 + (1440 + (-1080 + (432 - 72 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (2943 + (str_to_INTERVAL((char *)"-12222") + (str_to_INTERVAL((char *)"19665") + (str_to_INTERVAL((char *)"-15300") + (5985 + (-1278 + 207 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (3549 + (9570 + (str_to_INTERVAL((char *)"-52305") + (str_to_INTERVAL((char *)"64140") + (str_to_INTERVAL((char *)"-27525") + (2706 - 135 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"20567") + (str_to_INTERVAL((char *)"-45982") + (str_to_INTERVAL((char *)"81965") + (str_to_INTERVAL((char *)"-110900") + (str_to_INTERVAL((char *)"57025") + (-2702 + 27 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (-156 + (str_to_INTERVAL((char *)"88640") + (str_to_INTERVAL((char *)"-146120") + (str_to_INTERVAL((char *)"114000") + (str_to_INTERVAL((char *)"-57020") + 656 * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"31056") + (str_to_INTERVAL((char *)"-80416") + (str_to_INTERVAL((char *)"219600") + (str_to_INTERVAL((char *)"-148800") + str_to_INTERVAL((char *)"21760") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"40256") + (str_to_INTERVAL((char *)"121216") + (str_to_INTERVAL((char *)"-152960") + str_to_INTERVAL((char *)"87680") * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"81408") + (str_to_INTERVAL((char *)"25600") + str_to_INTERVAL((char *)"204800") * y * y) * y * y + (str_to_INTERVAL((char *)"96256") * y * y + str_to_INTERVAL((char *)"36864") + str_to_INTERVAL((char *)"32768") * x) * x) * x) * x) * x) * x) * x) * x) * x) * y * y;
	coef[1] = -15 * x *  pow( (2 * x + 1),  10) * (-270 + (1080 + (-1620 + (1080 - 270 * y * y) * y * y) * y * y) * y * y + (-360 + (-720 + (4320 + (-5040 + 1800 * y * y) * y * y) * y * y) * y * y + (-1770 + (3480 + (-6300 + (9240 - 4650 * y * y) * y * y) * y * y) * y * y + (192 + (-7728 + (9072 + (-4368 + 2832 * y * y) * y * y) * y * y) * y * y + (-924 + (1008 + (str_to_INTERVAL((char *)"-13608") + (7728 + 1476 * y * y) * y * y) * y * y) * y * y + (-7251 + (-1992 + (-2178 + (1704 - 1803 * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-14604") + (str_to_INTERVAL((char *)"-13704") + (-2304 + (-7080 - 420 * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-11596") + (str_to_INTERVAL((char *)"-13792") + (str_to_INTERVAL((char *)"-14632") + (-1248 + 900 * y * y) * y * y) * y * y) * y * y + (-5152 + (928 + (-1248 + 4896 * y * y) * y * y) * y * y + (2496 + (256 + str_to_INTERVAL((char *)"16000") * y * y) * y * y + (1024 + 9728 * y * y + 4096 * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * y * y;
	coef[2] = 15 * sqrt2xplus1 *  pow( (2 * x + 1),  10) * (-72 + (432 + (-1080 + (1440 + (-1080 + (432 - 72 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (2943 + (str_to_INTERVAL((char *)"-12222") + (str_to_INTERVAL((char *)"19665") + (str_to_INTERVAL((char *)"-15300") + (5985 + (-1278 + 207 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (3549 + (9570 + (str_to_INTERVAL((char *)"-52305") + (str_to_INTERVAL((char *)"64140") + (str_to_INTERVAL((char *)"-27525") + (2706 - 135 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"20567") + (str_to_INTERVAL((char *)"-45982") + (str_to_INTERVAL((char *)"81965") + (str_to_INTERVAL((char *)"-110900") + (str_to_INTERVAL((char *)"57025") + (-2702 + 27 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (-156 + (str_to_INTERVAL((char *)"88640") + (str_to_INTERVAL((char *)"-146120") + (str_to_INTERVAL((char *)"114000") + (str_to_INTERVAL((char *)"-57020") + 656 * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"31056") + (str_to_INTERVAL((char *)"-80416") + (str_to_INTERVAL((char *)"219600") + (str_to_INTERVAL((char *)"-148800") + str_to_INTERVAL((char *)"21760") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"40256") + (str_to_INTERVAL((char *)"121216") + (str_to_INTERVAL((char *)"-152960") + str_to_INTERVAL((char *)"87680") * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"81408") + (str_to_INTERVAL((char *)"25600") + str_to_INTERVAL((char *)"204800") * y * y) * y * y + (str_to_INTERVAL((char *)"96256") * y * y + str_to_INTERVAL((char *)"36864") + str_to_INTERVAL((char *)"32768") * x) * x) * x) * x) * x) * x) * x) * x) * x) * y * y;
	coef[3] = -10 * x *  pow( (2 * x + 1),  9) * (6120 + (str_to_INTERVAL((char *)"-24390") + (str_to_INTERVAL((char *)"36360") + (str_to_INTERVAL((char *)"-23940") + (5760 + 90 * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"20400") + (str_to_INTERVAL((char *)"-32520") + (str_to_INTERVAL((char *)"-24240") + (str_to_INTERVAL((char *)"63840") + (str_to_INTERVAL((char *)"-26880") - 600 * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"57280") + (str_to_INTERVAL((char *)"-49250") + (str_to_INTERVAL((char *)"-48200") + (str_to_INTERVAL((char *)"16340") + (str_to_INTERVAL((char *)"22520") + 1310 * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"77568") + (str_to_INTERVAL((char *)"17344") + (str_to_INTERVAL((char *)"70544") + (str_to_INTERVAL((char *)"-303696") + (str_to_INTERVAL((char *)"137264") + 976 * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"23664") + (str_to_INTERVAL((char *)"310996") + (str_to_INTERVAL((char *)"-104016") + (str_to_INTERVAL((char *)"30936") + (str_to_INTERVAL((char *)"-158848") - 3372 * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"303082") + (str_to_INTERVAL((char *)"-166867") + (str_to_INTERVAL((char *)"744668") + (str_to_INTERVAL((char *)"-405338") + (str_to_INTERVAL((char *)"-15654") + 909 * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"692496") + (str_to_INTERVAL((char *)"669020") + (str_to_INTERVAL((char *)"-159032") + (str_to_INTERVAL((char *)"121440") + (str_to_INTERVAL((char *)"81432") + 3060 * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"854760") + (str_to_INTERVAL((char *)"912260") + (str_to_INTERVAL((char *)"828096") + (str_to_INTERVAL((char *)"309528") + (-648 - 2700 * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"582208") + (str_to_INTERVAL((char *)"297792") + (str_to_INTERVAL((char *)"704832") + (str_to_INTERVAL((char *)"-86976") - str_to_INTERVAL((char *)"38016") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"84096") + (str_to_INTERVAL((char *)"383616") + (str_to_INTERVAL((char *)"-962688") - str_to_INTERVAL((char *)"150336") * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"144384") + (str_to_INTERVAL((char *)"-731136") + str_to_INTERVAL((char *)"104448") * y * y) * y * y + (str_to_INTERVAL((char *)"218112") * y * y - str_to_INTERVAL((char *)"368640") + str_to_INTERVAL((char *)"196608") * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * y * y;
	coef[4] = 105 * sqrt2xplus1 *  pow( (2 * x + 1),  8) * (72 + (-432 + (1080 + (-1440 + (1080 + (-432 + 72 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (-2655 + (str_to_INTERVAL((char *)"10494") + (str_to_INTERVAL((char *)"-15345") + (9540 + (-1665 + (-450 + 81 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-15033") + (str_to_INTERVAL((char *)"37590") + (str_to_INTERVAL((char *)"-22035") + (-8700 + (7905 + (678 - 405 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-46535") + (str_to_INTERVAL((char *)"56590") + (str_to_INTERVAL((char *)"48595") + (str_to_INTERVAL((char *)"-84460") + (str_to_INTERVAL((char *)"29135") + (-3010 - 315 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-96308") + (str_to_INTERVAL((char *)"57008") + (str_to_INTERVAL((char *)"27480") + (str_to_INTERVAL((char *)"73040") + (str_to_INTERVAL((char *)"-60980") + (-672 + 432 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-112700") + (str_to_INTERVAL((char *)"-90216") + (str_to_INTERVAL((char *)"37020") + (str_to_INTERVAL((char *)"136400") + (str_to_INTERVAL((char *)"-21780") + (8184 - 108 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-155456") + (str_to_INTERVAL((char *)"-170912") + (str_to_INTERVAL((char *)"-132560") + (str_to_INTERVAL((char *)"51520") + (str_to_INTERVAL((char *)"141040") - 2624 * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-358256") + (str_to_INTERVAL((char *)"-172000") + (str_to_INTERVAL((char *)"-496560") + (str_to_INTERVAL((char *)"244480") - str_to_INTERVAL((char *)"87040") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-523920") + (str_to_INTERVAL((char *)"-673120") + (str_to_INTERVAL((char *)"-183760") - str_to_INTERVAL((char *)"350720") * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-508896") + (str_to_INTERVAL((char *)"-516544") - str_to_INTERVAL((char *)"801440") * y * y) * y * y + (str_to_INTERVAL((char *)"-292208") + (str_to_INTERVAL((char *)"-351264") - str_to_INTERVAL((char *)"66800") * y * y) * y * y + (str_to_INTERVAL((char *)"-109312") + (str_to_INTERVAL((char *)"-44800") + str_to_INTERVAL((char *)"50880") * y * y) * y * y + (str_to_INTERVAL((char *)"-21440") + (str_to_INTERVAL((char *)"40320") + str_to_INTERVAL((char *)"33600") * y * y) * y * y + (str_to_INTERVAL((char *)"26880") + str_to_INTERVAL((char *)"38400") * y * y + str_to_INTERVAL((char *)"30720") * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * y * y;
	coef[5] = 3 * x *  pow( (2 * x + 1),  8) * (str_to_INTERVAL((char *)"144090") + (str_to_INTERVAL((char *)"-572040") + (str_to_INTERVAL((char *)"847350") + (str_to_INTERVAL((char *)"-550800") + (str_to_INTERVAL((char *)"127350") + (3960 + 90 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"768480") + (str_to_INTERVAL((char *)"-1906800") + (str_to_INTERVAL((char *)"1129800") + (str_to_INTERVAL((char *)"367200") + (str_to_INTERVAL((char *)"-339600") + (str_to_INTERVAL((char *)"-18480") - 600 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"2328070") + (str_to_INTERVAL((char *)"-2748520") + (str_to_INTERVAL((char *)"-2169550") + (str_to_INTERVAL((char *)"3282000") + (str_to_INTERVAL((char *)"-700550") + (7480 + 1070 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"4598936") + (str_to_INTERVAL((char *)"-2039376") + (str_to_INTERVAL((char *)"-602560") + (str_to_INTERVAL((char *)"-6172320") + (str_to_INTERVAL((char *)"4070040") + (str_to_INTERVAL((char *)"142384") + 2896 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"4568892") + (str_to_INTERVAL((char *)"7682640") + (str_to_INTERVAL((char *)"596060") + (str_to_INTERVAL((char *)"-12866880") + (str_to_INTERVAL((char *)"2476500") + (str_to_INTERVAL((char *)"-75152") - 6060 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"10419385") + (str_to_INTERVAL((char *)"7925440") + (str_to_INTERVAL((char *)"12532815") + (str_to_INTERVAL((char *)"-7251040") + (str_to_INTERVAL((char *)"-7748745") + (str_to_INTERVAL((char *)"-181280") - 575 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"34203560") + (str_to_INTERVAL((char *)"7680120") + (str_to_INTERVAL((char *)"29826060") + (str_to_INTERVAL((char *)"-17756240") + (str_to_INTERVAL((char *)"1483920") + (str_to_INTERVAL((char *)"176440") + 9500 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"53778400") + (str_to_INTERVAL((char *)"57305920") + (str_to_INTERVAL((char *)"11383460") + (str_to_INTERVAL((char *)"15180640") + (str_to_INTERVAL((char *)"3613480") + (str_to_INTERVAL((char *)"61600") - 7500 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"52149312") + (str_to_INTERVAL((char *)"45763648") + (str_to_INTERVAL((char *)"60029920") + (str_to_INTERVAL((char *)"13031200") + (str_to_INTERVAL((char *)"-1328160") - str_to_INTERVAL((char *)"132000") * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"25613104") + (str_to_INTERVAL((char *)"24312320") + (str_to_INTERVAL((char *)"4979680") + (str_to_INTERVAL((char *)"-14813440") - str_to_INTERVAL((char *)"1111760") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"8250240") + (str_to_INTERVAL((char *)"-507520") + (str_to_INTERVAL((char *)"-40973440") + str_to_INTERVAL((char *)"1427840") * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-4002560") + (str_to_INTERVAL((char *)"-23833600") + str_to_INTERVAL((char *)"16289280") * y * y) * y * y + (str_to_INTERVAL((char *)"-5672960") + str_to_INTERVAL((char *)"14878720") * y * y + str_to_INTERVAL((char *)"6471680") * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * y * y;
	coef[6] = -35 * sqrt2xplus1 *  pow( (2 * x + 1),  7) * (936 + (-5616 + (str_to_INTERVAL((char *)"14040") + (str_to_INTERVAL((char *)"-18720") + (str_to_INTERVAL((char *)"14040") + (-5616 + 936 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-32643") + (str_to_INTERVAL((char *)"125190") + (str_to_INTERVAL((char *)"-171405") + (str_to_INTERVAL((char *)"86580") + (6435 + (str_to_INTERVAL((char *)"-17082") + 2925 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-264459") + (str_to_INTERVAL((char *)"761514") + (str_to_INTERVAL((char *)"-685425") + (str_to_INTERVAL((char *)"134940") + (str_to_INTERVAL((char *)"59475") + (-2886 - 3159 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-995813") + (str_to_INTERVAL((char *)"1713010") + (str_to_INTERVAL((char *)"58825") + (str_to_INTERVAL((char *)"-1324180") + (str_to_INTERVAL((char *)"584285") + (str_to_INTERVAL((char *)"-21502") - str_to_INTERVAL((char *)"14625") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-2461914") + (str_to_INTERVAL((char *)"2212444") + (str_to_INTERVAL((char *)"1620710") + (str_to_INTERVAL((char *)"-1246440") + (str_to_INTERVAL((char *)"-35230") + (str_to_INTERVAL((char *)"-86996") - 2574 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-3969108") + (str_to_INTERVAL((char *)"309400") + (str_to_INTERVAL((char *)"1195740") + (str_to_INTERVAL((char *)"3672240") + (str_to_INTERVAL((char *)"-1868620") + (str_to_INTERVAL((char *)"88920") + 9828 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-4768008") + (str_to_INTERVAL((char *)"-4873232") + (str_to_INTERVAL((char *)"-698600") + (str_to_INTERVAL((char *)"4276640") + (str_to_INTERVAL((char *)"1267240") + (str_to_INTERVAL((char *)"178672") - 2808 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-8149824") + (str_to_INTERVAL((char *)"-6985472") + (str_to_INTERVAL((char *)"-9964000") + (str_to_INTERVAL((char *)"4336320") + (str_to_INTERVAL((char *)"2535520") - str_to_INTERVAL((char *)"68224") * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-15612576") + (str_to_INTERVAL((char *)"-12611840") + (str_to_INTERVAL((char *)"-15584160") + (str_to_INTERVAL((char *)"1932480") - str_to_INTERVAL((char *)"2263040") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-20062080") + (str_to_INTERVAL((char *)"-24304256") + (str_to_INTERVAL((char *)"-14514176") - str_to_INTERVAL((char *)"8841088") * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-17377824") + (str_to_INTERVAL((char *)"-18061952") + (str_to_INTERVAL((char *)"-22043296") - str_to_INTERVAL((char *)"748224") * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-9150912") + (str_to_INTERVAL((char *)"-9846656") + (str_to_INTERVAL((char *)"-1930560") + str_to_INTERVAL((char *)"529920") * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-3406720") + (str_to_INTERVAL((char *)"-878592") + (str_to_INTERVAL((char *)"3173760") + str_to_INTERVAL((char *)"403200") * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"189696") + (str_to_INTERVAL((char *)"2649600") + str_to_INTERVAL((char *)"1555200") * y * y) * y * y + (str_to_INTERVAL((char *)"1612800") + str_to_INTERVAL((char *)"1382400") * y * y + str_to_INTERVAL((char *)"368640") * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * y * y;
	coef[7] = -14 * x *  pow( (2 * x + 1),  7) * (str_to_INTERVAL((char *)"135270") + (str_to_INTERVAL((char *)"-534870") + (str_to_INTERVAL((char *)"787050") + (str_to_INTERVAL((char *)"-504900") + (str_to_INTERVAL((char *)"112050") + (5130 + 270 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"991980") + (str_to_INTERVAL((char *)"-2852640") + (str_to_INTERVAL((char *)"2623500") + (str_to_INTERVAL((char *)"-673200") + (str_to_INTERVAL((char *)"-74700") + (str_to_INTERVAL((char *)"-13680") - 1260 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"3645450") + (str_to_INTERVAL((char *)"-6193770") + (str_to_INTERVAL((char *)"146550") + (str_to_INTERVAL((char *)"3670500") + (str_to_INTERVAL((char *)"-1232850") + (str_to_INTERVAL((char *)"-35850") - 30 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"8790588") + (str_to_INTERVAL((char *)"-7278888") + (str_to_INTERVAL((char *)"-4464180") + (str_to_INTERVAL((char *)"359040") + (str_to_INTERVAL((char *)"2386020") + (str_to_INTERVAL((char *)"194472") + str_to_INTERVAL((char *)"12948") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"13403844") + (str_to_INTERVAL((char *)"2711772") + (str_to_INTERVAL((char *)"-726300") + (str_to_INTERVAL((char *)"-22810200") + (str_to_INTERVAL((char *)"9457020") + (str_to_INTERVAL((char *)"235836") - 3972 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"20624151") + (str_to_INTERVAL((char *)"19562025") + (str_to_INTERVAL((char *)"11513595") + (str_to_INTERVAL((char *)"-28942290") + (str_to_INTERVAL((char *)"-2879115") + (str_to_INTERVAL((char *)"-415671") - str_to_INTERVAL((char *)"22695") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"57060102") + (str_to_INTERVAL((char *)"19810968") + (str_to_INTERVAL((char *)"49322430") + (str_to_INTERVAL((char *)"-30141720") + (str_to_INTERVAL((char *)"-13120950") + (str_to_INTERVAL((char *)"-141552") + str_to_INTERVAL((char *)"20802") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"120198256") + (str_to_INTERVAL((char *)"70493248") + (str_to_INTERVAL((char *)"66046780") + (str_to_INTERVAL((char *)"-19499680") + (str_to_INTERVAL((char *)"6731480") + (str_to_INTERVAL((char *)"455024") + 5052 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"151264048") + (str_to_INTERVAL((char *)"150462656") + (str_to_INTERVAL((char *)"78769560") + (str_to_INTERVAL((char *)"45187520") + (str_to_INTERVAL((char *)"5316640") + (str_to_INTERVAL((char *)"-144192") - str_to_INTERVAL((char *)"13032") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"118897104") + (str_to_INTERVAL((char *)"104606384") + (str_to_INTERVAL((char *)"116846240") + (str_to_INTERVAL((char *)"4572960") + (str_to_INTERVAL((char *)"-5094320") - str_to_INTERVAL((char *)"105296") * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"52796512") + (str_to_INTERVAL((char *)"45709440") + (str_to_INTERVAL((char *)"-32528320") + (str_to_INTERVAL((char *)"-27121280") + str_to_INTERVAL((char *)"186080") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"10361344") + (str_to_INTERVAL((char *)"-24652544") + (str_to_INTERVAL((char *)"-45692160") + str_to_INTERVAL((char *)"11407360") * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-11816448") + (str_to_INTERVAL((char *)"-16400384") + str_to_INTERVAL((char *)"30100480") * y * y) * y * y + (str_to_INTERVAL((char *)"630784") + str_to_INTERVAL((char *)"19841024") * y * y + str_to_INTERVAL((char *)"3899392") * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * y * y;
	coef[8] = 105 * sqrt2xplus1 *  pow( (2 * x + 1),  6) * (936 + (-5616 + (str_to_INTERVAL((char *)"14040") + (str_to_INTERVAL((char *)"-18720") + (str_to_INTERVAL((char *)"14040") + (-5616 + 936 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-30771") + (str_to_INTERVAL((char *)"113958") + (str_to_INTERVAL((char *)"-143325") + (str_to_INTERVAL((char *)"49140") + (str_to_INTERVAL((char *)"34515") + (str_to_INTERVAL((char *)"-28314") + 4797 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-329745") + (str_to_INTERVAL((char *)"1011894") + (str_to_INTERVAL((char *)"-1028235") + (str_to_INTERVAL((char *)"308100") + (str_to_INTERVAL((char *)"72345") + (str_to_INTERVAL((char *)"-37050") + 2691 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-1524731") + (str_to_INTERVAL((char *)"3236038") + (str_to_INTERVAL((char *)"-1312025") + (str_to_INTERVAL((char *)"-1054300") + (str_to_INTERVAL((char *)"703235") + (str_to_INTERVAL((char *)"-27274") - str_to_INTERVAL((char *)"20943") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-4453540") + (str_to_INTERVAL((char *)"5638464") + (str_to_INTERVAL((char *)"1738360") + (str_to_INTERVAL((char *)"-3894800") + (str_to_INTERVAL((char *)"1133340") + (str_to_INTERVAL((char *)"-130000") - str_to_INTERVAL((char *)"31824") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-8892936") + (str_to_INTERVAL((char *)"4734288") + (str_to_INTERVAL((char *)"4437160") + (str_to_INTERVAL((char *)"1179360") + (str_to_INTERVAL((char *)"-1939080") + (str_to_INTERVAL((char *)"-85072") + 4680 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-12472704") + (str_to_INTERVAL((char *)"-4560192") + (str_to_INTERVAL((char *)"1620640") + (str_to_INTERVAL((char *)"11748800") + (str_to_INTERVAL((char *)"-2453200") + (str_to_INTERVAL((char *)"356512") + str_to_INTERVAL((char *)"16848") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-16518240") + (str_to_INTERVAL((char *)"-17649216") + (str_to_INTERVAL((char *)"-11433440") + (str_to_INTERVAL((char *)"12761920") + (str_to_INTERVAL((char *)"5019600") + (str_to_INTERVAL((char *)"289120") - 5616 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-29715344") + (str_to_INTERVAL((char *)"-26589344") + (str_to_INTERVAL((char *)"-35697840") + (str_to_INTERVAL((char *)"10218880") + (str_to_INTERVAL((char *)"2836000") - str_to_INTERVAL((char *)"136448") * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-49236064") + (str_to_INTERVAL((char *)"-48404160") + (str_to_INTERVAL((char *)"-45593312") + (str_to_INTERVAL((char *)"-4038016") - str_to_INTERVAL((char *)"4407360") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-56767600") + (str_to_INTERVAL((char *)"-66815968") + (str_to_INTERVAL((char *)"-50025296") + (str_to_INTERVAL((char *)"-18933248") - str_to_INTERVAL((char *)"280544") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-44393664") + (str_to_INTERVAL((char *)"-46199296") + (str_to_INTERVAL((char *)"-47663872") + (str_to_INTERVAL((char *)"-1969024") + str_to_INTERVAL((char *)"183936") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-22488576") + (str_to_INTERVAL((char *)"-21766912") + (str_to_INTERVAL((char *)"-378240") + (str_to_INTERVAL((char *)"3207168") + str_to_INTERVAL((char *)"174720") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-7106560") + (str_to_INTERVAL((char *)"2180608") + (str_to_INTERVAL((char *)"9856512") + str_to_INTERVAL((char *)"1512960") * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"2801408") + (str_to_INTERVAL((char *)"7408128") + str_to_INTERVAL((char *)"3333888") * y * y) * y * y + (str_to_INTERVAL((char *)"1763328") * y * y + str_to_INTERVAL((char *)"1207296") + str_to_INTERVAL((char *)"172032") * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * y * y;
	coef[9] = x *  pow( (2 * x + 1),  6) * (str_to_INTERVAL((char *)"5757570") + (str_to_INTERVAL((char *)"-22669920") + (str_to_INTERVAL((char *)"33128550") + (str_to_INTERVAL((char *)"-20966400") + (str_to_INTERVAL((char *)"4463550") + (str_to_INTERVAL((char *)"262080") + str_to_INTERVAL((char *)"24570") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"53737320") + (str_to_INTERVAL((char *)"-166246080") + (str_to_INTERVAL((char *)"176685600") + (str_to_INTERVAL((char *)"-69888000") + (str_to_INTERVAL((char *)"5951400") + (str_to_INTERVAL((char *)"-174720") - str_to_INTERVAL((char *)"65520") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"240308630") + (str_to_INTERVAL((char *)"-506584720") + (str_to_INTERVAL((char *)"229111850") + (str_to_INTERVAL((char *)"96707200") + (str_to_INTERVAL((char *)"-56154950") + (str_to_INTERVAL((char *)"-3187280") - str_to_INTERVAL((char *)"200730") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"690093168") + (str_to_INTERVAL((char *)"-847075168") + (str_to_INTERVAL((char *)"-167224280") + (str_to_INTERVAL((char *)"320144640") + (str_to_INTERVAL((char *)"-3174080") + (str_to_INTERVAL((char *)"6188192") + str_to_INTERVAL((char *)"1047528") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"1347670260") + (str_to_INTERVAL((char *)"-547343584") + (str_to_INTERVAL((char *)"-403808460") + (str_to_INTERVAL((char *)"-909071520") + (str_to_INTERVAL((char *)"578680060") + (str_to_INTERVAL((char *)"30704640") + str_to_INTERVAL((char *)"1448604") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"2142002925") + (str_to_INTERVAL((char *)"943510988") + (str_to_INTERVAL((char *)"347464535") + (str_to_INTERVAL((char *)"-3038716760") + (str_to_INTERVAL((char *)"643660715") + (str_to_INTERVAL((char *)"-588340") - str_to_INTERVAL((char *)"2118063") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"4513252716") + (str_to_INTERVAL((char *)"2365865888") + (str_to_INTERVAL((char *)"2842373560") + (str_to_INTERVAL((char *)"-3591036000") + (str_to_INTERVAL((char *)"-827454100") + (str_to_INTERVAL((char *)"-48064736") - str_to_INTERVAL((char *)"844848") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"10445633596") + (str_to_INTERVAL((char *)"4759224288") + (str_to_INTERVAL((char *)"6801461800") + (str_to_INTERVAL((char *)"-3353155360") + (str_to_INTERVAL((char *)"-822162740") + (str_to_INTERVAL((char *)"15470144") + str_to_INTERVAL((char *)"2151792") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"16982842176") + (str_to_INTERVAL((char *)"12545877408") + (str_to_INTERVAL((char *)"8966920480") + (str_to_INTERVAL((char *)"416253280") + (str_to_INTERVAL((char *)"872896160") + (str_to_INTERVAL((char *)"27574720") - str_to_INTERVAL((char *)"781824") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"17916588608") + (str_to_INTERVAL((char *)"16999722176") + (str_to_INTERVAL((char *)"11650031280") + (str_to_INTERVAL((char *)"4079610880") + (str_to_INTERVAL((char *)"119227680") + (str_to_INTERVAL((char *)"-20216768") - str_to_INTERVAL((char *)"285264") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"12183329152") + (str_to_INTERVAL((char *)"10547834496") + (str_to_INTERVAL((char *)"8333462400") + (str_to_INTERVAL((char *)"-1079312000") + (str_to_INTERVAL((char *)"-449239040") + str_to_INTERVAL((char *)"3991808") * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"4839225472") + (str_to_INTERVAL((char *)"2748445696") + (str_to_INTERVAL((char *)"-4356469760") + (str_to_INTERVAL((char *)"-1370019840") + str_to_INTERVAL((char *)"177516160") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"416955392") + (str_to_INTERVAL((char *)"-2455686144") + (str_to_INTERVAL((char *)"-1581194240") + str_to_INTERVAL((char *)"1061534720") * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-795822080") + (str_to_INTERVAL((char *)"-15736832") + str_to_INTERVAL((char *)"1922600960") * y * y) * y * y + (str_to_INTERVAL((char *)"322154496") + str_to_INTERVAL((char *)"842088448") * y * y + str_to_INTERVAL((char *)"12713984") * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * y * y;
	coef[10] = sqrt2xplus1 *  pow( (2 * x + 1),  5) * (str_to_INTERVAL((char *)"-216216") + (str_to_INTERVAL((char *)"1297296") + (str_to_INTERVAL((char *)"-3243240") + (str_to_INTERVAL((char *)"4324320") + (str_to_INTERVAL((char *)"-3243240") + (str_to_INTERVAL((char *)"1297296") - str_to_INTERVAL((char *)"216216") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"6675669") + (str_to_INTERVAL((char *)"-23729706") + (str_to_INTERVAL((char *)"26621595") + (str_to_INTERVAL((char *)"-2702700") + (str_to_INTERVAL((char *)"-14459445") + (str_to_INTERVAL((char *)"9135126") - str_to_INTERVAL((char *)"1540539") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"90387297") + (str_to_INTERVAL((char *)"-286396110") + (str_to_INTERVAL((char *)"303738435") + (str_to_INTERVAL((char *)"-93873780") + (str_to_INTERVAL((char *)"-32657625") + (str_to_INTERVAL((char *)"21639618") - str_to_INTERVAL((char *)"2837835") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"504555051") + (str_to_INTERVAL((char *)"-1215019806") + (str_to_INTERVAL((char *)"778122345") + (str_to_INTERVAL((char *)"101201100") + (str_to_INTERVAL((char *)"-195870675") + (str_to_INTERVAL((char *)"23417394") + str_to_INTERVAL((char *)"3594591") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"1733193462") + (str_to_INTERVAL((char *)"-2797534740") + (str_to_INTERVAL((char *)"204594390") + (str_to_INTERVAL((char *)"1386785400") + (str_to_INTERVAL((char *)"-586696110") + (str_to_INTERVAL((char *)"42630588") + str_to_INTERVAL((char *)"17027010") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"4111803696") + (str_to_INTERVAL((char *)"-3698590896") + (str_to_INTERVAL((char *)"-1828106280") + (str_to_INTERVAL((char *)"1526965440") + (str_to_INTERVAL((char *)"-75675600") + (str_to_INTERVAL((char *)"79711632") + str_to_INTERVAL((char *)"13621608") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"6927172896") + (str_to_INTERVAL((char *)"-1075127424") + (str_to_INTERVAL((char *)"-2369351040") + (str_to_INTERVAL((char *)"-3294936960") + (str_to_INTERVAL((char *)"1447743360") + (str_to_INTERVAL((char *)"-43286208") - str_to_INTERVAL((char *)"6054048") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"9140195568") + (str_to_INTERVAL((char *)"6477324000") + (str_to_INTERVAL((char *)"2057343120") + (str_to_INTERVAL((char *)"-8412048960") + (str_to_INTERVAL((char *)"-11348400") + (str_to_INTERVAL((char *)"-230789664") - str_to_INTERVAL((char *)"6486480") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"13257553184") + (str_to_INTERVAL((char *)"14589627776") + (str_to_INTERVAL((char *)"13665393440") + (str_to_INTERVAL((char *)"-8075235200") + (str_to_INTERVAL((char *)"-2925509600") + (str_to_INTERVAL((char *)"-102311552") + str_to_INTERVAL((char *)"2594592") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"23239419008") + (str_to_INTERVAL((char *)"23032512640") + (str_to_INTERVAL((char *)"27048641280") + (str_to_INTERVAL((char *)"-3835147520") + (str_to_INTERVAL((char *)"-409951360") + str_to_INTERVAL((char *)"60794496") * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"34240491456") + (str_to_INTERVAL((char *)"37176706624") + (str_to_INTERVAL((char *)"32269259200") + (str_to_INTERVAL((char *)"5873521920") + (str_to_INTERVAL((char *)"2161214720") + str_to_INTERVAL((char *)"4665920") * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"35769454976") + (str_to_INTERVAL((char *)"41615782656") + (str_to_INTERVAL((char *)"34004368000") + (str_to_INTERVAL((char *)"9732024320") + (str_to_INTERVAL((char *)"224727040") - str_to_INTERVAL((char *)"1505280") * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"25786419968") + (str_to_INTERVAL((char *)"26898982400") + (str_to_INTERVAL((char *)"22904989440") + (str_to_INTERVAL((char *)"71463680") + (str_to_INTERVAL((char *)"-359654400") - str_to_INTERVAL((char *)"6585600") * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"12386740224") + (str_to_INTERVAL((char *)"9958639616") + (str_to_INTERVAL((char *)"-2784117760") + (str_to_INTERVAL((char *)"-2435896320") - str_to_INTERVAL((char *)"174336000") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"2815198208") + (str_to_INTERVAL((char *)"-3337180160") + (str_to_INTERVAL((char *)"-5461294080") - str_to_INTERVAL((char *)"765281280") * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-1238425600") + (str_to_INTERVAL((char *)"-2294415360") - str_to_INTERVAL((char *)"1191567360") * y * y) * y * y + (str_to_INTERVAL((char *)"-382156800") * y * y + str_to_INTERVAL((char *)"223887360") + str_to_INTERVAL((char *)"21626880") * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * y * y;
	coef[11] = 2 * x *  pow( (2 * x + 1),  5) * (str_to_INTERVAL((char *)"-6437340") + (str_to_INTERVAL((char *)"25233390") + (str_to_INTERVAL((char *)"-36609300") + (str_to_INTERVAL((char *)"22850100") + (str_to_INTERVAL((char *)"-4668300") + (str_to_INTERVAL((char *)"-319410") - str_to_INTERVAL((char *)"49140") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-72956520") + (str_to_INTERVAL((char *)"235511640") + (str_to_INTERVAL((char *)"-268468200") + (str_to_INTERVAL((char *)"121867200") + (str_to_INTERVAL((char *)"-15561000") + (str_to_INTERVAL((char *)"-425880") + str_to_INTERVAL((char *)"32760") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-389611500") + (str_to_INTERVAL((char *)"936275370") + (str_to_INTERVAL((char *)"-645433500") + (str_to_INTERVAL((char *)"46100700") + (str_to_INTERVAL((char *)"47658900") + (str_to_INTERVAL((char *)"4406250") + str_to_INTERVAL((char *)"603780") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-1316601336") + (str_to_INTERVAL((char *)"2089136016") + (str_to_INTERVAL((char *)"-332074920") + (str_to_INTERVAL((char *)"-563049360") + (str_to_INTERVAL((char *)"123535080") + (str_to_INTERVAL((char *)"227136") - str_to_INTERVAL((char *)"1172616") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-3094296264") + (str_to_INTERVAL((char *)"2570193612") + (str_to_INTERVAL((char *)"802263480") + (str_to_INTERVAL((char *)"282508200") + (str_to_INTERVAL((char *)"-613381320") + (str_to_INTERVAL((char *)"-53578356") - str_to_INTERVAL((char *)"5748552") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-5599177212") + (str_to_INTERVAL((char *)"365800023") + (str_to_INTERVAL((char *)"593931030") + (str_to_INTERVAL((char *)"5242241490") + (str_to_INTERVAL((char *)"-1931045640") + (str_to_INTERVAL((char *)"-70345353") - str_to_INTERVAL((char *)"911538") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-10381912476") + (str_to_INTERVAL((char *)"-4480849620") + (str_to_INTERVAL((char *)"-3526935060") + (str_to_INTERVAL((char *)"10420002480") + (str_to_INTERVAL((char *)"-425930940") + (str_to_INTERVAL((char *)"72350196") + str_to_INTERVAL((char *)"6559740") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-22696423168") + (str_to_INTERVAL((char *)"-10696050172") + (str_to_INTERVAL((char *)"-13231579600") + (str_to_INTERVAL((char *)"11393985880") + (str_to_INTERVAL((char *)"2824922320") + (str_to_INTERVAL((char *)"94073428") - str_to_INTERVAL((char *)"1448928") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-43140342720") + (str_to_INTERVAL((char *)"-25253418016") + (str_to_INTERVAL((char *)"-24730013360") + (str_to_INTERVAL((char *)"6685361280") + (str_to_INTERVAL((char *)"748531360") + (str_to_INTERVAL((char *)"-74930272") - str_to_INTERVAL((char *)"2613552") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-58079935488") + (str_to_INTERVAL((char *)"-47142184960") + (str_to_INTERVAL((char *)"-32886348000") + (str_to_INTERVAL((char *)"-5779045440") + (str_to_INTERVAL((char *)"-2127383360") + (str_to_INTERVAL((char *)"-20138880") + str_to_INTERVAL((char *)"2012064") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-53170322240") + (str_to_INTERVAL((char *)"-48976129920") + (str_to_INTERVAL((char *)"-34883598720") + (str_to_INTERVAL((char *)"-7655996160") + (str_to_INTERVAL((char *)"496288960") + (str_to_INTERVAL((char *)"30407296") - str_to_INTERVAL((char *)"900864") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-32236535040") + (str_to_INTERVAL((char *)"-26010629760") + (str_to_INTERVAL((char *)"-13513344000") + (str_to_INTERVAL((char *)"3809184000") + (str_to_INTERVAL((char *)"619191040") - str_to_INTERVAL((char *)"24164992") * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-11350358016") + (str_to_INTERVAL((char *)"-3570837504") + (str_to_INTERVAL((char *)"9950607360") + (str_to_INTERVAL((char *)"1041991680") - str_to_INTERVAL((char *)"533795840") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-424595456") + (str_to_INTERVAL((char *)"4522921984") + (str_to_INTERVAL((char *)"353382400") - str_to_INTERVAL((char *)"1832089600") * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"1105459200") + (str_to_INTERVAL((char *)"-1308397568") - str_to_INTERVAL((char *)"2757591040") * y * y) * y * y + (str_to_INTERVAL((char *)"-694259712") - str_to_INTERVAL((char *)"603226112") * y * y + str_to_INTERVAL((char *)"93323264") * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * y * y;
	coef[12] = (-1)*sqrt2xplus1 *  pow( (2 * x + 1),  4) * (str_to_INTERVAL((char *)"-360360") + (str_to_INTERVAL((char *)"2162160") + (str_to_INTERVAL((char *)"-5405400") + (str_to_INTERVAL((char *)"7207200") + (str_to_INTERVAL((char *)"-5405400") + (str_to_INTERVAL((char *)"2162160") - str_to_INTERVAL((char *)"360360") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"10405395") + (str_to_INTERVAL((char *)"-35225190") + (str_to_INTERVAL((char *)"33558525") + (str_to_INTERVAL((char *)"9909900") + (str_to_INTERVAL((char *)"-34909875") + (str_to_INTERVAL((char *)"19549530") - str_to_INTERVAL((char *)"3288285") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"172897725") + (str_to_INTERVAL((char *)"-556425870") + (str_to_INTERVAL((char *)"594969375") + (str_to_INTERVAL((char *)"-165465300") + (str_to_INTERVAL((char *)"-102627525") + (str_to_INTERVAL((char *)"66516450") - str_to_INTERVAL((char *)"9864855") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"1142216075") + (str_to_INTERVAL((char *)"-2979686710") + (str_to_INTERVAL((char *)"2309332025") + (str_to_INTERVAL((char *)"-144244100") + (str_to_INTERVAL((char *)"-435309875") + (str_to_INTERVAL((char *)"111161050") - str_to_INTERVAL((char *)"3468465") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"4570505940") + (str_to_INTERVAL((char *)"-8712623920") + (str_to_INTERVAL((char *)"2934731800") + (str_to_INTERVAL((char *)"2648646000") + (str_to_INTERVAL((char *)"-1630729100") + (str_to_INTERVAL((char *)"149108960") + str_to_INTERVAL((char *)"40360320") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"12630317700") + (str_to_INTERVAL((char *)"-15489433960") + (str_to_INTERVAL((char *)"-2364862500") + (str_to_INTERVAL((char *)"7167560400") + (str_to_INTERVAL((char *)"-2081779700") + (str_to_INTERVAL((char *)"274954680") + str_to_INTERVAL((char *)"79459380") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"25134104800") + (str_to_INTERVAL((char *)"-14051597280") + (str_to_INTERVAL((char *)"-9891999600") + (str_to_INTERVAL((char *)"-442814400") + (str_to_INTERVAL((char *)"2101380400") + (str_to_INTERVAL((char *)"191679040") + str_to_INTERVAL((char *)"35280000") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"37269474480") + (str_to_INTERVAL((char *)"7694208480") + (str_to_INTERVAL((char *)"-3715899600") + (str_to_INTERVAL((char *)"-25126617600") + (str_to_INTERVAL((char *)"4747624000") + (str_to_INTERVAL((char *)"-527054080") - str_to_INTERVAL((char *)"30885120") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"48579220000") + (str_to_INTERVAL((char *)"46715808640") + (str_to_INTERVAL((char *)"30900160000") + (str_to_INTERVAL((char *)"-41252008000") + (str_to_INTERVAL((char *)"-4649782800") + (str_to_INTERVAL((char *)"-933138400") - str_to_INTERVAL((char *)"17315760") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"74613374208") + (str_to_INTERVAL((char *)"85924828672") + (str_to_INTERVAL((char *)"91404666560") + (str_to_INTERVAL((char *)"-32745229440") + (str_to_INTERVAL((char *)"-10500523680") + (str_to_INTERVAL((char *)"-255844288") + str_to_INTERVAL((char *)"8224608") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"123910543904") + (str_to_INTERVAL((char *)"134436732672") + (str_to_INTERVAL((char *)"143319069440") + (str_to_INTERVAL((char *)"-3895275200") + (str_to_INTERVAL((char *)"1772034640") + (str_to_INTERVAL((char *)"217557216") + str_to_INTERVAL((char *)"807408") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"165151257856") + (str_to_INTERVAL((char *)"189803512960") + (str_to_INTERVAL((char *)"162564756480") + (str_to_INTERVAL((char *)"35413415680") + (str_to_INTERVAL((char *)"8236574720") + (str_to_INTERVAL((char *)"36059264") - str_to_INTERVAL((char *)"47040") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"158115468160") + (str_to_INTERVAL((char *)"184021356800") + (str_to_INTERVAL((char *)"152700548480") + (str_to_INTERVAL((char *)"34122771200") + (str_to_INTERVAL((char *)"64868800") + (str_to_INTERVAL((char *)"-53105280") - str_to_INTERVAL((char *)"1646400") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"106091379200") + (str_to_INTERVAL((char *)"108851737600") + (str_to_INTERVAL((char *)"73597301760") + (str_to_INTERVAL((char *)"-5067957760") + (str_to_INTERVAL((char *)"-2337588480") - str_to_INTERVAL((char *)"38246400") * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"46802427136") + (str_to_INTERVAL((char *)"28700701184") + (str_to_INTERVAL((char *)"-20469510400") + (str_to_INTERVAL((char *)"-9526394880") - str_to_INTERVAL((char *)"711544320") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"8205252608") + (str_to_INTERVAL((char *)"-13069592576") + (str_to_INTERVAL((char *)"-16246502400") - str_to_INTERVAL((char *)"1988597760") * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-2837122048") + (str_to_INTERVAL((char *)"-1988904960") - str_to_INTERVAL((char *)"2659630080") * y * y) * y * y + (str_to_INTERVAL((char *)"-316907520") * y * y + str_to_INTERVAL((char *)"1387253760") + str_to_INTERVAL((char *)"65372160") * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * y * y;
	coef[13] = (-1)*x *  pow( (2 * x + 1),  4) * (str_to_INTERVAL((char *)"-21891870") + (str_to_INTERVAL((char *)"85405320") + (str_to_INTERVAL((char *)"-122972850") + (str_to_INTERVAL((char *)"75675600") + (str_to_INTERVAL((char *)"-14864850") + (str_to_INTERVAL((char *)"-1081080") - str_to_INTERVAL((char *)"270270") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-291891600") + (str_to_INTERVAL((char *)"967926960") + (str_to_INTERVAL((char *)"-1147746600") + (str_to_INTERVAL((char *)"554954400") + (str_to_INTERVAL((char *)"-79279200") + (str_to_INTERVAL((char *)"-3603600") - str_to_INTERVAL((char *)"360360") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-1823777010") + (str_to_INTERVAL((char *)"4770476040") + (str_to_INTERVAL((char *)"-3975897750") + (str_to_INTERVAL((char *)"955455600") + (str_to_INTERVAL((char *)"57131250") + (str_to_INTERVAL((char *)"13243560") + str_to_INTERVAL((char *)"3368310") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-7158419928") + (str_to_INTERVAL((char *)"13481899728") + (str_to_INTERVAL((char *)"-5485526640") + (str_to_INTERVAL((char *)"-1585937760") + (str_to_INTERVAL((char *)"714770760") + (str_to_INTERVAL((char *)"33021648") + str_to_INTERVAL((char *)"192192") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-19680488100") + (str_to_INTERVAL((char *)"23196083184") + (str_to_INTERVAL((char *)"395309340") + (str_to_INTERVAL((char *)"-2868840960") + (str_to_INTERVAL((char *)"-1196971020") + (str_to_INTERVAL((char *)"-197166960") - str_to_INTERVAL((char *)"37114284") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-41013737595") + (str_to_INTERVAL((char *)"19704655992") + (str_to_INTERVAL((char *)"7644051795") + (str_to_INTERVAL((char *)"19074214320") + (str_to_INTERVAL((char *)"-10340337525") + (str_to_INTERVAL((char *)"-637146600") - str_to_INTERVAL((char *)"53316387") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-76266541440") + (str_to_INTERVAL((char *)"-10942229784") + (str_to_INTERVAL((char *)"-6150629340") + (str_to_INTERVAL((char *)"68941804560") + (str_to_INTERVAL((char *)"-13881256680") + (str_to_INTERVAL((char *)"-168606360") + str_to_INTERVAL((char *)"18488484") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-153364991160") + (str_to_INTERVAL((char *)"-66416191392") + (str_to_INTERVAL((char *)"-64328019060") + (str_to_INTERVAL((char *)"106275877920") + (str_to_INTERVAL((char *)"7375830240") + (str_to_INTERVAL((char *)"949512000") + str_to_INTERVAL((char *)"38361132") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-306771539808") + (str_to_INTERVAL((char *)"-162658796928") + (str_to_INTERVAL((char *)"-168196740480") + (str_to_INTERVAL((char *)"96440643360") + (str_to_INTERVAL((char *)"21687027840") + (str_to_INTERVAL((char *)"384951072") - str_to_INTERVAL((char *)"20139936") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-491397202512") + (str_to_INTERVAL((char *)"-337107144192") + (str_to_INTERVAL((char *)"-275751497760") + (str_to_INTERVAL((char *)"22397867520") + (str_to_INTERVAL((char *)"-3082779600") + (str_to_INTERVAL((char *)"-546440448") - str_to_INTERVAL((char *)"2659200") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-569288399488") + (str_to_INTERVAL((char *)"-485318908288") + (str_to_INTERVAL((char *)"-338131271680") + (str_to_INTERVAL((char *)"-65896607360") + (str_to_INTERVAL((char *)"-12287056640") + (str_to_INTERVAL((char *)"53049088") + str_to_INTERVAL((char *)"5469312") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-462721925120") + (str_to_INTERVAL((char *)"-414928456704") + (str_to_INTERVAL((char *)"-277834065280") + (str_to_INTERVAL((char *)"-38057589760") + (str_to_INTERVAL((char *)"5917693440") + (str_to_INTERVAL((char *)"44723200") - str_to_INTERVAL((char *)"9201024") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-253560235008") + (str_to_INTERVAL((char *)"-186751924224") + (str_to_INTERVAL((char *)"-60476892160") + (str_to_INTERVAL((char *)"25650109440") + (str_to_INTERVAL((char *)"1040087040") - str_to_INTERVAL((char *)"164713472") * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-79737700864") + (str_to_INTERVAL((char *)"-13432987648") + (str_to_INTERVAL((char *)"57443445760") + (str_to_INTERVAL((char *)"-1115668480") - str_to_INTERVAL((char *)"3120140800") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-2075131904") + (str_to_INTERVAL((char *)"20287180800") + (str_to_INTERVAL((char *)"-9330851840") - str_to_INTERVAL((char *)"7744163840") * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"4130955264") + (str_to_INTERVAL((char *)"-10510516224") - str_to_INTERVAL((char *)"10436034560") * y * y) * y * y + (str_to_INTERVAL((char *)"-83034112") * y * y - str_to_INTERVAL((char *)"3988111360") + str_to_INTERVAL((char *)"326631424") * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * y * y;
	coef[14] = sqrt2xplus1 *  pow( (2 * x + 1),  3) * (str_to_INTERVAL((char *)"-463320") + (str_to_INTERVAL((char *)"2779920") + (str_to_INTERVAL((char *)"-6949800") + (str_to_INTERVAL((char *)"9266400") + (str_to_INTERVAL((char *)"-6949800") + (str_to_INTERVAL((char *)"2779920") - str_to_INTERVAL((char *)"463320") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"12451725") + (str_to_INTERVAL((char *)"-39729690") + (str_to_INTERVAL((char *)"29247075") + (str_to_INTERVAL((char *)"31274100") + (str_to_INTERVAL((char *)"-58783725") + (str_to_INTERVAL((char *)"30694950") - str_to_INTERVAL((char *)"5154435") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"249053805") + (str_to_INTERVAL((char *)"-805983750") + (str_to_INTERVAL((char *)"851253975") + (str_to_INTERVAL((char *)"-187258500") + (str_to_INTERVAL((char *)"-221717925") + (str_to_INTERVAL((char *)"135791370") - str_to_INTERVAL((char *)"21138975") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"1913157675") + (str_to_INTERVAL((char *)"-5261835150") + (str_to_INTERVAL((char *)"4499062425") + (str_to_INTERVAL((char *)"-610938900") + (str_to_INTERVAL((char *)"-823583475") + (str_to_INTERVAL((char *)"313963650") - str_to_INTERVAL((char *)"29826225") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"8813491830") + (str_to_INTERVAL((char *)"-18863996580") + (str_to_INTERVAL((char *)"9711508950") + (str_to_INTERVAL((char *)"3034488600") + (str_to_INTERVAL((char *)"-3216019950") + (str_to_INTERVAL((char *)"477554220") + str_to_INTERVAL((char *)"42972930") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"27991709460") + (str_to_INTERVAL((char *)"-42318876600") + (str_to_INTERVAL((char *)"4505915700") + (str_to_INTERVAL((char *)"16026238800") + (str_to_INTERVAL((char *)"-6869877300") + (str_to_INTERVAL((char *)"736936200") + str_to_INTERVAL((char *)"205945740") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"64623340920") + (str_to_INTERVAL((char *)"-57846365520") + (str_to_INTERVAL((char *)"-18552582600") + (str_to_INTERVAL((char *)"17890144800") + (str_to_INTERVAL((char *)"-2799514200") + (str_to_INTERVAL((char *)"946463280") + str_to_INTERVAL((char *)"249396840") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"110679589680") + (str_to_INTERVAL((char *)"-25790603040") + (str_to_INTERVAL((char *)"-28486712400") + (str_to_INTERVAL((char *)"-33301276800") + (str_to_INTERVAL((char *)"11063214000") + (str_to_INTERVAL((char *)"-191759520") + str_to_INTERVAL((char *)"51297840") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"149447172320") + (str_to_INTERVAL((char *)"80634186560") + (str_to_INTERVAL((char *)"34687527200") + (str_to_INTERVAL((char *)"-116924801600") + (str_to_INTERVAL((char *)"6497982400") + (str_to_INTERVAL((char *)"-2523753920") - str_to_INTERVAL((char *)"100575360") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"197115754944") + (str_to_INTERVAL((char *)"227150873216") + (str_to_INTERVAL((char *)"202227732160") + (str_to_INTERVAL((char *)"-146255556480") + (str_to_INTERVAL((char *)"-24460658240") + (str_to_INTERVAL((char *)"-2736219904") - str_to_INTERVAL((char *)"36616896") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"311108684736") + (str_to_INTERVAL((char *)"377635557632") + (str_to_INTERVAL((char *)"420525388800") + (str_to_INTERVAL((char *)"-89049330240") + (str_to_INTERVAL((char *)"-25766615840") + (str_to_INTERVAL((char *)"-433166976") + str_to_INTERVAL((char *)"23224608") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"486252411520") + (str_to_INTERVAL((char *)"564130457856") + (str_to_INTERVAL((char *)"572673788160") + (str_to_INTERVAL((char *)"31965523200") + (str_to_INTERVAL((char *)"14645039680") + (str_to_INTERVAL((char *)"683885440") + str_to_INTERVAL((char *)"6352704") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"594629050368") + (str_to_INTERVAL((char *)"707686417920") + (str_to_INTERVAL((char *)"610766737920") + (str_to_INTERVAL((char *)"135904212480") + (str_to_INTERVAL((char *)"22938497920") + (str_to_INTERVAL((char *)"12668672") - str_to_INTERVAL((char *)"9256320") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"527319390720") + (str_to_INTERVAL((char *)"612863923200") + (str_to_INTERVAL((char *)"493711464960") + (str_to_INTERVAL((char *)"83924705280") + (str_to_INTERVAL((char *)"-4132424960") + (str_to_INTERVAL((char *)"-281264640") - str_to_INTERVAL((char *)"6163200") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"329847705088") + (str_to_INTERVAL((char *)"323932413952") + (str_to_INTERVAL((char *)"166646172160") + (str_to_INTERVAL((char *)"-27005839360") + (str_to_INTERVAL((char *)"-7548334080") - str_to_INTERVAL((char *)"91315200") * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"132359334912") + (str_to_INTERVAL((char *)"63391750144") + (str_to_INTERVAL((char *)"-70192092160") + (str_to_INTERVAL((char *)"-21146480640") - str_to_INTERVAL((char *)"1514342400") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"18721105920") + (str_to_INTERVAL((char *)"-25948479488") + (str_to_INTERVAL((char *)"-28857845760") - str_to_INTERVAL((char *)"3143331840") * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-5933707264") + (str_to_INTERVAL((char *)"5361623040") - str_to_INTERVAL((char *)"4038021120") * y * y) * y * y + (str_to_INTERVAL((char *)"2697707520") + str_to_INTERVAL((char *)"271810560") * y * y + str_to_INTERVAL((char *)"21626880") * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * y * y;
	coef[15] = 2 * x *  pow( (2 * x + 1),  3) * (str_to_INTERVAL((char *)"-14427270") + (str_to_INTERVAL((char *)"55997370") + (str_to_INTERVAL((char *)"-79987050") + (str_to_INTERVAL((char *)"48519900") + (str_to_INTERVAL((char *)"-9202050") + (str_to_INTERVAL((char *)"-630630") - str_to_INTERVAL((char *)"270270") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-221218140") + (str_to_INTERVAL((char *)"746631600") + (str_to_INTERVAL((char *)"-906519900") + (str_to_INTERVAL((char *)"452852400") + (str_to_INTERVAL((char *)"-67481700") + (str_to_INTERVAL((char *)"-3363360") - str_to_INTERVAL((char *)"900900") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-1588352810") + (str_to_INTERVAL((char *)"4401643510") + (str_to_INTERVAL((char *)"-4081053350") + (str_to_INTERVAL((char *)"1320654500") + (str_to_INTERVAL((char *)"-60608350") + (str_to_INTERVAL((char *)"5365910") + str_to_INTERVAL((char *)"2350590") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-7145373708") + (str_to_INTERVAL((char *)"15149669128") + (str_to_INTERVAL((char *)"-8758833820") + (str_to_INTERVAL((char *)"179820960") + (str_to_INTERVAL((char *)"526290380") + (str_to_INTERVAL((char *)"42092248") + str_to_INTERVAL((char *)"6334812") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-22581105636") + (str_to_INTERVAL((char *)"33198173740") + (str_to_INTERVAL((char *)"-6924680580") + (str_to_INTERVAL((char *)"-3972863160") + (str_to_INTERVAL((char *)"143236100") + (str_to_INTERVAL((char *)"-94879284") - str_to_INTERVAL((char *)"30429180") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-53844846175") + (str_to_INTERVAL((char *)"44424370305") + (str_to_INTERVAL((char *)"5657842905") + (str_to_INTERVAL((char *)"8378703870") + (str_to_INTERVAL((char *)"-8096838565") + (str_to_INTERVAL((char *)"-720101215") - str_to_INTERVAL((char *)"102859125") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-107314578750") + (str_to_INTERVAL((char *)"20943677760") + (str_to_INTERVAL((char *)"7311168570") + (str_to_INTERVAL((char *)"68600839320") + (str_to_INTERVAL((char *)"-21905386210") + (str_to_INTERVAL((char *)"-939825800") - str_to_INTERVAL((char *)"68456970") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-208231938360") + (str_to_INTERVAL((char *)"-56163834840") + (str_to_INTERVAL((char *)"-46043474580") + (str_to_INTERVAL((char *)"156697422480") + (str_to_INTERVAL((char *)"-12261045040") + (str_to_INTERVAL((char *)"586019160") + str_to_INTERVAL((char *)"51136380") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-412586071344") + (str_to_INTERVAL((char *)"-197509472736") + (str_to_INTERVAL((char *)"-187700446680") + (str_to_INTERVAL((char *)"197507170560") + (str_to_INTERVAL((char *)"24643345760") + (str_to_INTERVAL((char *)"1677426848") + str_to_INTERVAL((char *)"32389032") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-730033592592") + (str_to_INTERVAL((char *)"-444798663696") + (str_to_INTERVAL((char *)"-394717302240") + (str_to_INTERVAL((char *)"134813736480") + (str_to_INTERVAL((char *)"25749710960") + (str_to_INTERVAL((char *)"181538800") - str_to_INTERVAL((char *)"25717824") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-1010662708384") + (str_to_INTERVAL((char *)"-767541756544") + (str_to_INTERVAL((char *)"-577962999040") + (str_to_INTERVAL((char *)"-18461456000") + (str_to_INTERVAL((char *)"-12979754400") + (str_to_INTERVAL((char *)"-577177856") + str_to_INTERVAL((char *)"5298624") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-1032397751552") + (str_to_INTERVAL((char *)"-904313886720") + (str_to_INTERVAL((char *)"-618231341440") + (str_to_INTERVAL((char *)"-111386437120") + (str_to_INTERVAL((char *)"-11804846080") + (str_to_INTERVAL((char *)"155542272") - str_to_INTERVAL((char *)"1787520") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-756255264256") + (str_to_INTERVAL((char *)"-659531858944") + (str_to_INTERVAL((char *)"-399092983040") + (str_to_INTERVAL((char *)"-34109327360") + (str_to_INTERVAL((char *)"7607216640") + (str_to_INTERVAL((char *)"-68189184") - str_to_INTERVAL((char *)"8822016") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-377575587328") + (str_to_INTERVAL((char *)"-255507659264") + (str_to_INTERVAL((char *)"-50784824320") + (str_to_INTERVAL((char *)"27712947200") + (str_to_INTERVAL((char *)"-1146826240") - str_to_INTERVAL((char *)"141019648") * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-107378027520") + (str_to_INTERVAL((char *)"-12940746752") + (str_to_INTERVAL((char *)"55245680640") + (str_to_INTERVAL((char *)"-6439137280") - str_to_INTERVAL((char *)"2507801600") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-3167653888") + (str_to_INTERVAL((char *)"14799593472") + (str_to_INTERVAL((char *)"-18451845120") - str_to_INTERVAL((char *)"5146869760") * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"2561908736") + (str_to_INTERVAL((char *)"-12235816960") - str_to_INTERVAL((char *)"6291619840") * y * y) * y * y + (str_to_INTERVAL((char *)"1210580992") * y * y - str_to_INTERVAL((char *)"4209098752") + str_to_INTERVAL((char *)"200278016") * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * y * y;
	coef[16] = (-1)*sqrt2xplus1 *  pow( (2 * x + 1),  2) * (str_to_INTERVAL((char *)"-463320") + (str_to_INTERVAL((char *)"2779920") + (str_to_INTERVAL((char *)"-6949800") + (str_to_INTERVAL((char *)"9266400") + (str_to_INTERVAL((char *)"-6949800") + (str_to_INTERVAL((char *)"2779920") - str_to_INTERVAL((char *)"463320") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"11525085") + (str_to_INTERVAL((char *)"-34169850") + (str_to_INTERVAL((char *)"15347475") + (str_to_INTERVAL((char *)"49806900") + (str_to_INTERVAL((char *)"-72683325") + (str_to_INTERVAL((char *)"36254790") - str_to_INTERVAL((char *)"6081075") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"273957255") + (str_to_INTERVAL((char *)"-885443130") + (str_to_INTERVAL((char *)"909748125") + (str_to_INTERVAL((char *)"-124710300") + (str_to_INTERVAL((char *)"-339285375") + (str_to_INTERVAL((char *)"197181270") - str_to_INTERVAL((char *)"31447845") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"2411265285") + (str_to_INTERVAL((char *)"-6873802650") + (str_to_INTERVAL((char *)"6201570375") + (str_to_INTERVAL((char *)"-985455900") + (str_to_INTERVAL((char *)"-1267019325") + (str_to_INTERVAL((char *)"585546390") - str_to_INTERVAL((char *)"72104175") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"12639807180") + (str_to_INTERVAL((char *)"-29387666880") + (str_to_INTERVAL((char *)"18709633800") + (str_to_INTERVAL((char *)"1812610800") + (str_to_INTERVAL((char *)"-4863186900") + (str_to_INTERVAL((char *)"1105481520") - str_to_INTERVAL((char *)"16679520") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"45618693120") + (str_to_INTERVAL((char *)"-80046869760") + (str_to_INTERVAL((char *)"23928933600") + (str_to_INTERVAL((char *)"22095216000") + (str_to_INTERVAL((char *)"-13301917200") + (str_to_INTERVAL((char *)"1692044640") + str_to_INTERVAL((char *)"291891600") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"120410377920") + (str_to_INTERVAL((char *)"-142467782400") + (str_to_INTERVAL((char *)"-9275227200") + (str_to_INTERVAL((char *)"50130043200") + (str_to_INTERVAL((char *)"-16795191600") + (str_to_INTERVAL((char *)"2404432800") + str_to_INTERVAL((char *)"660214800") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"237373306560") + (str_to_INTERVAL((char *)"-141303634560") + (str_to_INTERVAL((char *)"-63202161600") + (str_to_INTERVAL((char *)"3790958400") + (str_to_INTERVAL((char *)"4184571600") + (str_to_INTERVAL((char *)"1653458400") + str_to_INTERVAL((char *)"549018000") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"356254415520") + (str_to_INTERVAL((char *)"28746427200") + (str_to_INTERVAL((char *)"-13860285600") + (str_to_INTERVAL((char *)"-179464516800") + (str_to_INTERVAL((char *)"27431893200") + (str_to_INTERVAL((char *)"-2881039200") + str_to_INTERVAL((char *)"6814800") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"448068490176") + (str_to_INTERVAL((char *)"379514196864") + (str_to_INTERVAL((char *)"285826292160") + (str_to_INTERVAL((char *)"-372521717760") + (str_to_INTERVAL((char *)"-9257196960") + (str_to_INTERVAL((char *)"-7680987456") - str_to_INTERVAL((char *)"238960224") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"603880168800") + (str_to_INTERVAL((char *)"791478132672") + (str_to_INTERVAL((char *)"833920947360") + (str_to_INTERVAL((char *)"-374942827200") + (str_to_INTERVAL((char *)"-72938136720") + (str_to_INTERVAL((char *)"-6008228640") - str_to_INTERVAL((char *)"58022352") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"963320298880") + (str_to_INTERVAL((char *)"1231439443456") + (str_to_INTERVAL((char *)"1404449812480") + (str_to_INTERVAL((char *)"-150422884480") + (str_to_INTERVAL((char *)"-40141518400") + (str_to_INTERVAL((char *)"-232526080") + str_to_INTERVAL((char *)"64076544") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"1423566364928") + (str_to_INTERVAL((char *)"1732121441280") + (str_to_INTERVAL((char *)"1734441687040") + (str_to_INTERVAL((char *)"191317212160") + (str_to_INTERVAL((char *)"52453512960") + (str_to_INTERVAL((char *)"1534456832") - str_to_INTERVAL((char *)"115200") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"1619032845312") + (str_to_INTERVAL((char *)"1964858600448") + (str_to_INTERVAL((char *)"1710337100800") + (str_to_INTERVAL((char *)"360648238080") + (str_to_INTERVAL((char *)"43787612160") + (str_to_INTERVAL((char *)"-372232192") - str_to_INTERVAL((char *)"32790528") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"1342068206848") + (str_to_INTERVAL((char *)"1539940850176") + (str_to_INTERVAL((char *)"1172912295680") + (str_to_INTERVAL((char *)"146377292800") + (str_to_INTERVAL((char *)"-16665032960") + (str_to_INTERVAL((char *)"-613940736") - str_to_INTERVAL((char *)"6974208") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"784098581504") + (str_to_INTERVAL((char *)"726334009344") + (str_to_INTERVAL((char *)"279086520320") + (str_to_INTERVAL((char *)"-67733217280") + (str_to_INTERVAL((char *)"-12697666560") - str_to_INTERVAL((char *)"104116224") * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"286297098240") + (str_to_INTERVAL((char *)"118117466112") + (str_to_INTERVAL((char *)"-142720921600") + (str_to_INTERVAL((char *)"-25934807040") - str_to_INTERVAL((char *)"1650186240") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"33134411776") + (str_to_INTERVAL((char *)"-31772549120") + (str_to_INTERVAL((char *)"-27360215040") - str_to_INTERVAL((char *)"3116605440") * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-10329788416") + (str_to_INTERVAL((char *)"18372796416") - str_to_INTERVAL((char *)"3649474560") * y * y) * y * y + (str_to_INTERVAL((char *)"672301056") * y * y + str_to_INTERVAL((char *)"4364746752") + str_to_INTERVAL((char *)"87687168") * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * y * y;
	coef[17] = (-1)*x *  pow( (2 * x + 1),  2) * (str_to_INTERVAL((char *)"-29768310") + (str_to_INTERVAL((char *)"114903360") + (str_to_INTERVAL((char *)"-162741150") + (str_to_INTERVAL((char *)"97297200") + (str_to_INTERVAL((char *)"-17953650") + (str_to_INTERVAL((char *)"-926640") - str_to_INTERVAL((char *)"810810") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-515984040") + (str_to_INTERVAL((char *)"1761851520") + (str_to_INTERVAL((char *)"-2169882000") + (str_to_INTERVAL((char *)"1102701600") + (str_to_INTERVAL((char *)"-167567400") + (str_to_INTERVAL((char *)"-6795360") - str_to_INTERVAL((char *)"4324320") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-4193813250") + (str_to_INTERVAL((char *)"12104951760") + (str_to_INTERVAL((char *)"-11994379650") + (str_to_INTERVAL((char *)"4456782000") + (str_to_INTERVAL((char *)"-380362950") + (str_to_INTERVAL((char *)"6019200") + str_to_INTERVAL((char *)"802890") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-21355719264") + (str_to_INTERVAL((char *)"49275303264") + (str_to_INTERVAL((char *)"-34454775960") + (str_to_INTERVAL((char *)"5579750880") + (str_to_INTERVAL((char *)"799975440") + (str_to_INTERVAL((char *)"125731584") + str_to_INTERVAL((char *)"29734056") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-76552638732") + (str_to_INTERVAL((char *)"131117210784") + (str_to_INTERVAL((char *)"-49794578100") + (str_to_INTERVAL((char *)"-7582550400") + (str_to_INTERVAL((char *)"2323868940") + (str_to_INTERVAL((char *)"-26723808") - str_to_INTERVAL((char *)"40572684") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-206867305863") + (str_to_INTERVAL((char *)"230735262060") + (str_to_INTERVAL((char *)"-16191721035") + (str_to_INTERVAL((char *)"164025720") + (str_to_INTERVAL((char *)"-15638058405") + (str_to_INTERVAL((char *)"-1997505252") - str_to_INTERVAL((char *)"397737225") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-453394153836") + (str_to_INTERVAL((char *)"233899383456") + (str_to_INTERVAL((char *)"41507364960") + (str_to_INTERVAL((char *)"171015921360") + (str_to_INTERVAL((char *)"-75812805900") + (str_to_INTERVAL((char *)"-5043633264") - str_to_INTERVAL((char *)"650135016") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-897295446348") + (str_to_INTERVAL((char *)"-17388849984") + (str_to_INTERVAL((char *)"-52946647440") + (str_to_INTERVAL((char *)"591729390240") + (str_to_INTERVAL((char *)"-110094661740") + (str_to_INTERVAL((char *)"-2248221792") - str_to_INTERVAL((char *)"200685816") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-1748618691168") + (str_to_INTERVAL((char *)"-636528982944") + (str_to_INTERVAL((char *)"-551128573440") + (str_to_INTERVAL((char *)"1027580202720") + (str_to_INTERVAL((char *)"5618464320") + (str_to_INTERVAL((char *)"6886293120") + str_to_INTERVAL((char *)"277768992") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-3228693258432") + (str_to_INTERVAL((char *)"-1754554221120") + (str_to_INTERVAL((char *)"-1551807300720") + (str_to_INTERVAL((char *)"1056006067200") + (str_to_INTERVAL((char *)"154803122400") + (str_to_INTERVAL((char *)"7941880128") + str_to_INTERVAL((char *)"65376528") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-5046802861312") + (str_to_INTERVAL((char *)"-3440168576128") + (str_to_INTERVAL((char *)"-2778722008960") + (str_to_INTERVAL((char *)"481343632000") + (str_to_INTERVAL((char *)"74619272320") + (str_to_INTERVAL((char *)"-138316544") - str_to_INTERVAL((char *)"81545472") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-6159535344384") + (str_to_INTERVAL((char *)"-4997117793280") + (str_to_INTERVAL((char *)"-3605210109440") + (str_to_INTERVAL((char *)"-320877480960") + (str_to_INTERVAL((char *)"-79707104000") + (str_to_INTERVAL((char *)"-1618183168") + str_to_INTERVAL((char *)"29121792") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-5641468818432") + (str_to_INTERVAL((char *)"-5003832076288") + (str_to_INTERVAL((char *)"-3310552089600") + (str_to_INTERVAL((char *)"-526881392640") + (str_to_INTERVAL((char *)"-34156436480") + (str_to_INTERVAL((char *)"664283136") - str_to_INTERVAL((char *)"14945280") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-3763986465024") + (str_to_INTERVAL((char *)"-3187508039680") + (str_to_INTERVAL((char *)"-1734268349440") + (str_to_INTERVAL((char *)"-96256010240") + (str_to_INTERVAL((char *)"24390337280") + (str_to_INTERVAL((char *)"-331687936") - str_to_INTERVAL((char *)"15128064") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-1724349502464") + (str_to_INTERVAL((char *)"-1090029930496") + (str_to_INTERVAL((char *)"-151990599680") + (str_to_INTERVAL((char *)"80696678400") + (str_to_INTERVAL((char *)"-7400381440") - str_to_INTERVAL((char *)"237166592") * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-449964966912") + (str_to_INTERVAL((char *)"-57836855296") + (str_to_INTERVAL((char *)"142162421760") + (str_to_INTERVAL((char *)"-32195420160") - str_to_INTERVAL((char *)"4037923840") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-17848295424") + (str_to_INTERVAL((char *)"26864869376") + (str_to_INTERVAL((char *)"-76870574080") - str_to_INTERVAL((char *)"7495393280") * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"3220389888") + (str_to_INTERVAL((char *)"-41663856640") - str_to_INTERVAL((char *)"7467991040") * y * y) * y * y + (str_to_INTERVAL((char *)"-12902596608") + str_to_INTERVAL((char *)"3828482048") * y * y + str_to_INTERVAL((char *)"900726784") * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * y * y;
	coef[18] = sqrt2xplus1 *  pow( (2 * x + 1),  2) * (str_to_INTERVAL((char *)"-360360") + (str_to_INTERVAL((char *)"2162160") + (str_to_INTERVAL((char *)"-5405400") + (str_to_INTERVAL((char *)"7207200") + (str_to_INTERVAL((char *)"-5405400") + (str_to_INTERVAL((char *)"2162160") - str_to_INTERVAL((char *)"360360") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"8963955") + (str_to_INTERVAL((char *)"-26576550") + (str_to_INTERVAL((char *)"11936925") + (str_to_INTERVAL((char *)"38738700") + (str_to_INTERVAL((char *)"-56531475") + (str_to_INTERVAL((char *)"28198170") - str_to_INTERVAL((char *)"4729725") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"213077865") + (str_to_INTERVAL((char *)"-688677990") + (str_to_INTERVAL((char *)"707581875") + (str_to_INTERVAL((char *)"-96996900") + (str_to_INTERVAL((char *)"-263888625") + (str_to_INTERVAL((char *)"153363210") - str_to_INTERVAL((char *)"24459435") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"1875428555") + (str_to_INTERVAL((char *)"-5346290950") + (str_to_INTERVAL((char *)"4823443625") + (str_to_INTERVAL((char *)"-766465700") + (str_to_INTERVAL((char *)"-985459475") + (str_to_INTERVAL((char *)"455424970") - str_to_INTERVAL((char *)"56081025") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"9830961140") + (str_to_INTERVAL((char *)"-22857074240") + (str_to_INTERVAL((char *)"14551937400") + (str_to_INTERVAL((char *)"1409808400") + (str_to_INTERVAL((char *)"-3782478700") + (str_to_INTERVAL((char *)"859818960") - str_to_INTERVAL((char *)"12972960") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"35481205760") + (str_to_INTERVAL((char *)"-62258676480") + (str_to_INTERVAL((char *)"18611392800") + (str_to_INTERVAL((char *)"17185168000") + (str_to_INTERVAL((char *)"-10345935600") + (str_to_INTERVAL((char *)"1316034720") + str_to_INTERVAL((char *)"227026800") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"93468944800") + (str_to_INTERVAL((char *)"-110815624640") + (str_to_INTERVAL((char *)"-7025933600") + (str_to_INTERVAL((char *)"39340896000") + (str_to_INTERVAL((char *)"-13384207200") + (str_to_INTERVAL((char *)"1845735360") + str_to_INTERVAL((char *)"511086240") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"182237255200") + (str_to_INTERVAL((char *)"-109983670720") + (str_to_INTERVAL((char *)"-47464048800") + (str_to_INTERVAL((char *)"5404560000") + (str_to_INTERVAL((char *)"1648264800") + (str_to_INTERVAL((char *)"1212886080") + str_to_INTERVAL((char *)"424599840") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"263481950080") + (str_to_INTERVAL((char *)"20835903360") + (str_to_INTERVAL((char *)"-5114342400") + (str_to_INTERVAL((char *)"-132425088000") + (str_to_INTERVAL((char *)"19736160000") + (str_to_INTERVAL((char *)"-2213973120") + str_to_INTERVAL((char *)"13299840") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"303658435584") + (str_to_INTERVAL((char *)"283133968896") + (str_to_INTERVAL((char *)"230057018880") + (str_to_INTERVAL((char *)"-278077969920") + (str_to_INTERVAL((char *)"-4805341440") + (str_to_INTERVAL((char *)"-5838902784") - str_to_INTERVAL((char *)"187612416") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"374717099520") + (str_to_INTERVAL((char *)"570046355328") + (str_to_INTERVAL((char *)"647341735680") + (str_to_INTERVAL((char *)"-282405480960") + (str_to_INTERVAL((char *)"-54434528640") + (str_to_INTERVAL((char *)"-4778847360") - str_to_INTERVAL((char *)"53140608") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"613208834048") + (str_to_INTERVAL((char *)"863523896576") + (str_to_INTERVAL((char *)"1071570744320") + (str_to_INTERVAL((char *)"-120962754560") + (str_to_INTERVAL((char *)"-34251729920") + (str_to_INTERVAL((char *)"-237171968") + str_to_INTERVAL((char *)"59810304") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"972373694720") + (str_to_INTERVAL((char *)"1235292172288") + (str_to_INTERVAL((char *)"1317438462720") + (str_to_INTERVAL((char *)"139229708800") + (str_to_INTERVAL((char *)"40532629760") + (str_to_INTERVAL((char *)"1300631040") - str_to_INTERVAL((char *)"7918848") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"1167319596544") + (str_to_INTERVAL((char *)"1454099978240") + (str_to_INTERVAL((char *)"1317778726400") + (str_to_INTERVAL((char *)"283918976000") + (str_to_INTERVAL((char *)"35579338240") + (str_to_INTERVAL((char *)"-356639744") - str_to_INTERVAL((char *)"17625600") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"1003617248256") + (str_to_INTERVAL((char *)"1176807425024") + (str_to_INTERVAL((char *)"926948224000") + (str_to_INTERVAL((char *)"119621171200") + (str_to_INTERVAL((char *)"-12198108160") + (str_to_INTERVAL((char *)"-268151808") - str_to_INTERVAL((char *)"1016832") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"602192244736") + (str_to_INTERVAL((char *)"571376510976") + (str_to_INTERVAL((char *)"233194823680") + (str_to_INTERVAL((char *)"-45159321600") + (str_to_INTERVAL((char *)"-4371804160") - str_to_INTERVAL((char *)"18896896") * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"225739616256") + (str_to_INTERVAL((char *)"100312707072") + (str_to_INTERVAL((char *)"-89991188480") + (str_to_INTERVAL((char *)"-5406699520") - str_to_INTERVAL((char *)"321167360") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"28965724160") + (str_to_INTERVAL((char *)"-22897844224") + (str_to_INTERVAL((char *)"1391616000") - str_to_INTERVAL((char *)"679567360") * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-6625247232") + (str_to_INTERVAL((char *)"12679086080") - str_to_INTERVAL((char *)"611860480") * y * y) * y * y + (str_to_INTERVAL((char *)"291241984") * y * y + str_to_INTERVAL((char *)"3734306816") + str_to_INTERVAL((char *)"104333312") * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * y * y;
	coef[19] = 2 * x * (2 * x + 1) * (str_to_INTERVAL((char *)"-12046320") + (str_to_INTERVAL((char *)"46216170") + (str_to_INTERVAL((char *)"-64864800") + (str_to_INTERVAL((char *)"38223900") + (str_to_INTERVAL((char *)"-6949800") + (str_to_INTERVAL((char *)"-115830") - str_to_INTERVAL((char *)"463320") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-232895520") + (str_to_INTERVAL((char *)"801080280") + (str_to_INTERVAL((char *)"-994593600") + (str_to_INTERVAL((char *)"509652000") + (str_to_INTERVAL((char *)"-78764400") + (str_to_INTERVAL((char *)"-1081080") - str_to_INTERVAL((char *)"3397680") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-2116235880") + (str_to_INTERVAL((char *)"6289661070") + (str_to_INTERVAL((char *)"-6510873600") + (str_to_INTERVAL((char *)"2614461300") + (str_to_INTERVAL((char *)"-278704800") + (str_to_INTERVAL((char *)"6630030") - str_to_INTERVAL((char *)"4938120") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-12063628368") + (str_to_INTERVAL((char *)"29613675168") + (str_to_INTERVAL((char *)"-23300766720") + (str_to_INTERVAL((char *)"5659893360") + (str_to_INTERVAL((char *)"-2185920") + (str_to_INTERVAL((char *)"77833008") + str_to_INTERVAL((char *)"15179472") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-48511265760") + (str_to_INTERVAL((char *)"92800104084") + (str_to_INTERVAL((char *)"-47317163040") + (str_to_INTERVAL((char *)"1228034520") + (str_to_INTERVAL((char *)"1443655440") + (str_to_INTERVAL((char *)"111811860") + str_to_INTERVAL((char *)"13162896") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-147155859810") + (str_to_INTERVAL((char *)"200237737797") + (str_to_INTERVAL((char *)"-45844124760") + (str_to_INTERVAL((char *)"-6354212490") + (str_to_INTERVAL((char *)"-4482633690") + (str_to_INTERVAL((char *)"-878787675") - str_to_INTERVAL((char *)"217095372") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-357107171256") + (str_to_INTERVAL((char *)"285670835532") + (str_to_INTERVAL((char *)"5700205440") + (str_to_INTERVAL((char *)"67037832000") + (str_to_INTERVAL((char *)"-41967720000") + (str_to_INTERVAL((char *)"-3809295324") - str_to_INTERVAL((char *)"655966872") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-747953487816") + (str_to_INTERVAL((char *)"192625355892") + (str_to_INTERVAL((char *)"18560640000") + (str_to_INTERVAL((char *)"368628488760") + (str_to_INTERVAL((char *)"-101485957560") + (str_to_INTERVAL((char *)"-4939690524") - str_to_INTERVAL((char *)"664623072") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-1466990466624") + (str_to_INTERVAL((char *)"-260060618880") + (str_to_INTERVAL((char *)"-250212559200") + (str_to_INTERVAL((char *)"873385337280") + (str_to_INTERVAL((char *)"-80323702080") + (str_to_INTERVAL((char *)"1623029568") - str_to_INTERVAL((char *)"51576864") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-2750734643904") + (str_to_INTERVAL((char *)"-1225348519296") + (str_to_INTERVAL((char *)"-1047542094720") + (str_to_INTERVAL((char *)"1223459505600") + (str_to_INTERVAL((char *)"70751405760") + (str_to_INTERVAL((char *)"9642307584") + str_to_INTERVAL((char *)"253997184") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-4626613463296") + (str_to_INTERVAL((char *)"-2817815172352") + (str_to_INTERVAL((char *)"-2344068718720") + (str_to_INTERVAL((char *)"1004989396480") + (str_to_INTERVAL((char *)"152696227840") + (str_to_INTERVAL((char *)"6790387456") + str_to_INTERVAL((char *)"16046976") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-6438570585344") + (str_to_INTERVAL((char *)"-4768100904192") + (str_to_INTERVAL((char *)"-3655043875840") + (str_to_INTERVAL((char *)"225710789120") + (str_to_INTERVAL((char *)"24104190720") + (str_to_INTERVAL((char *)"-521869568") - str_to_INTERVAL((char *)"45387264") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-7033446088704") + (str_to_INTERVAL((char *)"-5949377958912") + (str_to_INTERVAL((char *)"-4180469870080") + (str_to_INTERVAL((char *)"-479969648640") + (str_to_INTERVAL((char *)"-79235435520") + (str_to_INTERVAL((char *)"-815111168") + str_to_INTERVAL((char *)"23548416") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-5846616353280") + (str_to_INTERVAL((char *)"-5192009876736") + (str_to_INTERVAL((char *)"-3310901401600") + (str_to_INTERVAL((char *)"-465246778880") + (str_to_INTERVAL((char *)"-20548436480") + (str_to_INTERVAL((char *)"502409984") - str_to_INTERVAL((char *)"5753856") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-3583003416576") + (str_to_INTERVAL((char *)"-2940622973952") + (str_to_INTERVAL((char *)"-1462758277120") + (str_to_INTERVAL((char *)"-61965598720") + (str_to_INTERVAL((char *)"13574277120") + (str_to_INTERVAL((char *)"-169587712") - str_to_INTERVAL((char *)"1253376") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-1517636265984") + (str_to_INTERVAL((char *)"-908439217152") + (str_to_INTERVAL((char *)"-118830346240") + (str_to_INTERVAL((char *)"39169320960") + (str_to_INTERVAL((char *)"-5263779840") - str_to_INTERVAL((char *)"20681728") * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-369703501824") + (str_to_INTERVAL((char *)"-57654853632") + (str_to_INTERVAL((char *)"57471918080") + (str_to_INTERVAL((char *)"-23344824320") - str_to_INTERVAL((char *)"230850560") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-20395229184") + (str_to_INTERVAL((char *)"5652185088") + (str_to_INTERVAL((char *)"-53251194880") - str_to_INTERVAL((char *)"35307520") * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-1735262208") + (str_to_INTERVAL((char *)"-26297499648") + str_to_INTERVAL((char *)"1523056640") * y * y) * y * y + (str_to_INTERVAL((char *)"1678508032") * y * y - str_to_INTERVAL((char *)"7887650816") + str_to_INTERVAL((char *)"687865856") * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * y * y;
	coef[20] = (-1)*sqrt2xplus1 *  pow( (2 * x + 1),  2) * (str_to_INTERVAL((char *)"-216216") + (str_to_INTERVAL((char *)"1297296") + (str_to_INTERVAL((char *)"-3243240") + (str_to_INTERVAL((char *)"4324320") + (str_to_INTERVAL((char *)"-3243240") + (str_to_INTERVAL((char *)"1297296") - str_to_INTERVAL((char *)"216216") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"5378373") + (str_to_INTERVAL((char *)"-15945930") + (str_to_INTERVAL((char *)"7162155") + (str_to_INTERVAL((char *)"23243220") + (str_to_INTERVAL((char *)"-33918885") + (str_to_INTERVAL((char *)"16918902") - str_to_INTERVAL((char *)"2837835") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"127846719") + (str_to_INTERVAL((char *)"-413206794") + (str_to_INTERVAL((char *)"424549125") + (str_to_INTERVAL((char *)"-58198140") + (str_to_INTERVAL((char *)"-158333175") + (str_to_INTERVAL((char *)"92017926") - str_to_INTERVAL((char *)"14675661") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"1125257133") + (str_to_INTERVAL((char *)"-3207774570") + (str_to_INTERVAL((char *)"2894066175") + (str_to_INTERVAL((char *)"-459879420") + (str_to_INTERVAL((char *)"-591275685") + (str_to_INTERVAL((char *)"273254982") - str_to_INTERVAL((char *)"33648615") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"5898576684") + (str_to_INTERVAL((char *)"-13714244544") + (str_to_INTERVAL((char *)"8731162440") + (str_to_INTERVAL((char *)"845885040") + (str_to_INTERVAL((char *)"-2269487220") + (str_to_INTERVAL((char *)"515891376") - str_to_INTERVAL((char *)"7783776") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"21288723456") + (str_to_INTERVAL((char *)"-37355205888") + (str_to_INTERVAL((char *)"11166835680") + (str_to_INTERVAL((char *)"10311100800") + (str_to_INTERVAL((char *)"-6207561360") + (str_to_INTERVAL((char *)"789620832") + str_to_INTERVAL((char *)"136216080") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"55942336464") + (str_to_INTERVAL((char *)"-66502779168") + (str_to_INTERVAL((char *)"-4136245680") + (str_to_INTERVAL((char *)"24008503680") + (str_to_INTERVAL((char *)"-8331240960") + (str_to_INTERVAL((char *)"1080922752") + str_to_INTERVAL((char *)"303041088") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"107534957712") + (str_to_INTERVAL((char *)"-66137650656") + (str_to_INTERVAL((char *)"-27764598960") + (str_to_INTERVAL((char *)"6070498560") + (str_to_INTERVAL((char *)"-514624320") + (str_to_INTERVAL((char *)"648176256") + str_to_INTERVAL((char *)"251149248") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"147782831952") + (str_to_INTERVAL((char *)"10925144160") + (str_to_INTERVAL((char *)"-1086602160") + (str_to_INTERVAL((char *)"-71357919360") + (str_to_INTERVAL((char *)"10285655520") + (str_to_INTERVAL((char *)"-1307092416") + str_to_INTERVAL((char *)"16601760") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"148205830752") + (str_to_INTERVAL((char *)"159517192128") + (str_to_INTERVAL((char *)"138182731680") + (str_to_INTERVAL((char *)"-154240853760") + (str_to_INTERVAL((char *)"-856927680") + (str_to_INTERVAL((char *)"-3369442944") - str_to_INTERVAL((char *)"114077376") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"152754091248") + (str_to_INTERVAL((char *)"305042698656") + (str_to_INTERVAL((char *)"377676513840") + (str_to_INTERVAL((char *)"-159721622400") + (str_to_INTERVAL((char *)"-30420660960") + (str_to_INTERVAL((char *)"-2945854272") - str_to_INTERVAL((char *)"36529056") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"264443517376") + (str_to_INTERVAL((char *)"442957689472") + (str_to_INTERVAL((char *)"614983920640") + (str_to_INTERVAL((char *)"-75241661440") + (str_to_INTERVAL((char *)"-22702637440") + (str_to_INTERVAL((char *)"-189460480") + str_to_INTERVAL((char *)"40131072") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"480425092352") + (str_to_INTERVAL((char *)"651766808064") + (str_to_INTERVAL((char *)"755021785600") + (str_to_INTERVAL((char *)"74909040640") + (str_to_INTERVAL((char *)"23849064960") + (str_to_INTERVAL((char *)"824946176") - str_to_INTERVAL((char *)"9642240") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"629593009152") + (str_to_INTERVAL((char *)"811488706560") + (str_to_INTERVAL((char *)"772916654080") + (str_to_INTERVAL((char *)"171364439040") + (str_to_INTERVAL((char *)"22230865920") + (str_to_INTERVAL((char *)"-227522560") - str_to_INTERVAL((char *)"1244160") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"570581128960") + (str_to_INTERVAL((char *)"686040010240") + (str_to_INTERVAL((char *)"563507098880") + (str_to_INTERVAL((char *)"76012088320") + (str_to_INTERVAL((char *)"-6399468800") + (str_to_INTERVAL((char *)"-5555712") + str_to_INTERVAL((char *)"306432") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"354742412288") + (str_to_INTERVAL((char *)"344529580032") + (str_to_INTERVAL((char *)"151686440960") + (str_to_INTERVAL((char *)"-22095400960") + (str_to_INTERVAL((char *)"583879680") + str_to_INTERVAL((char *)"4816896") * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"137389332480") + (str_to_INTERVAL((char *)"64167346176") + (str_to_INTERVAL((char *)"-42202009600") + (str_to_INTERVAL((char *)"3679887360") + str_to_INTERVAL((char *)"87029760") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"19516284928") + (str_to_INTERVAL((char *)"-13952491520") + (str_to_INTERVAL((char *)"10632192000") + str_to_INTERVAL((char *)"202629120") * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-3303915520") + (str_to_INTERVAL((char *)"5970493440") + str_to_INTERVAL((char *)"361512960") * y * y) * y * y + (str_to_INTERVAL((char *)"92798976") * y * y + str_to_INTERVAL((char *)"2226192384") + str_to_INTERVAL((char *)"58195968") * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * y * y;
	coef[21] = (-1)*x * (str_to_INTERVAL((char *)"-15225210") + (str_to_INTERVAL((char *)"58017960") + (str_to_INTERVAL((char *)"-80630550") + (str_to_INTERVAL((char *)"46846800") + (str_to_INTERVAL((char *)"-8558550") + (str_to_INTERVAL((char *)"360360") - str_to_INTERVAL((char *)"810810") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-324804480") + (str_to_INTERVAL((char *)"1121680560") + (str_to_INTERVAL((char *)"-1397596200") + (str_to_INTERVAL((char *)"718317600") + (str_to_INTERVAL((char *)"-114114000") + (str_to_INTERVAL((char *)"4084080") - str_to_INTERVAL((char *)"7567560") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-3265442950") + (str_to_INTERVAL((char *)"9911566280") + (str_to_INTERVAL((char *)"-10566267250") + (str_to_INTERVAL((char *)"4451062000") + (str_to_INTERVAL((char *)"-543654650") + (str_to_INTERVAL((char *)"34018600") - str_to_INTERVAL((char *)"21282030") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-20637416184") + (str_to_INTERVAL((char *)"53048142224") + (str_to_INTERVAL((char *)"-45154413920") + (str_to_INTERVAL((char *)"13311033120") + (str_to_INTERVAL((char *)"-766424120") + (str_to_INTERVAL((char *)"195785744") + str_to_INTERVAL((char *)"3293136") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-92215702236") + (str_to_INTERVAL((char *)"191527563248") + (str_to_INTERVAL((char *)"-116721826620") + (str_to_INTERVAL((char *)"15172866240") + (str_to_INTERVAL((char *)"1394845900") + (str_to_INTERVAL((char *)"469661136") + str_to_INTERVAL((char *)"69889932") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-311280404953") + (str_to_INTERVAL((char *)"487893991312") + (str_to_INTERVAL((char *)"-174036812495") + (str_to_INTERVAL((char *)"-5390163520") + (str_to_INTERVAL((char *)"-2873431575") + (str_to_INTERVAL((char *)"-784397712") - str_to_INTERVAL((char *)"245891457") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-835558446744") + (str_to_INTERVAL((char *)"873783289960") + (str_to_INTERVAL((char *)"-103012463260") + (str_to_INTERVAL((char *)"66142018320") + (str_to_INTERVAL((char *)"-63397155360") + (str_to_INTERVAL((char *)"-7314996696") - str_to_INTERVAL((char *)"1496709900") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-1887664520112") + (str_to_INTERVAL((char *)"988628929792") + (str_to_INTERVAL((char *)"50663653740") + (str_to_INTERVAL((char *)"619216661280") + (str_to_INTERVAL((char *)"-225621852120") + (str_to_INTERVAL((char *)"-16174904928") - str_to_INTERVAL((char *)"2689936452") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-3831352384384") + (str_to_INTERVAL((char *)"200507684160") + (str_to_INTERVAL((char *)"-238016718240") + (str_to_INTERVAL((char *)"1984630310880") + (str_to_INTERVAL((char *)"-338139503520") + (str_to_INTERVAL((char *)"-9604941024") - str_to_INTERVAL((char *)"1820196672") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-7291180377264") + (str_to_INTERVAL((char *)"-2163875164416") + (str_to_INTERVAL((char *)"-1895606976480") + (str_to_INTERVAL((char *)"3649034645760") + (str_to_INTERVAL((char *)"-92996687280") + (str_to_INTERVAL((char *)"18466597632") + str_to_INTERVAL((char *)"236823552") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-12786234557824") + (str_to_INTERVAL((char *)"-6618740377984") + (str_to_INTERVAL((char *)"-5511324703360") + (str_to_INTERVAL((char *)"4205738481280") + (str_to_INTERVAL((char *)"374076559360") + (str_to_INTERVAL((char *)"34714236928") + str_to_INTERVAL((char *)"686262528") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-19472973741312") + (str_to_INTERVAL((char *)"-13000784312320") + (str_to_INTERVAL((char *)"-10380293638400") + (str_to_INTERVAL((char *)"2663198576640") + (str_to_INTERVAL((char *)"401763765760") + (str_to_INTERVAL((char *)"17057137664") - str_to_INTERVAL((char *)"12122880") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-24348871902208") + (str_to_INTERVAL((char *)"-19130931341312") + (str_to_INTERVAL((char *)"-14247115402240") + (str_to_INTERVAL((char *)"-123656099840") + (str_to_INTERVAL((char *)"-47744005120") + (str_to_INTERVAL((char *)"-2072999936") - str_to_INTERVAL((char *)"77220864") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-24082008991488") + (str_to_INTERVAL((char *)"-20879203401728") + (str_to_INTERVAL((char *)"-14392942690560") + (str_to_INTERVAL((char *)"-1801567846400") + (str_to_INTERVAL((char *)"-227850389760") + (str_to_INTERVAL((char *)"-1353089024") + str_to_INTERVAL((char *)"51593472") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-18335857960960") + (str_to_INTERVAL((char *)"-16165932509184") + (str_to_INTERVAL((char *)"-9963204510720") + (str_to_INTERVAL((char *)"-1261483888640") + (str_to_INTERVAL((char *)"-44792606720") + (str_to_INTERVAL((char *)"980596736") - str_to_INTERVAL((char *)"10429440") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-10393601052672") + (str_to_INTERVAL((char *)"-8257542144000") + (str_to_INTERVAL((char *)"-3854299806720") + (str_to_INTERVAL((char *)"-156946472960") + (str_to_INTERVAL((char *)"19709818880") + (str_to_INTERVAL((char *)"-301113344") + str_to_INTERVAL((char *)"4322304") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-4101090017280") + (str_to_INTERVAL((char *)"-2348696616960") + (str_to_INTERVAL((char *)"-355744358400") + (str_to_INTERVAL((char *)"47850496000") + (str_to_INTERVAL((char *)"-11222753280") + str_to_INTERVAL((char *)"78733312") * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-946475388928") + (str_to_INTERVAL((char *)"-177029251072") + (str_to_INTERVAL((char *)"44435824640") + (str_to_INTERVAL((char *)"-45673676800") + str_to_INTERVAL((char *)"1778954240") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-66390884352") + (str_to_INTERVAL((char *)"-6379700224") + (str_to_INTERVAL((char *)"-103939604480") + str_to_INTERVAL((char *)"5313495040") * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-10325000192") + (str_to_INTERVAL((char *)"-47693168640") + str_to_INTERVAL((char *)"10078781440") * y * y) * y * y + (str_to_INTERVAL((char *)"-15232663552") + str_to_INTERVAL((char *)"3108503552") * y * y + str_to_INTERVAL((char *)"1086324736") * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * y * y;
	coef[22] = sqrt2xplus1 *  pow( (2 * x + 1),  2) * (str_to_INTERVAL((char *)"-98280") + (str_to_INTERVAL((char *)"589680") + (str_to_INTERVAL((char *)"-1474200") + (str_to_INTERVAL((char *)"1965600") + (str_to_INTERVAL((char *)"-1474200") + (str_to_INTERVAL((char *)"589680") - str_to_INTERVAL((char *)"98280") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"2444715") + (str_to_INTERVAL((char *)"-7248150") + (str_to_INTERVAL((char *)"3255525") + (str_to_INTERVAL((char *)"10565100") + (str_to_INTERVAL((char *)"-15417675") + (str_to_INTERVAL((char *)"7690410") - str_to_INTERVAL((char *)"1289925") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"58112145") + (str_to_INTERVAL((char *)"-187821270") + (str_to_INTERVAL((char *)"192976875") + (str_to_INTERVAL((char *)"-26453700") + (str_to_INTERVAL((char *)"-71969625") + (str_to_INTERVAL((char *)"41826330") - str_to_INTERVAL((char *)"6670755") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"511480515") + (str_to_INTERVAL((char *)"-1458079350") + (str_to_INTERVAL((char *)"1315484625") + (str_to_INTERVAL((char *)"-209036100") + (str_to_INTERVAL((char *)"-268761675") + (str_to_INTERVAL((char *)"124206810") - str_to_INTERVAL((char *)"15294825") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"2681171220") + (str_to_INTERVAL((char *)"-6233747520") + (str_to_INTERVAL((char *)"3968710200") + (str_to_INTERVAL((char *)"384493200") + (str_to_INTERVAL((char *)"-1031585100") + (str_to_INTERVAL((char *)"234496080") - str_to_INTERVAL((char *)"3538080") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"9676692480") + (str_to_INTERVAL((char *)"-16979639040") + (str_to_INTERVAL((char *)"5075834400") + (str_to_INTERVAL((char *)"4686864000") + (str_to_INTERVAL((char *)"-2821618800") + (str_to_INTERVAL((char *)"358918560") + str_to_INTERVAL((char *)"61916400") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"25343646720") + (str_to_INTERVAL((char *)"-30238703040") + (str_to_INTERVAL((char *)"-1871083200") + (str_to_INTERVAL((char *)"11235672000") + (str_to_INTERVAL((char *)"-3999088800") + (str_to_INTERVAL((char *)"470352960") + str_to_INTERVAL((char *)"133993440") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"47778581760") + (str_to_INTERVAL((char *)"-30174406080") + (str_to_INTERVAL((char *)"-12539016000") + (str_to_INTERVAL((char *)"5018328000") + (str_to_INTERVAL((char *)"-1294725600") + (str_to_INTERVAL((char *)"231698880") + str_to_INTERVAL((char *)"110406240") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"60894197280") + (str_to_INTERVAL((char *)"3901255680") + (str_to_INTERVAL((char *)"-722450400") + (str_to_INTERVAL((char *)"-26013019200") + (str_to_INTERVAL((char *)"3557064000") + (str_to_INTERVAL((char *)"-578860800") + str_to_INTERVAL((char *)"13849920") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"46640328000") + (str_to_INTERVAL((char *)"65913402240") + (str_to_INTERVAL((char *)"59345388480") + (str_to_INTERVAL((char *)"-60300495360") + (str_to_INTERVAL((char *)"965193600") + (str_to_INTERVAL((char *)"-1428268800") - str_to_INTERVAL((char *)"52536960") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"25416707424") + (str_to_INTERVAL((char *)"115634092416") + (str_to_INTERVAL((char *)"159055336800") + (str_to_INTERVAL((char *)"-65090575680") + (str_to_INTERVAL((char *)"-12179674560") + (str_to_INTERVAL((char *)"-1378512384") - str_to_INTERVAL((char *)"18294336") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"56827642112") + (str_to_INTERVAL((char *)"154918825856") + (str_to_INTERVAL((char *)"254863560320") + (str_to_INTERVAL((char *)"-35602401920") + (str_to_INTERVAL((char *)"-11516072960") + (str_to_INTERVAL((char *)"-115488512") + str_to_INTERVAL((char *)"18664704") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"154961693440") + (str_to_INTERVAL((char *)"241036954368") + (str_to_INTERVAL((char *)"314287934720") + (str_to_INTERVAL((char *)"27926493440") + (str_to_INTERVAL((char *)"10419029760") + (str_to_INTERVAL((char *)"382927360") - str_to_INTERVAL((char *)"5015808") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"242111514624") + (str_to_INTERVAL((char *)"330874037760") + (str_to_INTERVAL((char *)"335067776000") + (str_to_INTERVAL((char *)"77379678720") + (str_to_INTERVAL((char *)"10492147200") + (str_to_INTERVAL((char *)"-94444544") + str_to_INTERVAL((char *)"2004480") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"239140771328") + (str_to_INTERVAL((char *)"298810022912") + (str_to_INTERVAL((char *)"257761876480") + (str_to_INTERVAL((char *)"36715166720") + (str_to_INTERVAL((char *)"-2416990720") + (str_to_INTERVAL((char *)"39585792") + str_to_INTERVAL((char *)"96768") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"156472029184") + (str_to_INTERVAL((char *)"156987789312") + (str_to_INTERVAL((char *)"75168778240") + (str_to_INTERVAL((char *)"-7802378240") + (str_to_INTERVAL((char *)"1032837120") + str_to_INTERVAL((char *)"2095104") * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"63108071424") + (str_to_INTERVAL((char *)"31465684992") + (str_to_INTERVAL((char *)"-14546554880") + (str_to_INTERVAL((char *)"3456245760") + str_to_INTERVAL((char *)"47247360") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"9752084480") + (str_to_INTERVAL((char *)"-5700911104") + (str_to_INTERVAL((char *)"7104184320") + str_to_INTERVAL((char *)"152371200") * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-1453064192") + (str_to_INTERVAL((char *)"2536243200") + str_to_INTERVAL((char *)"261488640") * y * y) * y * y + (str_to_INTERVAL((char *)"72351744") * y * y + str_to_INTERVAL((char *)"832045056") + str_to_INTERVAL((char *)"12582912") * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * y * y;
	coef[23] = 2 * x * (str_to_INTERVAL((char *)"-3710070") + (str_to_INTERVAL((char *)"14029470") + (str_to_INTERVAL((char *)"-19287450") + (str_to_INTERVAL((char *)"11056500") + (str_to_INTERVAL((char *)"-2088450") + (str_to_INTERVAL((char *)"270270") - str_to_INTERVAL((char *)"270270") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-79148160") + (str_to_INTERVAL((char *)"271236420") + (str_to_INTERVAL((char *)"-334315800") + (str_to_INTERVAL((char *)"169533000") + (str_to_INTERVAL((char *)"-27846000") + (str_to_INTERVAL((char *)"3063060") - str_to_INTERVAL((char *)"2522520") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-796264890") + (str_to_INTERVAL((char *)"2397867450") + (str_to_INTERVAL((char *)"-2527548150") + (str_to_INTERVAL((char *)"1050428700") + (str_to_INTERVAL((char *)"-135685950") + (str_to_INTERVAL((char *)"18530490") - str_to_INTERVAL((char *)"7327650") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-5039787528") + (str_to_INTERVAL((char *)"12847982388") + (str_to_INTERVAL((char *)"-10801531680") + (str_to_INTERVAL((char *)"3140411400") + (str_to_INTERVAL((char *)"-223307880") + (str_to_INTERVAL((char *)"77004708") - str_to_INTERVAL((char *)"771408") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-22579682004") + (str_to_INTERVAL((char *)"46483654668") + (str_to_INTERVAL((char *)"-27909830340") + (str_to_INTERVAL((char *)"3577792680") + (str_to_INTERVAL((char *)"166103220") + (str_to_INTERVAL((char *)"164271564") + str_to_INTERVAL((char *)"21031812") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-76560167607") + (str_to_INTERVAL((char *)"118858964211") + (str_to_INTERVAL((char *)"-41462344755") + (str_to_INTERVAL((char *)"-1240852710") + (str_to_INTERVAL((char *)"-1072832325") + (str_to_INTERVAL((char *)"-159272493") - str_to_INTERVAL((char *)"57841521") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-206831957976") + (str_to_INTERVAL((char *)"214382416914") + (str_to_INTERVAL((char *)"-23618495160") + (str_to_INTERVAL((char *)"15896530980") + (str_to_INTERVAL((char *)"-15443916000") + (str_to_INTERVAL((char *)"-1830625734") - str_to_INTERVAL((char *)"384958944") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-470243061648") + (str_to_INTERVAL((char *)"246456869580") + (str_to_INTERVAL((char *)"15492108900") + (str_to_INTERVAL((char *)"147141615480") + (str_to_INTERVAL((char *)"-52902148680") + (str_to_INTERVAL((char *)"-3994389492") - str_to_INTERVAL((char *)"686359980") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-955124276880") + (str_to_INTERVAL((char *)"59759190120") + (str_to_INTERVAL((char *)"-50430757920") + (str_to_INTERVAL((char *)"469721779920") + (str_to_INTERVAL((char *)"-77168630160") + (str_to_INTERVAL((char *)"-2149838616") - str_to_INTERVAL((char *)"446216064") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-1799671284336") + (str_to_INTERVAL((char *)"-513573367104") + (str_to_INTERVAL((char *)"-446328564000") + (str_to_INTERVAL((char *)"858871058880") + (str_to_INTERVAL((char *)"-17461538160") + (str_to_INTERVAL((char *)"4901128128") + str_to_INTERVAL((char *)"74704320") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-3098868249472") + (str_to_INTERVAL((char *)"-1591175349760") + (str_to_INTERVAL((char *)"-1313704768000") + (str_to_INTERVAL((char *)"979283288320") + (str_to_INTERVAL((char *)"90690864640") + (str_to_INTERVAL((char *)"8662162048") + str_to_INTERVAL((char *)"173331072") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-4626328896256") + (str_to_INTERVAL((char *)"-3102819033344") + (str_to_INTERVAL((char *)"-2474027116800") + (str_to_INTERVAL((char *)"604029836800") + (str_to_INTERVAL((char *)"93119068160") + (str_to_INTERVAL((char *)"4126320384") - str_to_INTERVAL((char *)"1890048") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-5685477571072") + (str_to_INTERVAL((char *)"-4504024158720") + (str_to_INTERVAL((char *)"-3375942620160") + (str_to_INTERVAL((char *)"-50283576320") + (str_to_INTERVAL((char *)"-13319600640") + (str_to_INTERVAL((char *)"-517814784") - str_to_INTERVAL((char *)"19722240") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-5547866375424") + (str_to_INTERVAL((char *)"-4850372687616") + (str_to_INTERVAL((char *)"-3383970873600") + (str_to_INTERVAL((char *)"-432807641600") + (str_to_INTERVAL((char *)"-54790188800") + (str_to_INTERVAL((char *)"-386242304") + str_to_INTERVAL((char *)"8987904") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-4184242526208") + (str_to_INTERVAL((char *)"-3721693186560") + (str_to_INTERVAL((char *)"-2335268659200") + (str_to_INTERVAL((char *)"-303641154560") + (str_to_INTERVAL((char *)"-12822046720") + (str_to_INTERVAL((char *)"161521152") - str_to_INTERVAL((char *)"4055040") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-2360950099968") + (str_to_INTERVAL((char *)"-1901299817472") + (str_to_INTERVAL((char *)"-922519925760") + (str_to_INTERVAL((char *)"-47532677120") + (str_to_INTERVAL((char *)"3125913600") + (str_to_INTERVAL((char *)"-83524608") + str_to_INTERVAL((char *)"580608") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-935478300672") + (str_to_INTERVAL((char *)"-554667362304") + (str_to_INTERVAL((char *)"-108128870400") + (str_to_INTERVAL((char *)"7601582080") + (str_to_INTERVAL((char *)"-2521640960") + str_to_INTERVAL((char *)"12826624") * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-221972385792") + (str_to_INTERVAL((char *)"-50513510400") + (str_to_INTERVAL((char *)"7082434560") + (str_to_INTERVAL((char *)"-8976220160") + str_to_INTERVAL((char *)"314224640") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-17855250432") + (str_to_INTERVAL((char *)"428900352") + (str_to_INTERVAL((char *)"-19297566720") + str_to_INTERVAL((char *)"1076101120") * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-1207107584") + (str_to_INTERVAL((char *)"-7916748800") + str_to_INTERVAL((char *)"2041446400") * y * y) * y * y + (str_to_INTERVAL((char *)"-2609905664") + str_to_INTERVAL((char *)"706740224") * y * y + str_to_INTERVAL((char *)"180355072") * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * y * y;
	coef[24] = (-1)*sqrt2xplus1 *  pow( (2 * x + 1),  2) * (str_to_INTERVAL((char *)"-32760") + (str_to_INTERVAL((char *)"196560") + (str_to_INTERVAL((char *)"-491400") + (str_to_INTERVAL((char *)"655200") + (str_to_INTERVAL((char *)"-491400") + (str_to_INTERVAL((char *)"196560") - str_to_INTERVAL((char *)"32760") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"814905") + (str_to_INTERVAL((char *)"-2416050") + (str_to_INTERVAL((char *)"1085175") + (str_to_INTERVAL((char *)"3521700") + (str_to_INTERVAL((char *)"-5139225") + (str_to_INTERVAL((char *)"2563470") - str_to_INTERVAL((char *)"429975") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"19370715") + (str_to_INTERVAL((char *)"-62607090") + (str_to_INTERVAL((char *)"64325625") + (str_to_INTERVAL((char *)"-8817900") + (str_to_INTERVAL((char *)"-23989875") + (str_to_INTERVAL((char *)"13942110") - str_to_INTERVAL((char *)"2223585") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"170493505") + (str_to_INTERVAL((char *)"-486026450") + (str_to_INTERVAL((char *)"438494875") + (str_to_INTERVAL((char *)"-69678700") + (str_to_INTERVAL((char *)"-89587225") + (str_to_INTERVAL((char *)"41402270") - str_to_INTERVAL((char *)"5098275") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"893723740") + (str_to_INTERVAL((char *)"-2077915840") + (str_to_INTERVAL((char *)"1322903400") + (str_to_INTERVAL((char *)"128164400") + (str_to_INTERVAL((char *)"-343861700") + (str_to_INTERVAL((char *)"78165360") - str_to_INTERVAL((char *)"1179360") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"3225564160") + (str_to_INTERVAL((char *)"-5659879680") + (str_to_INTERVAL((char *)"1691944800") + (str_to_INTERVAL((char *)"1562288000") + (str_to_INTERVAL((char *)"-940539600") + (str_to_INTERVAL((char *)"119639520") + str_to_INTERVAL((char *)"20638800") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"8406986000") + (str_to_INTERVAL((char *)"-10085350240") + (str_to_INTERVAL((char *)"-634824400") + (str_to_INTERVAL((char *)"3930729600") + (str_to_INTERVAL((char *)"-1445791200") + (str_to_INTERVAL((char *)"144600960") + str_to_INTERVAL((char *)"41912640") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"15394542800") + (str_to_INTERVAL((char *)"-10121743520") + (str_to_INTERVAL((char *)"-4279842000") + (str_to_INTERVAL((char *)"2971315200") + (str_to_INTERVAL((char *)"-995383200") + (str_to_INTERVAL((char *)"40682880") + str_to_INTERVAL((char *)"34050240") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"17264719760") + (str_to_INTERVAL((char *)"740597280") + (str_to_INTERVAL((char *)"-918714000") + (str_to_INTERVAL((char *)"-4993689600") + (str_to_INTERVAL((char *)"588955200") + (str_to_INTERVAL((char *)"-182884800") + str_to_INTERVAL((char *)"7852320") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"5528857824") + (str_to_INTERVAL((char *)"18662709696") + (str_to_INTERVAL((char *)"16696068960") + (str_to_INTERVAL((char *)"-14543575680") + (str_to_INTERVAL((char *)"1032936960") + (str_to_INTERVAL((char *)"-414094464") - str_to_INTERVAL((char *)"17551296") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-12831170256") + (str_to_INTERVAL((char *)"27207721824") + (str_to_INTERVAL((char *)"44491320720") + (str_to_INTERVAL((char *)"-17470320000") + (str_to_INTERVAL((char *)"-3149123520") + (str_to_INTERVAL((char *)"-471008064") - str_to_INTERVAL((char *)"6598944") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-11802458048") + (str_to_INTERVAL((char *)"28925367040") + (str_to_INTERVAL((char *)"69943058560") + (str_to_INTERVAL((char *)"-12490295680") + (str_to_INTERVAL((char *)"-4373958400") + (str_to_INTERVAL((char *)"-52037632") + str_to_INTERVAL((char *)"5948160") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"20755055488") + (str_to_INTERVAL((char *)"53323085312") + (str_to_INTERVAL((char *)"87780810240") + (str_to_INTERVAL((char *)"5967457280") + (str_to_INTERVAL((char *)"3204119680") + (str_to_INTERVAL((char *)"126753792") - str_to_INTERVAL((char *)"1285632") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"59045520896") + (str_to_INTERVAL((char *)"91556631040") + (str_to_INTERVAL((char *)"101480128000") + (str_to_INTERVAL((char *)"25050058240") + (str_to_INTERVAL((char *)"3614074880") + (str_to_INTERVAL((char *)"-24518656") + str_to_INTERVAL((char *)"552960") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"69570650880") + (str_to_INTERVAL((char *)"92973591040") + (str_to_INTERVAL((char *)"85285717760") + (str_to_INTERVAL((char *)"12998312960") + (str_to_INTERVAL((char *)"-635889920") + (str_to_INTERVAL((char *)"12430848") + 6912 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"49553308672") + (str_to_INTERVAL((char *)"52321630208") + (str_to_INTERVAL((char *)"27350978560") + (str_to_INTERVAL((char *)"-1995376640") + (str_to_INTERVAL((char *)"320568320") + str_to_INTERVAL((char *)"167936") * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"21169946624") + (str_to_INTERVAL((char *)"11609866240") + (str_to_INTERVAL((char *)"-3665428480") + (str_to_INTERVAL((char *)"1141391360") + str_to_INTERVAL((char *)"5570560") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"3619454976") + (str_to_INTERVAL((char *)"-1418756096") + (str_to_INTERVAL((char *)"2290974720") + str_to_INTERVAL((char *)"22446080") * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-409468928") + (str_to_INTERVAL((char *)"874250240") + str_to_INTERVAL((char *)"52428800") * y * y) * y * y + (str_to_INTERVAL((char *)"24641536") * y * y + str_to_INTERVAL((char *)"250609664") + str_to_INTERVAL((char *)"8388608") * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * y * y;
	coef[25] = (-1)*x * (str_to_INTERVAL((char *)"-2727270") + (str_to_INTERVAL((char *)"10221120") + (str_to_INTERVAL((char *)"-13882050") + (str_to_INTERVAL((char *)"7862400") + (str_to_INTERVAL((char *)"-1597050") + (str_to_INTERVAL((char *)"393120") - str_to_INTERVAL((char *)"270270") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-58181760") + (str_to_INTERVAL((char *)"197608320") + (str_to_INTERVAL((char *)"-240622200") + (str_to_INTERVAL((char *)"120556800") + (str_to_INTERVAL((char *)"-21294000") + (str_to_INTERVAL((char *)"4455360") - str_to_INTERVAL((char *)"2522520") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-585781290") + (str_to_INTERVAL((char *)"1747837200") + (str_to_INTERVAL((char *)"-1819213350") + (str_to_INTERVAL((char *)"747523200") + (str_to_INTERVAL((char *)"-107394150") + (str_to_INTERVAL((char *)"24573840") - str_to_INTERVAL((char *)"7545450") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-3713694408") + (str_to_INTERVAL((char *)"9376182048") + (str_to_INTERVAL((char *)"-7774713120") + (str_to_INTERVAL((char *)"2240904960") + (str_to_INTERVAL((char *)"-214376520") + (str_to_INTERVAL((char *)"88210848") - str_to_INTERVAL((char *)"2513808") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-16687785204") + (str_to_INTERVAL((char *)"33997739808") + (str_to_INTERVAL((char *)"-20080988100") + (str_to_INTERVAL((char *)"2598098400") + (str_to_INTERVAL((char *)"-81166620") + (str_to_INTERVAL((char *)"176442624") + str_to_INTERVAL((char *)"18691092") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-56861408247") + (str_to_INTERVAL((char *)"87276334956") + (str_to_INTERVAL((char *)"-29718664365") + (str_to_INTERVAL((char *)"-646159320") + (str_to_INTERVAL((char *)"-1240030305") + (str_to_INTERVAL((char *)"-69517908") - str_to_INTERVAL((char *)"37850811") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-154675170216") + (str_to_INTERVAL((char *)"158574972624") + (str_to_INTERVAL((char *)"-16219220820") + (str_to_INTERVAL((char *)"11947593600") + (str_to_INTERVAL((char *)"-11598456960") + (str_to_INTERVAL((char *)"-1355464944") - str_to_INTERVAL((char *)"289571364") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-353908457768") + (str_to_INTERVAL((char *)"185285427760") + (str_to_INTERVAL((char *)"13819710460") + (str_to_INTERVAL((char *)"105451165760") + (str_to_INTERVAL((char *)"-37384868080") + (str_to_INTERVAL((char *)"-2943173872") - str_to_INTERVAL((char *)"513369540") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-718649169216") + (str_to_INTERVAL((char *)"52578418336") + (str_to_INTERVAL((char *)"-30676164160") + (str_to_INTERVAL((char *)"332874701280") + (str_to_INTERVAL((char *)"-52382600800") + (str_to_INTERVAL((char *)"-1431640640") - str_to_INTERVAL((char *)"318930720") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-1337717974816") + (str_to_INTERVAL((char *)"-364564580736") + (str_to_INTERVAL((char *)"-312948739840") + (str_to_INTERVAL((char *)"602602526080") + (str_to_INTERVAL((char *)"-8676875040") + (str_to_INTERVAL((char *)"3800048128") + str_to_INTERVAL((char *)"67312512") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-2253918717440") + (str_to_INTERVAL((char *)"-1143999615488") + (str_to_INTERVAL((char *)"-933132336000") + (str_to_INTERVAL((char *)"676738209280") + (str_to_INTERVAL((char *)"65319381760") + (str_to_INTERVAL((char *)"6335277056") + str_to_INTERVAL((char *)"127969920") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-3284763128320") + (str_to_INTERVAL((char *)"-2208582280704") + (str_to_INTERVAL((char *)"-1754086164480") + (str_to_INTERVAL((char *)"403715883520") + (str_to_INTERVAL((char *)"63481948160") + (str_to_INTERVAL((char *)"2904352768") - str_to_INTERVAL((char *)"1118208") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-3950968614912") + (str_to_INTERVAL((char *)"-3151514311680") + (str_to_INTERVAL((char *)"-2373570716160") + (str_to_INTERVAL((char *)"-51289041920") + (str_to_INTERVAL((char *)"-10943539200") + (str_to_INTERVAL((char *)"-396130304") - str_to_INTERVAL((char *)"15183360") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-3789460209408") + (str_to_INTERVAL((char *)"-3336806495232") + (str_to_INTERVAL((char *)"-2353608871680") + (str_to_INTERVAL((char *)"-306912358400") + (str_to_INTERVAL((char *)"-38731224320") + (str_to_INTERVAL((char *)"-322604032") + str_to_INTERVAL((char *)"4154112") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-2821965453312") + (str_to_INTERVAL((char *)"-2529005973504") + (str_to_INTERVAL((char *)"-1613679436800") + (str_to_INTERVAL((char *)"-214334259200") + (str_to_INTERVAL((char *)"-10249768960") + (str_to_INTERVAL((char *)"73486336") - str_to_INTERVAL((char *)"2073600") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-1580640933888") + (str_to_INTERVAL((char *)"-1288439844864") + (str_to_INTERVAL((char *)"-647028096000") + (str_to_INTERVAL((char *)"-38900817920") + (str_to_INTERVAL((char *)"1608192000") + (str_to_INTERVAL((char *)"-46813184") + str_to_INTERVAL((char *)"82944") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-627374522368") + (str_to_INTERVAL((char *)"-384135061504") + (str_to_INTERVAL((char *)"-88082882560") + (str_to_INTERVAL((char *)"4025016320") + (str_to_INTERVAL((char *)"-1296916480") + str_to_INTERVAL((char *)"2015232") * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-152558764032") + (str_to_INTERVAL((char *)"-40800747520") + (str_to_INTERVAL((char *)"4849254400") + (str_to_INTERVAL((char *)"-4711055360") + str_to_INTERVAL((char *)"66846720") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-13781958656") + (str_to_INTERVAL((char *)"812384256") + (str_to_INTERVAL((char *)"-9745203200") + str_to_INTERVAL((char *)"269352960") * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-125304832") + (str_to_INTERVAL((char *)"-3926917120") + str_to_INTERVAL((char *)"629145600") * y * y) * y * y + (str_to_INTERVAL((char *)"-1166016512") + str_to_INTERVAL((char *)"295698432") * y * y + str_to_INTERVAL((char *)"100663296") * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * y * y;
	coef[26] = sqrt2xplus1 *  pow( (2 * x + 1),  2) * (-7560 + (str_to_INTERVAL((char *)"45360") + (str_to_INTERVAL((char *)"-113400") + (str_to_INTERVAL((char *)"151200") + (str_to_INTERVAL((char *)"-113400") + (str_to_INTERVAL((char *)"45360") - 7560 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"188055") + (str_to_INTERVAL((char *)"-557550") + (str_to_INTERVAL((char *)"250425") + (str_to_INTERVAL((char *)"812700") + (str_to_INTERVAL((char *)"-1185975") + (str_to_INTERVAL((char *)"591570") - str_to_INTERVAL((char *)"99225") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"4470165") + (str_to_INTERVAL((char *)"-14447790") + (str_to_INTERVAL((char *)"14844375") + (str_to_INTERVAL((char *)"-2034900") + (str_to_INTERVAL((char *)"-5536125") + (str_to_INTERVAL((char *)"3217410") - str_to_INTERVAL((char *)"513135") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"39344655") + (str_to_INTERVAL((char *)"-112159950") + (str_to_INTERVAL((char *)"101191125") + (str_to_INTERVAL((char *)"-16079700") + (str_to_INTERVAL((char *)"-20673975") + (str_to_INTERVAL((char *)"9554370") - str_to_INTERVAL((char *)"1176525") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"206243940") + (str_to_INTERVAL((char *)"-479519040") + (str_to_INTERVAL((char *)"305285400") + (str_to_INTERVAL((char *)"29576400") + (str_to_INTERVAL((char *)"-79352700") + (str_to_INTERVAL((char *)"18038160") - str_to_INTERVAL((char *)"272160") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"744360960") + (str_to_INTERVAL((char *)"-1306126080") + (str_to_INTERVAL((char *)"390448800") + (str_to_INTERVAL((char *)"360528000") + (str_to_INTERVAL((char *)"-217047600") + (str_to_INTERVAL((char *)"27609120") + str_to_INTERVAL((char *)"4762800") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"1924752480") + (str_to_INTERVAL((char *)"-2330065920") + (str_to_INTERVAL((char *)"-154408800") + (str_to_INTERVAL((char *)"984211200") + (str_to_INTERVAL((char *)"-378252000") + (str_to_INTERVAL((char *)"28183680") + str_to_INTERVAL((char *)"8255520") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"3353411040") + (str_to_INTERVAL((char *)"-2365238400") + (str_to_INTERVAL((char *)"-1058853600") + (str_to_INTERVAL((char *)"1225526400") + (str_to_INTERVAL((char *)"-452743200") + (str_to_INTERVAL((char *)"-6168960") + str_to_INTERVAL((char *)"6441120") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"2847508800") + (str_to_INTERVAL((char *)"-66168000") + (str_to_INTERVAL((char *)"-604219200") + (str_to_INTERVAL((char *)"372422400") + (str_to_INTERVAL((char *)"-98688000") + (str_to_INTERVAL((char *)"-36911040") + str_to_INTERVAL((char *)"3024000") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-2480171904") + (str_to_INTERVAL((char *)"2988854784") + (str_to_INTERVAL((char *)"2346787200") + (str_to_INTERVAL((char *)"-1068560640") + (str_to_INTERVAL((char *)"525085440") + (str_to_INTERVAL((char *)"-67493376") - str_to_INTERVAL((char *)"3919104") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-10957356480") + (str_to_INTERVAL((char *)"1879213632") + (str_to_INTERVAL((char *)"6454152000") + (str_to_INTERVAL((char *)"-2305501440") + (str_to_INTERVAL((char *)"-353416320") + (str_to_INTERVAL((char *)"-109632960") - str_to_INTERVAL((char *)"1683072") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-14287390720") + (str_to_INTERVAL((char *)"-2058112384") + (str_to_INTERVAL((char *)"9707098880") + (str_to_INTERVAL((char *)"-3118369280") + (str_to_INTERVAL((char *)"-1202508800") + (str_to_INTERVAL((char *)"-16699520") + str_to_INTERVAL((char *)"1281024") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-6874921472") + (str_to_INTERVAL((char *)"1908107520") + (str_to_INTERVAL((char *)"13073891840") + (str_to_INTERVAL((char *)"7919360") + (str_to_INTERVAL((char *)"615663360") + (str_to_INTERVAL((char *)"28598272") - str_to_INTERVAL((char *)"172800") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"5391255552") + (str_to_INTERVAL((char *)"13824910848") + (str_to_INTERVAL((char *)"18914816000") + (str_to_INTERVAL((char *)"5356500480") + (str_to_INTERVAL((char *)"855452160") + (str_to_INTERVAL((char *)"-3694592") + str_to_INTERVAL((char *)"41472") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"12135289856") + (str_to_INTERVAL((char *)"18733611008") + (str_to_INTERVAL((char *)"18976506880") + (str_to_INTERVAL((char *)"3177697280") + (str_to_INTERVAL((char *)"-105840640") + str_to_INTERVAL((char *)"1007616") * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"10371297280") + (str_to_INTERVAL((char *)"11917492224") + (str_to_INTERVAL((char *)"6871859200") + (str_to_INTERVAL((char *)"-359137280") + str_to_INTERVAL((char *)"33423360") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"4888657920") + (str_to_INTERVAL((char *)"2998075392") + (str_to_INTERVAL((char *)"-681574400") + str_to_INTERVAL((char *)"134676480") * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"963117056") + (str_to_INTERVAL((char *)"-246415360") + str_to_INTERVAL((char *)"314572800") * y * y) * y * y + (str_to_INTERVAL((char *)"-58720256") + str_to_INTERVAL((char *)"147849216") * y * y + str_to_INTERVAL((char *)"50331648") * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * y * y;
	coef[27] = -2 * x * (str_to_INTERVAL((char *)"364140") + (str_to_INTERVAL((char *)"-1350090") + (str_to_INTERVAL((char *)"1808100") + (str_to_INTERVAL((char *)"-1014300") + (str_to_INTERVAL((char *)"233100") + (str_to_INTERVAL((char *)"-90090") + str_to_INTERVAL((char *)"49140") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"7768320") + (str_to_INTERVAL((char *)"-26101740") + (str_to_INTERVAL((char *)"31340400") + (str_to_INTERVAL((char *)"-15552600") + (str_to_INTERVAL((char *)"3108000") + (str_to_INTERVAL((char *)"-1021020") + str_to_INTERVAL((char *)"458640") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"78281500") + (str_to_INTERVAL((char *)"-230997470") + (str_to_INTERVAL((char *)"236957500") + (str_to_INTERVAL((char *)"-96637300") + (str_to_INTERVAL((char *)"16393100") + (str_to_INTERVAL((char *)"-5405950") + str_to_INTERVAL((char *)"1408620") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"497227056") + (str_to_INTERVAL((char *)"-1240799996") + (str_to_INTERVAL((char *)"1012792640") + (str_to_INTERVAL((char *)"-291919320") + (str_to_INTERVAL((char *)"39908240") + (str_to_INTERVAL((char *)"-17959436") + str_to_INTERVAL((char *)"750816") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"2241945912") + (str_to_INTERVAL((char *)"-4509987860") + (str_to_INTERVAL((char *)"2615334360") + (str_to_INTERVAL((char *)"-354121080") + (str_to_INTERVAL((char *)"52871000") + (str_to_INTERVAL((char *)"-34752372") - str_to_INTERVAL((char *)"2973960") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"7681840768") + (str_to_INTERVAL((char *)"-11627166733") + (str_to_INTERVAL((char *)"3856796450") + (str_to_INTERVAL((char *)"7604890") + (str_to_INTERVAL((char *)"260968140") + (str_to_INTERVAL((char *)"-1935213") + str_to_INTERVAL((char *)"3835698") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"21055364604") + (str_to_INTERVAL((char *)"-21291124918") + (str_to_INTERVAL((char *)"2008012120") + (str_to_INTERVAL((char *)"-1714172460") + (str_to_INTERVAL((char *)"1630346340") + (str_to_INTERVAL((char *)"176939826") + str_to_INTERVAL((char *)"37977048") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"48492290696") + (str_to_INTERVAL((char *)"-25305836060") + (str_to_INTERVAL((char *)"-2198773240") + (str_to_INTERVAL((char *)"-13713271640") + (str_to_INTERVAL((char *)"4792614040") + (str_to_INTERVAL((char *)"387033604") + str_to_INTERVAL((char *)"67201560") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"98329881280") + (str_to_INTERVAL((char *)"-8307889480") + (str_to_INTERVAL((char *)"3010181200") + (str_to_INTERVAL((char *)"-42299363760") + (str_to_INTERVAL((char *)"6306425920") + (str_to_INTERVAL((char *)"168571352") + str_to_INTERVAL((char *)"39581808") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"180268079104") + (str_to_INTERVAL((char *)"46415119536") + (str_to_INTERVAL((char *)"38817421600") + (str_to_INTERVAL((char *)"-75235300000") + (str_to_INTERVAL((char *)"522563520") + (str_to_INTERVAL((char *)"-516546640") - str_to_INTERVAL((char *)"10237536") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"295688251520") + (str_to_INTERVAL((char *)"147573591968") + (str_to_INTERVAL((char *)"117650591040") + (str_to_INTERVAL((char *)"-82560616000") + (str_to_INTERVAL((char *)"-8352841600") + (str_to_INTERVAL((char *)"-809003744") - str_to_INTERVAL((char *)"16190784") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"417910392576") + (str_to_INTERVAL((char *)"280852008128") + (str_to_INTERVAL((char *)"220360822400") + (str_to_INTERVAL((char *)"-46978642560") + (str_to_INTERVAL((char *)"-7519321600") + (str_to_INTERVAL((char *)"-353287488") + str_to_INTERVAL((char *)"160896") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"488593757184") + (str_to_INTERVAL((char *)"391617664640") + (str_to_INTERVAL((char *)"294596181760") + (str_to_INTERVAL((char *)"8722446080") + (str_to_INTERVAL((char *)"1612641280") + (str_to_INTERVAL((char *)"51845248") + str_to_INTERVAL((char *)"1946880") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"457644914688") + (str_to_INTERVAL((char *)"405061002752") + (str_to_INTERVAL((char *)"287537648640") + (str_to_INTERVAL((char *)"38177269760") + (str_to_INTERVAL((char *)"4764641280") + (str_to_INTERVAL((char *)"43714048") - str_to_INTERVAL((char *)"304128") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"334599134208") + (str_to_INTERVAL((char *)"301418451968") + (str_to_INTERVAL((char *)"194714485760") + (str_to_INTERVAL((char *)"26248396800") + (str_to_INTERVAL((char *)"1371509760") + (str_to_INTERVAL((char *)"-6253568") + str_to_INTERVAL((char *)"82944") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"185160124416") + (str_to_INTERVAL((char *)"152337317888") + (str_to_INTERVAL((char *)"78637281280") + (str_to_INTERVAL((char *)"5247672320") + (str_to_INTERVAL((char *)"-162887680") + str_to_INTERVAL((char *)"2015232") * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"73342713856") + (str_to_INTERVAL((char *)"46141882368") + (str_to_INTERVAL((char *)"11833016320") + (str_to_INTERVAL((char *)"-502661120") + str_to_INTERVAL((char *)"66846720") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"18231558144") + (str_to_INTERVAL((char *)"5467668480") + (str_to_INTERVAL((char *)"-812318720") + str_to_INTERVAL((char *)"269352960") * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"1854930944") + (str_to_INTERVAL((char *)"-208666624") + str_to_INTERVAL((char *)"629145600") * y * y) * y * y + (str_to_INTERVAL((char *)"295698432") * y * y - str_to_INTERVAL((char *)"14680064") + str_to_INTERVAL((char *)"100663296") * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * y * y;
	coef[28] = sqrt2xplus1 *  pow( (2 * x + 1),  2) * (1080 + (-6480 + (str_to_INTERVAL((char *)"16200") + (str_to_INTERVAL((char *)"-21600") + (str_to_INTERVAL((char *)"16200") + (-6480 + 1080 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-26865") + (str_to_INTERVAL((char *)"79650") + (str_to_INTERVAL((char *)"-35775") + (str_to_INTERVAL((char *)"-116100") + (str_to_INTERVAL((char *)"169425") + (str_to_INTERVAL((char *)"-84510") + str_to_INTERVAL((char *)"14175") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-638595") + (str_to_INTERVAL((char *)"2063970") + (str_to_INTERVAL((char *)"-2120625") + (str_to_INTERVAL((char *)"290700") + (str_to_INTERVAL((char *)"790875") + (str_to_INTERVAL((char *)"-459630") + str_to_INTERVAL((char *)"73305") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-5620665") + (str_to_INTERVAL((char *)"16022850") + (str_to_INTERVAL((char *)"-14455875") + (str_to_INTERVAL((char *)"2297100") + (str_to_INTERVAL((char *)"2953425") + (str_to_INTERVAL((char *)"-1364910") + str_to_INTERVAL((char *)"168075") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-29463420") + (str_to_INTERVAL((char *)"68502720") + (str_to_INTERVAL((char *)"-43612200") + (str_to_INTERVAL((char *)"-4225200") + (str_to_INTERVAL((char *)"11336100") + (str_to_INTERVAL((char *)"-2576880") + str_to_INTERVAL((char *)"38880") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-106337280") + (str_to_INTERVAL((char *)"186589440") + (str_to_INTERVAL((char *)"-55778400") + (str_to_INTERVAL((char *)"-51504000") + (str_to_INTERVAL((char *)"31006800") + (str_to_INTERVAL((char *)"-3944160") - str_to_INTERVAL((char *)"680400") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-270658080") + (str_to_INTERVAL((char *)"333792000") + (str_to_INTERVAL((char *)"24753600") + (str_to_INTERVAL((char *)"-163406400") + (str_to_INTERVAL((char *)"66824400") + (str_to_INTERVAL((char *)"-2436000") - str_to_INTERVAL((char *)"680400") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-423073440") + (str_to_INTERVAL((char *)"348071040") + (str_to_INTERVAL((char *)"175521600") + (str_to_INTERVAL((char *)"-334708800") + (str_to_INTERVAL((char *)"128619600") + (str_to_INTERVAL((char *)"5652000") - str_to_INTERVAL((char *)"421200") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-87206720") + (str_to_INTERVAL((char *)"85349440") + (str_to_INTERVAL((char *)"214516000") + (str_to_INTERVAL((char *)"-503396800") + (str_to_INTERVAL((char *)"80483600") + (str_to_INTERVAL((char *)"3340640") - str_to_INTERVAL((char *)"775440") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"1411052416") + (str_to_INTERVAL((char *)"-29902336") + (str_to_INTERVAL((char *)"136294080") + (str_to_INTERVAL((char *)"-519836800") + (str_to_INTERVAL((char *)"-160403360") + (str_to_INTERVAL((char *)"519744") + str_to_INTERVAL((char *)"485856") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"3817963200") + (str_to_INTERVAL((char *)"1019961152") + (str_to_INTERVAL((char *)"241222880") + (str_to_INTERVAL((char *)"-174346560") + (str_to_INTERVAL((char *)"-60111440") + (str_to_INTERVAL((char *)"15143200") + str_to_INTERVAL((char *)"284688") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"5306427392") + (str_to_INTERVAL((char *)"2824718464") + (str_to_INTERVAL((char *)"550053120") + (str_to_INTERVAL((char *)"516395520") + (str_to_INTERVAL((char *)"225793600") + (str_to_INTERVAL((char *)"3630848") - str_to_INTERVAL((char *)"169344") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"4290511360") + (str_to_INTERVAL((char *)"2742025472") + (str_to_INTERVAL((char *)"290883200") + (str_to_INTERVAL((char *)"406973440") + (str_to_INTERVAL((char *)"-48613120") + (str_to_INTERVAL((char *)"-3956480") + str_to_INTERVAL((char *)"10368") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"1585883136") + (str_to_INTERVAL((char *)"173350400") + (str_to_INTERVAL((char *)"-1310758400") + (str_to_INTERVAL((char *)"-616942080") + (str_to_INTERVAL((char *)"-124280320") + str_to_INTERVAL((char *)"251904") * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-589063168") + (str_to_INTERVAL((char *)"-1829169152") + (str_to_INTERVAL((char *)"-2413388800") + (str_to_INTERVAL((char *)"-479191040") + str_to_INTERVAL((char *)"8355840") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-1148211200") + (str_to_INTERVAL((char *)"-1599225856") + (str_to_INTERVAL((char *)"-1068154880") + str_to_INTERVAL((char *)"33669120") * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-675250176") + (str_to_INTERVAL((char *)"-477233152") + str_to_INTERVAL((char *)"78643200") * y * y) * y * y + (str_to_INTERVAL((char *)"36962304") * y * y - str_to_INTERVAL((char *)"154664960") + str_to_INTERVAL((char *)"12582912") * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * y * y;
	coef[29] = (-1)*x *  pow( (2 * x + 1),  6) * (str_to_INTERVAL((char *)"-132570") + (str_to_INTERVAL((char *)"484920") + (str_to_INTERVAL((char *)"-638550") + (str_to_INTERVAL((char *)"356400") + (str_to_INTERVAL((char *)"-98550") + (str_to_INTERVAL((char *)"52920") - str_to_INTERVAL((char *)"24570") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-1237320") + (str_to_INTERVAL((char *)"3556080") + (str_to_INTERVAL((char *)"-3405600") + (str_to_INTERVAL((char *)"1188000") + (str_to_INTERVAL((char *)"-131400") + (str_to_INTERVAL((char *)"-35280") + str_to_INTERVAL((char *)"65520") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-5727630") + (str_to_INTERVAL((char *)"11253720") + (str_to_INTERVAL((char *)"-4511850") + (str_to_INTERVAL((char *)"-1537200") + (str_to_INTERVAL((char *)"199950") + (str_to_INTERVAL((char *)"356280") - str_to_INTERVAL((char *)"33270") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-17445168") + (str_to_INTERVAL((char *)"20625168") + (str_to_INTERVAL((char *)"2840280") + (str_to_INTERVAL((char *)"-5228640") + (str_to_INTERVAL((char *)"70080") + (str_to_INTERVAL((char *)"-750192") - str_to_INTERVAL((char *)"111528") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-39482100") + (str_to_INTERVAL((char *)"19728624") + (str_to_INTERVAL((char *)"10696860") + (str_to_INTERVAL((char *)"17392320") + (str_to_INTERVAL((char *)"-10989660") + (str_to_INTERVAL((char *)"-615600") + str_to_INTERVAL((char *)"29556") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-76664625") + (str_to_INTERVAL((char *)"-4759368") + (str_to_INTERVAL((char *)"4331865") + (str_to_INTERVAL((char *)"47454960") + (str_to_INTERVAL((char *)"-3128415") + (str_to_INTERVAL((char *)"1199640") + str_to_INTERVAL((char *)"29943") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-139977036") + (str_to_INTERVAL((char *)"-53497608") + (str_to_INTERVAL((char *)"-42463560") + (str_to_INTERVAL((char *)"55767600") + (str_to_INTERVAL((char *)"18801300") + (str_to_INTERVAL((char *)"472056") - str_to_INTERVAL((char *)"11232") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-220010516") + (str_to_INTERVAL((char *)"-137004128") + (str_to_INTERVAL((char *)"-114627200") + (str_to_INTERVAL((char *)"30539360") + (str_to_INTERVAL((char *)"631340") + (str_to_INTERVAL((char *)"-265024") + 648 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-262745120") + (str_to_INTERVAL((char *)"-227728864") + (str_to_INTERVAL((char *)"-162851040") + (str_to_INTERVAL((char *)"-33599200") + (str_to_INTERVAL((char *)"-8529920") + str_to_INTERVAL((char *)"15744") * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-225625280") + (str_to_INTERVAL((char *)"-221713152") + (str_to_INTERVAL((char *)"-165408000") + (str_to_INTERVAL((char *)"-33318400") + str_to_INTERVAL((char *)"522240") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-135577600") + (str_to_INTERVAL((char *)"-122000896") + (str_to_INTERVAL((char *)"-75366400") + str_to_INTERVAL((char *)"2104320") * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-53788672") + (str_to_INTERVAL((char *)"-34267136") + str_to_INTERVAL((char *)"4915200") * y * y) * y * y + (str_to_INTERVAL((char *)"-11272192") + str_to_INTERVAL((char *)"2310144") * y * y + str_to_INTERVAL((char *)"786432") * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * y * y;
	coef[30] = sqrt2xplus1 *  pow( (2 * x + 1),  2) * (-72 + (432 + (-1080 + (1440 + (-1080 + (432 - 72 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (1791 + (-5310 + (2385 + (7740 + (str_to_INTERVAL((char *)"-11295") + (5634 - 945 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"42573") + (str_to_INTERVAL((char *)"-137598") + (str_to_INTERVAL((char *)"141375") + (str_to_INTERVAL((char *)"-19380") + (str_to_INTERVAL((char *)"-52725") + (str_to_INTERVAL((char *)"30642") - 4887 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"374711") + (str_to_INTERVAL((char *)"-1068190") + (str_to_INTERVAL((char *)"963725") + (str_to_INTERVAL((char *)"-153140") + (str_to_INTERVAL((char *)"-196895") + (str_to_INTERVAL((char *)"90994") - str_to_INTERVAL((char *)"11205") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"1964228") + (str_to_INTERVAL((char *)"-4566848") + (str_to_INTERVAL((char *)"2907480") + (str_to_INTERVAL((char *)"281680") + (str_to_INTERVAL((char *)"-755740") + (str_to_INTERVAL((char *)"171792") - 2592 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"7089152") + (str_to_INTERVAL((char *)"-12439296") + (str_to_INTERVAL((char *)"3718560") + (str_to_INTERVAL((char *)"3433600") + (str_to_INTERVAL((char *)"-2067120") + (str_to_INTERVAL((char *)"262944") + str_to_INTERVAL((char *)"45360") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"17186848") + (str_to_INTERVAL((char *)"-22462016") + (str_to_INTERVAL((char *)"-2200160") + (str_to_INTERVAL((char *)"15485760") + (str_to_INTERVAL((char *)"-6981120") + (str_to_INTERVAL((char *)"-173376") - str_to_INTERVAL((char *)"68544") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"17063584") + (str_to_INTERVAL((char *)"-25506112") + (str_to_INTERVAL((char *)"-16650720") + (str_to_INTERVAL((char *)"54457920") + (str_to_INTERVAL((char *)"-21205440") + (str_to_INTERVAL((char *)"-1384128") - str_to_INTERVAL((char *)"85824") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-57801856") + (str_to_INTERVAL((char *)"-22123520") + (str_to_INTERVAL((char *)"-40454720") + (str_to_INTERVAL((char *)"124162880") + (str_to_INTERVAL((char *)"-18273760") + (str_to_INTERVAL((char *)"224768") + str_to_INTERVAL((char *)"123360") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-304573952") + (str_to_INTERVAL((char *)"-80770048") + (str_to_INTERVAL((char *)"-105271680") + (str_to_INTERVAL((char *)"169794560") + (str_to_INTERVAL((char *)"28313920") + (str_to_INTERVAL((char *)"1957248") - str_to_INTERVAL((char *)"13248") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-703877376") + (str_to_INTERVAL((char *)"-331468288") + (str_to_INTERVAL((char *)"-253359040") + (str_to_INTERVAL((char *)"112422720") + (str_to_INTERVAL((char *)"26500960") + (str_to_INTERVAL((char *)"-816512") - str_to_INTERVAL((char *)"28512") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-1006768384") + (str_to_INTERVAL((char *)"-702164480") + (str_to_INTERVAL((char *)"-431827200") + (str_to_INTERVAL((char *)"-49495680") + (str_to_INTERVAL((char *)"-25859840") + (str_to_INTERVAL((char *)"-486784") + str_to_INTERVAL((char *)"10368") * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-950668288") + (str_to_INTERVAL((char *)"-795574272") + (str_to_INTERVAL((char *)"-461157120") + (str_to_INTERVAL((char *)"-110753280") + (str_to_INTERVAL((char *)"-5016320") + str_to_INTERVAL((char *)"251904") * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-583233536") + (str_to_INTERVAL((char *)"-453386240") + (str_to_INTERVAL((char *)"-202634240") + (str_to_INTERVAL((char *)"8499200") + str_to_INTERVAL((char *)"8355840") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"-197369856") + (str_to_INTERVAL((char *)"-59899904") + (str_to_INTERVAL((char *)"88678400") + str_to_INTERVAL((char *)"33669120") * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"3047424") + (str_to_INTERVAL((char *)"75956224") + str_to_INTERVAL((char *)"78643200") * y * y) * y * y + (str_to_INTERVAL((char *)"36962304") * y * y + str_to_INTERVAL((char *)"35913728") + str_to_INTERVAL((char *)"12582912") * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * y * y;
	coef[31] = -6 * x *  pow( (2 * x + 1),  6) * (2430 + (-8730 + (str_to_INTERVAL((char *)"11250") + (-6300 + (2250 + (-1530 + 630 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"22680") + (str_to_INTERVAL((char *)"-64020") + (str_to_INTERVAL((char *)"60000") + (str_to_INTERVAL((char *)"-21000") + (3000 + (1020 - 1680 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"105690") + (str_to_INTERVAL((char *)"-203750") + (str_to_INTERVAL((char *)"79750") + (str_to_INTERVAL((char *)"22300") + (3950 + (-9190 + 1250 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"325392") + (str_to_INTERVAL((char *)"-378212") + (str_to_INTERVAL((char *)"-49000") + (str_to_INTERVAL((char *)"82680") + (-1600 + (str_to_INTERVAL((char *)"19468") + 1272 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"755004") + (str_to_INTERVAL((char *)"-374060") + (str_to_INTERVAL((char *)"-195300") + (str_to_INTERVAL((char *)"-346920") + (str_to_INTERVAL((char *)"224420") + (2196 - 540 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"1492851") + (str_to_INTERVAL((char *)"46199") + (str_to_INTERVAL((char *)"-114925") + (str_to_INTERVAL((char *)"-714670") + (str_to_INTERVAL((char *)"-72975") + (str_to_INTERVAL((char *)"-17001") - 279 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"2605140") + (str_to_INTERVAL((char *)"962342") + (str_to_INTERVAL((char *)"718300") + (str_to_INTERVAL((char *)"-888420") + (str_to_INTERVAL((char *)"-227820") + (-4050 + 108 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"3703996") + (str_to_INTERVAL((char *)"2420640") + (str_to_INTERVAL((char *)"1765240") + (str_to_INTERVAL((char *)"-217360") + (str_to_INTERVAL((char *)"32300") + 2624 * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"3971104") + (str_to_INTERVAL((char *)"3489056") + (str_to_INTERVAL((char *)"2513760") + (str_to_INTERVAL((char *)"515840") + str_to_INTERVAL((char *)"87040") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"3097408") + (str_to_INTERVAL((char *)"2972160") + (str_to_INTERVAL((char *)"2136960") + str_to_INTERVAL((char *)"350720") * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"1713152") + (str_to_INTERVAL((char *)"1476608") + str_to_INTERVAL((char *)"819200") * y * y) * y * y + (str_to_INTERVAL((char *)"385024") * y * y + str_to_INTERVAL((char *)"634880") + str_to_INTERVAL((char *)"131072") * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * y * y;
	coef[32] = 16 *  pow( x,  6) * sqrt2xplus1 *  pow( (2 * x + 1),  2) * (6720 + (1680 + (4200 + (str_to_INTERVAL((char *)"-35700") + (str_to_INTERVAL((char *)"19425") + (2730 + 945 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"87360") + (str_to_INTERVAL((char *)"18480") + (str_to_INTERVAL((char *)"37800") + (str_to_INTERVAL((char *)"-249900") + (str_to_INTERVAL((char *)"97125") + (8190 + 945 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"498880") + (str_to_INTERVAL((char *)"130960") + (str_to_INTERVAL((char *)"201400") + (str_to_INTERVAL((char *)"-704500") + (str_to_INTERVAL((char *)"98075") + (-3730 - 585 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"1651328") + (str_to_INTERVAL((char *)"654912") + (str_to_INTERVAL((char *)"747280") + (str_to_INTERVAL((char *)"-1051280") + (str_to_INTERVAL((char *)"-139830") + (str_to_INTERVAL((char *)"-16348") - 162 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"3527232") + (str_to_INTERVAL((char *)"2077584") + (str_to_INTERVAL((char *)"1854280") + (str_to_INTERVAL((char *)"-783780") + (str_to_INTERVAL((char *)"-176295") + (-1702 + 81 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"5132032") + (str_to_INTERVAL((char *)"4048480") + (str_to_INTERVAL((char *)"3100160") + (str_to_INTERVAL((char *)"121240") + (str_to_INTERVAL((char *)"84820") + 1968 * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"5235200") + (str_to_INTERVAL((char *)"4835520") + (str_to_INTERVAL((char *)"3479840") + (str_to_INTERVAL((char *)"658880") + str_to_INTERVAL((char *)"65280") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"3778560") + (str_to_INTERVAL((char *)"3507840") + (str_to_INTERVAL((char *)"2307200") + str_to_INTERVAL((char *)"263040") * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"1893376") + (str_to_INTERVAL((char *)"1475584") + str_to_INTERVAL((char *)"614400") * y * y) * y * y + (str_to_INTERVAL((char *)"288768") * y * y + str_to_INTERVAL((char *)"610304") + str_to_INTERVAL((char *)"98304") * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * y * y;
	coef[33] = x *  pow( (2 * x + 1),  6) * (720 + (-2520 + (3150 + (-1800 + (900 + (-720 + 270 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (6720 + (str_to_INTERVAL((char *)"-18480") + (str_to_INTERVAL((char *)"16800") + (-6000 + (1200 + (480 - 720 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"31600") + (str_to_INTERVAL((char *)"-59240") + (str_to_INTERVAL((char *)"22450") + (3800 + (4700 + (-4000 + 690 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"98688") + (str_to_INTERVAL((char *)"-111728") + (str_to_INTERVAL((char *)"-13240") + (str_to_INTERVAL((char *)"18480") + (-640 + (8512 - 72 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"236448") + (str_to_INTERVAL((char *)"-114512") + (str_to_INTERVAL((char *)"-56460") + (str_to_INTERVAL((char *)"-119280") + (str_to_INTERVAL((char *)"79640") + (-4128 - 108 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"476616") + (-812 + (str_to_INTERVAL((char *)"-48605") + (str_to_INTERVAL((char *)"-138160") + (str_to_INTERVAL((char *)"-85670") + (-1796 + 27 * y * y) * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"778464") + (str_to_INTERVAL((char *)"276520") + (str_to_INTERVAL((char *)"186440") + (str_to_INTERVAL((char *)"-203400") + (-5240 + 656 * y * y) * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"938400") + (str_to_INTERVAL((char *)"661120") + (str_to_INTERVAL((char *)"358800") + (str_to_INTERVAL((char *)"96960") + str_to_INTERVAL((char *)"21760") * y * y) * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"796416") + (str_to_INTERVAL((char *)"707584") + (str_to_INTERVAL((char *)"513280") + str_to_INTERVAL((char *)"87680") * y * y) * y * y) * y * y + (str_to_INTERVAL((char *)"462336") + (str_to_INTERVAL((char *)"388096") + str_to_INTERVAL((char *)"204800") * y * y) * y * y + (str_to_INTERVAL((char *)"172032") + str_to_INTERVAL((char *)"96256") * y * y + str_to_INTERVAL((char *)"32768") * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * x) * y * y;
	
	return ;
}
